// Arquivo auxiliar para o traador de raios de Persistence of Vision
// Arquivo: Alambique.inc
// Vers: 3.5
// Descri: Um alambique.
// Data: 3-XI-2002
// Autor: Xos B. Carballeira Rabual

#local versionAlabmiq=version;
#version 3.5;

#macro CorpoDoAlambique(Transparente)
  #local Asa=sphere_sweep{
    cubic_spline 8
    <-.5,0, .6>, .03,
    <-.02,0, .4>, .03,
    < .1,0, .2>, .03,
    < .3,0, .2>, .03,
    < .3,0,-.2>, .03,
    < .1,0,-.2>, .03,
    <-.02,0,-.4>, .03,
    <-.5,0,-.6>, .03
    //tolerance .0001
  }
  difference{
    #if(Transparente) merge #else union #end {
      sphere{.45*y, .55} cylinder{.5*y, 1.05*y, .22} torus{.23, .03 translate 1.05*y}
      object{Asa rotate 45*z translate<.7071*.42,.7071*.42+.45,0>}
      object{Asa rotate 135*z translate<-.7071*.42,.7071*.42+.45,0>}
    }
    plane{y, 0}
    cylinder{.4*y, 1.0501*y, .2}
    difference{sphere{.45*y, .53} plane{y, .02}}
  }
#end

#macro CampaDoAlambique(Transparente)
  difference{
    #if(Transparente) merge #else union #end {
      sor{ 9,
        <.1999,-1>, <.1999,-.05>, <.1999,0>, <.22,.01>, <.3,.05>, <.35,.15>, <.25,.3>, <.15,.5>, <.1,.7>
        sturm
      }
      sphere_sweep{
        b_spline  10,
        .29*y, .22,
        .4*y, .18
        .445*y, .17,
        <0,.7,.13>, .1,
        <0,.9,.38>, .07
        <0,1,.75>, .06,
        <0,.8,1.1>, .055
        <0,.2,1.43>, .05
        <0,-.2,1.5>, .05,
        <0,-1,1.5>, .05
        //tolerance .0001
      }
    }
    sor{ 9,
      <.1799,-1>, <.1799,-.0501>, <.1799,0>, <.2,.01>, <.28,.05>, <.33,.15>, <.23,.3>, <.13,.5001>,
      <.08,.7>
      sturm
    }
    sphere_sweep{
      b_spline  10,
      .29*y, .2,
      .4*y, .16
      .445*y, .15,
      <0,.7,.13>, .08,
      <0,.9,.38>, .05
      <0,1,.75>, .04,
      <0,.8,1.1>, .035
      <0,.2,1.43>, .03
      <0,-.25,1.5>, .03,
      <0,-1,1.5>, .03
      //tolerance .0001
    }
  }
#end

#macro RefrigeradorDoAlambique(Transparente)
  difference{
    #if(Transparente) merge #else union #end {
      sphere_sweep{
        cubic_spline 38,
        <0,1,0>, .08
        <0,.78,0>, .07,
        <0,.75,0>, .05,
        #local Conta=1/6;
        #while(Conta<=5.5)
          <.4*sin(2*pi*Conta),.75-Conta*.65/5.5,.4*(1-cos(2*pi*Conta))>, .05
          #declare Conta=Conta+1/6;
        #end
        <0,.1,.9>, .05
        <0,.1,2>, .05
        //tolerance .0001
      }
      difference{cylinder{.4*z, <0,.77,.4>, .47} cylinder{<0,.02,.4>, <0,.7701,.4>, .4499}}
      torus{.46, .02 translate <0,.77,.4>}
    }
    sphere_sweep{
      cubic_spline 38,
      <0,1,0>, .06
      <0,.85,0>, .05,
      <0,.75,0>, .03,
      #local Conta=1/6;
      #while(Conta<=5.5)
        <.4*sin(2*pi*Conta),.75-Conta*.65/5.5,.4*(1-cos(2*pi*Conta))>, .03
        #declare Conta=Conta+1/6;
      #end
      <0,.1,1>, .03
      <0,.1,2>, .03
      //tolerance .0001
    }
  }
#end

#macro Alambique(Transparente, AngCorpo, AngRefrig)
  union{
    object{CorpoDoAlambique(Transparente) rotate AngCorpo*y}
    object{CampaDoAlambique(Transparente) translate 1.05*y}
    object{RefrigeradorDoAlambique(Transparente) rotate AngRefrig*y translate 1.49*z}
  }
#end

#version versionAlabmiq;
