// Arquivo auxiliar para o traador de raios de Persistence of Vision
// Arquivo: Chou.inc
// Vers: 3.5
// Descri: Ajuda a obter certos dados aleatrios.
// Data de cria: 12-VI-2003
// Data da ltima actualiza: _
// Autor: Xos B. Carballeira Rabual


#local versionChouChou=version;
#version 3.5;

#ifndef(Xose_Mensagens)
  #declare Xose_Mensagens=false;
#end

#if(Xose_Mensagens)
  #debug "    Esta-se a executar o arquivo \"Chou.inc\", que ajuda a obter certos dados aleatorios\n"
#end

// Ponto aleatrio na circunferncia de raio 1 centrada na origem, no plano UV; distribui
#macro ChouCircf(N)                                                           //linear uniforme
  #local Ang=2*pi*rand(N);
  <cos(Ang),sin(Ang)>
#end

// Ponto aleatrio no crculo de raio 1 centrado na origem, no plano UV; distribui superficial
#macro ChouCircl(N)                                                                      //uniforme
  ( sqrt(rand(N))*ChouCircf(N) )
#end

// Ponto aleatrio na superfcie esfrica de raio 1 centrada na origem; distribui superficial uniforme
#macro ChouFrB(N)
  #local Longitude=2*pi*rand(N);
  #local Altura=1-2*rand(N);
  #local Afastamento=sqrt(1-pow(Altura,2));
  <Afastamento*cos(Longitude),Afastamento*sin(Longitude),Altura>
#end

// Ponto aleatrio na bola de raio 1 centrada na origem; distribui volumnica uniforme
#macro ChouB(N)
  ( pow(rand(N),1/3)*ChouFrB(N) )
#end

// Ponto aleatrio na superfcie da esfera descrita; distribui superficial uniforme
#macro ChouFrEsf(Centro,Raio,N)
  ( Centro+Raio*ChouFrB(N) )
#end

// Ponto aleatrio dentro da esfera descrita; distribui volumnica uniforme
#macro ChouEsf(Centro,Raio,N)
  ( Centro+Raio*ChouB(N) )
#end

// Ponto aleatrio na diferena entre as esferas do centro e mais os raios dados; distribui
#macro ChouEsfOca(Centro,Raio,RaioOco,N)                                    //volumnica uniforme
  #local ParMin=pow(RaioOco,3);
  #local ParMax=pow(Raio,3);
  #local Par=ParMin+rand(N)*(ParMax-ParMin);
  ( Centro+pow(Par,1/3)*ChouFrB(N) )
#end

// Ponto aleatrio na circunferncia descrita no espao; distribui linear uniforme
#macro ChouFrDisco(Centro,Raio,Normal,N)
  #if(vlength(Normal)=0) #error "O vector normal do disco no pode ser nulo." #end
  #local SouAssentadorDeTijolos=vcross(x,Normal);
  #if(vlength(SouAssentadorDeTijolos)=0) #local SouAssentadorDeTijolos=vcross(y,Normal); #end
  #local EmbatiNoBidao=vcross(Normal,SouAssentadorDeTijolos);
  #local Ang=2*pi*rand(N);
  ( Centro+Raio*(cos(Ang)*vnormalize(SouAssentadorDeTijolos)+sin(Ang)*vnormalize(EmbatiNoBidao)) )
#end

// Ponto aleatrio no crculo descrito no espao; distribui superficial uniforme
#macro ChouDisco(Centro,Raio,Normal,N)
  ( Centro+sqrt(rand(N))*ChouFrDisco(<0,0,0>,Raio,Normal,N) )
#end

// Ponto aleatrio na coroa circular descrita no espao; distribui superficial uniforme
#macro ChouDiscoFurado(Centro,Raio,RaioFuro,Normal,N)
  #local ParMin=pow(RaioFuro,2);
  #local ParMax=pow(Raio,2);
  #local Par=ParMin+rand(N)*(ParMax-ParMin);
  ( Centro+sqrt(Par)*ChouFrDisco(<0,0,0>,1,Normal,N) )
#end

// Ponto aleatrio na superfcie cilndrica de raio dado, limitada polas bases de centros dados; as bases
//n se incluem na superfcie; distribui superficial uniforme
#macro ChouFrCil(Cnt1,Cnt2,Raio,N)
  #if(vlength(Cnt2-Cnt1)=0)
    #error "O cilindro deve ter altura no nula; para un disco, empregue-se ChouFrDisco(Centro,Raio,Normal,N)."
  #end
  ( Cnt1+rand(N)*(Cnt2-Cnt1)+ChouFrDisco(<0,0,0>,Raio,Cnt2-Cnt1,N) )
#end

// Ponto aleatrio dentro do cilindro de raio dado, limitado polas bases de centros dados; distribui
#macro ChouCil(Cnt1,Cnt2,Raio,N)                                             //volumnica uniforme
  #if(vlength(Cnt2-Cnt1)=0)
    #error "O cilindro deve ter altura no nula; para un disco, empregue-se ChouDisco(Centro,Raio,Normal,N)."
  #end
  ( Cnt1+rand(N)*(Cnt2-Cnt1)+ChouDisco(<0,0,0>,Raio,Cnt2-Cnt1,N) )
#end

// Ponto aleatrio dentro do cilindro furado de raios dados, limitado polas bases de centros dados;
#macro ChouCilFurado(Cnt1,Cnt2,Raio,RaioFuro,N)                     //distribui volumnica uniforme
  #if(vlength(Cnt2-Cnt1)=0)
    #error "O cilindro deve ter altura no nula; para un disco, empregue-se ChouDiscoFurado(Centro,Raio,RaioFuro,Normal,N)."
  #end
  ( Cnt1+rand(N)*(Cnt2-Cnt1)+ChouDiscoFurado(<0,0,0>,Raio,RaioFuro,Cnt2-Cnt1,N) )
#end

// Ponto aleatrio na superfcie cnica de raios dados, limitada polas bases de centros dados; as bases
//n se incluem na superfcie; distribui superficial uniforme
#macro ChouFrCono(Cnt1,Raio1,Cnt2,Raio2,N)
  #if(vlength(Cnt2-Cnt1)=0)
    #error "O cono deve ter altura no nula; para un disco, empregue-se ChouFrDisco(Centro,Raio,Normal,N)."
  #end
  #local Par=rand(N);
  ( Cnt1+Par*(Cnt2-Cnt1)+ChouFrDisco(<0,0,0>,Raio1+Par*(Raio2-Raio1),Cnt2-Cnt1,N) )
#end

// Ponto aleatrio dentro do cono de raios dados, limitado polas bases de centros dados; distribui
#macro ChouCono(Cnt1,Raio1,Cnt2,Raio2,N)                                         //volumnica uniforme
  #if(vlength(Cnt2-Cnt1)=0)
    #error "O cono deve ter altura no nula; para un disco, empregue-se ChouDisco(Centro,Raio,Normal,N)."
  #end
  #local Par=rand(N);
  ( Cnt1+Par*(Cnt2-Cnt1)+ChouDisco(<0,0,0>,Raio1+Par*(Raio2-Raio1),Cnt2-Cnt1,N) )
#end

// Ponto aleatrio dentro do cono furado de raios dados, limitado polas bases de centros dados;
#macro ChouConoFurado(Cnt1,Raio1,RaioFuro1,Cnt2,Raio2,RaioFuro2,N)  //distribui volumnica uniforme
  #if(vlength(Cnt2-Cnt1)=0)
    #error "O cono deve ter altura no nula; para un disco, empregue-se ChouDiscoFurado(Centro,Raio,RaioFuro,Normal,N)."
  #end
  #local Par=rand(N);
  ( Cnt1+Par*(Cnt2-Cnt1) +
    ChouDiscoFurado(<0,0,0>,Raio1+Par*(Raio2-Raio1),RaioFuro1+Par*(RaioFuro2-RaioFuro1),Cnt2-Cnt1,N)
  )
#end

// Ponto aleatrio dentro dum objecto qualquer; distribui volumnica uniforme
#macro ChouObjecto(Objecto,N,NoTentativas)
  #local Min=min_extent(Objecto);
  #local Max=max_extent(Objecto);
  #local Conta=0;
  #local Exito=false;
  #while((Conta<NoTentativas)&!Exito)
    #local Ponto=Min+<rand(N),rand(N),rand(N)>*(Max-Min);
    #declare Exito=inside(Objecto,Ponto);
    #declare Conta=Conta+1;
  #end
  #if(!Exito)
    #warning concat("No se deu encontrado um ponto dentro do objecto nas ",str(NoTentativas,0,0)," tentativas.")
  #end
  Ponto
#end


#version versionChouChou;