// Arquivo auxiliar para o traador de raios de Persistence of Vision
// Arquivo: Corpos lineares.inc
// Vers: 3.5
// Descri: Cordas, fios, cabos, arames...
// Data de cria: 4-VI-2003
// Data da ltima actualiza: _
// Autor: Xos B. Carballeira Rabual


#local versionCablemania=version;
#version 3.5;

#ifndef(Xose_Mensagens)
  #declare Xose_Mensagens=false;
#end

#if(Xose_Mensagens)
  #debug "    Esta-se a executar o arquivo \"Corpos lineares.inc\", que define cordas, fios, arames, cabos...\n"
#end

// Spline ensarilhada arredor dumha linha central percorrida cumha macro
//CorpLin_LinhaCentral(Parametro), segundo outra macro CorpLin_Ensarilhamento(Parametro), que h-de de-
//volver um vector bidimensional.
// Comeco e Fim s os valores do parmetro entre os que se definir a spline; Passo  o chimpo do parme-
//tro entre cada dous pontos guardados na defini da spline. Interpolaco determina o tipo de inter-
//pola usado nessa spline: 1=linear, 2=quadrtica, 3=cbica, 4=natural.
#macro LinhaEnsarilhada(Comeco, Fim, Passo, Interpolaco)
  #local Passo=select(Fim-Comeco,-1,1)*abs(Passo);
  #local NoPtos=ceil((Fim-Comeco)/Passo);
  #local ContaPtos=0;
  #local Tangente=x;
  #local Binormal=z;
  spline{
    #switch(Interpolaco)
      #case(2)
        quadratic_spline
      #break
      #case(3)
        cubic_spline
        #declare ContaPtos=-1;
        #declare NoPtos=NoPtos+1;
      #break
      #case(4)
        natural_spline
      #break
      #else
        linear_spline
    #end
    #while(ContaPtos<NoPtos)
      #local Param=Comeco+ContaPtos*Passo;
      #if(vlength(CorpLin_LinhaCentral(Param+Passo/2)-CorpLin_LinhaCentral(Param-Passo/2))!=0)
        #declare Tangente =
          vnormalize(CorpLin_LinhaCentral(Param+Passo/2)-CorpLin_LinhaCentral(Param-Passo/2));
      #end
      #local Guaix=vcross( CorpLin_LinhaCentral(Param+Passo/2)-CorpLin_LinhaCentral(Param) ,
        CorpLin_LinhaCentral(Param)-CorpLin_LinhaCentral(Param-Passo/2) );
      #if(vlength(Guaix)!=0)
        #declare Binormal=vnormalize(Guaix);
      #end
      #local Normal=vcross(Binormal,Tangente);
      Param,
      CorpLin_LinhaCentral(Param)+CorpLin_Ensarilhamento(Param).u*Normal +
        CorpLin_Ensarilhamento(Param).v*Binormal,
      #declare ContaPtos=ContaPtos+1;
    #end
  }
#end


//============================================== FIOS ==============================================================

// Fio de esferas soltas
// Supe-se definida umha macro CorpLin_LinhaCentral(Parametro) que percorra, segundo Parametro, a linha
//central do fio. Comeco e Fim s os valores do parmetro com os que corresponderm os extremos do fio;
//Passo, o chimpo do parmetro entre cada duas esferinhas; Grosso, o dimetro destas.
#macro FioE(Comeco, Fim, Passo, Grosso)
  #local Passo=select(Fim-Comeco,-1,1)*abs(Passo);
  #local NoEsfs=ceil((Fim-Comeco)/Passo);
  #local ContaEsfs=0;
  union{
    #while(ContaEsfs<NoEsfs)
      sphere{CorpLin_LinhaCentral(Comeco+ContaEsfs*Passo), Grosso/2}
      #declare ContaEsfs=ContaEsfs+1;
    #end
  }
#end

// Fio de cilindros
// Supe-se definida umha macro CorpLin_LinhaCentral(Parametro) que percorra, segundo Parametro, a linha
//central do fio. Comeco e Fim s os valores do parmetro com os que corresponderm os extremos do fio; Passo,
//a varia do parmetro entre as duas bases de cada cilindro; Grosso, o dimetro destes. EmpalmeEsf 
//booleano, e decide empalmar ou n os cilindros com esferas.
#macro FioC(Comeco, Fim, Passo, Grosso, EmpalmeEsf)
  #local Passo=select(Fim-Comeco,-1,1)*abs(Passo);
  #local NoCils=ceil((Fim-Comeco)/Passo)-1;
  #local ContaCils=0;
  union{
    #while(ContaCils<NoCils)
      #local Actual=CorpLin_LinhaCentral(Comeco+ContaCils*Passo);
      #local Seguinte=CorpLin_LinhaCentral(Comeco+(ContaCils+1)*Passo);
      #if(vlength(Seguinte-Actual)!=0)
        #if(EmpalmeEsf) sphere{Actual, Grosso/2} #end
        cylinder{Actual, Seguinte, Grosso/2}
      #end
      #declare ContaCils=ContaCils+1;
    #end
    #if(EmpalmeEsf) sphere{CorpLin_LinhaCentral(Comeco+NoCils*Passo), Grosso/2} #end
  }
#end

// Fio de anacos de toro
// Supe-se definida umha macro CorpLin_LinhaCentral(Parametro) que percorra, segundo Parametro, a linha
//central do fio. Comeco e Fim s os valores do parmetro com os que corresponderm os extremos do fio; Passo,
//a varia do parmetro entre os extremos de cada anaco de toro; Grosso, o dimetro secundario destes.
//EmpalmeEsf  booleano, e decide empalmar ou n os segmentos com esferas.
/*#macro FioAT(Comeco, Fim, Passo, Grosso, EmpalmeEsf)
  #local Passo=select(Fim-Comeco,-1,1)*abs(Passo);
  #local NoAT=ceil((Fim-Comeco)/Passo)-1;
  #local ContaAT=0;
  union{
    ...
    Hai-no que fazer.
    #while(ContaAT<NoAT)
      #if(EmpalmeEsf) ... #end
      difference{
        torus{..., Grosso/2}
      }
      #declare ContaAT=ContaAT+1;
    #end
    #if(EmpalmeEsf) ... #end
  }
#end*/

// Fio de varrido de esfera
// Supe-se definida umha macro CorpLin_LinhaCentral(Parametro) que percorra, segundo Parametro, a linha
//central do fio. Comeco e Fim s os valores do parmetro com os que corresponderm os extremos daquele; Passo,
//o chimpo do parmetro entre cada duas posies empregadas ao definir o sphere sweep; Grosso, o grossor
//do fio. Interpolaco determina o tipo de interpola usado nesse varrido de esfera: 1 = linear, 2 =
//beta, 3 = cbica. Tolerancia  a tolerncia do varrido de esfera.
#macro FioVE(Comeco, Fim, Passo, Grosso, Interpolaco, Tolerancia)
  #local Passo=select(Fim-Comeco,-1,1)*abs(Passo);
  #local NoPtos=ceil((Fim-Comeco)/Passo);
  sphere_sweep {
    #switch(Interpolaco)
      #case(2)
        b_spline
      #break
      #case(3)
        cubic_spline
      #break
      #else
        linear_spline
    #end
    #if((Interpolaco=2)|(Interpolaco=3))
      #local ContaPtos=-1;
      #declare NoPtos=NoPtos+1;
      NoPtos+1
    #else
      #local ContaPtos=0;
      NoPtos
    #end
    #while(ContaPtos<NoPtos)
      , CorpLin_LinhaCentral(Comeco+ContaPtos*Passo), Grosso/2
      #declare ContaPtos=ContaPtos+1;
    #end
    tolerance Tolerancia
  }
#end

//============================================== CORDAS ==============================================================

// Trana
// Supe-se definida umha macro CorpLin_LinhaCentral(Parametro) que percorra, segundo Parametro, a linha
//central da trana. Comeco e Fim s os valores do parmetro com os que corresponderm os extremos desta;
//Passo, o chimpo do parmetro entre cada duas esferinhas dos fios. Modalidade escolhe o jeito de construir
//a trana: 0 = com esferinhas, 1 = com cilindros sem empalme, 2 = com cilindros empalmados mediante esfe-
//ras, 5 = com um varrido de esfera de interpola linear, 6 = id. beta, 7 = id. cbica; nestas trs lti-
//mas, Tolerancia estabelece a tolerncia do varrido. InterpolacoEnsarilh escolhe o tipo de interpola
//que se empregar nos ensarilhamentos: 1=linear, 2=quadrtica, 3=cbica, 4=natural. Grosso  o dos
//fios. Frequencia regula a escala longitudinal das voltas dos fios; com FreqFactor varia-se a freqncia
//dos nveis pequenos a respeito dos grandes. Nivel  o ndice de recorrncia dos tranados: com 1, obtem-se
//umha trana simples; com 2, umha trana de tranas, e assim sucessivamente.
#macro Tranca(Comeco, Fim, Passo, Modalidade, Tolerancia, InterpolacoEnsarilh, Grosso, Frequencia,
  FreqFactor, Nivel)
  #local NivelInf=floor(Nivel)-1;
  #ifndef(CorpLin_Trajectorias)
    #declare CorpLin_Trajectorias=array[NivelInf+1][3]
  #else
  #end
  #macro CorpLin_Ensarilhamento(Parametro)
    pow(3,NivelInf)*Grosso*<cos(Frequencia*Parametro),sin(2*Frequencia*Parametro)/2>
  #end
  #declare CorpLin_Trajectorias[NivelInf][0]=LinhaEnsarilhada(Comeco,Fim,Passo,InterpolacoEnsarilh)
  #macro CorpLin_Ensarilhamento(Parametro)
    pow(3,NivelInf)*Grosso*<cos(Frequencia*Parametro+2*pi/3),sin(2*Frequencia*Parametro+4*pi/3)/2>
  #end
  #declare CorpLin_Trajectorias[NivelInf][1]=LinhaEnsarilhada(Comeco,Fim,Passo,InterpolacoEnsarilh)
  #macro CorpLin_Ensarilhamento(Parametro)
    pow(3,NivelInf)*Grosso*<cos(Frequencia*Parametro+4*pi/3),sin(2*Frequencia*Parametro+2*pi/3)/2>
  #end
  #declare CorpLin_Trajectorias[NivelInf][2]=LinhaEnsarilhada(Comeco,Fim,Passo,InterpolacoEnsarilh)
  #local ContaTres=0;
  union{
    #if(NivelInf<1)
      #while(ContaTres<3)
        #macro CorpLin_LinhaCentral(Parametro)
          CorpLin_Trajectorias[0][ContaTres](Parametro)
        #end
        #switch(Modalidade)
          #case(0)
            FioE(Comeco,Fim,Passo,Grosso)
          #break
          #case(1)
            FioC(Comeco, Fim, Passo, Grosso, false)
          #break
          #case(2)
            FioC(Comeco, Fim, Passo, Grosso, true)
          #break
          #case(3)
            //FioAT(Comeco, Fim, Passo, Grosso, false)
          #break
          #case(4)
            //FioAT(Comeco, Fim, Passo, Grosso, true)
          #break
          #case(5)
            FioVE(Comeco, Fim, Passo, Grosso, 1, Tolerancia)
          #break
          #case(6)
            FioVE(Comeco, Fim, Passo, Grosso, 2, Tolerancia)
          #break
          #case(7)
            FioVE(Comeco, Fim, Passo, Grosso, 3, Tolerancia)
          #break
        #end
        #declare ContaTres=ContaTres+1;
      #end
    #else
      #while(ContaTres<3)
        #macro CorpLin_LinhaCentral(Parametro)
          CorpLin_Trajectorias[Nivel][ContaTres](Parametro)
        #end
        TrancaE(Comeco, Fim, Passo, Modalidade, Tolerancia, InterpolacoEnsarilh, Grosso,
          FreqFactor*3*Frequencia, FreqFactor, NivelInf)
        #declare ContaTres=ContaTres+1;
      #end
    #end
  }
#end

//J continuarei quando puider...

#version versionCablemania;