// Arquivo principal de cena para o traador de raios de Persistence of Vision
// Arquivo: Disco ou buxaina.pov
// Vers: 3.6
// Descri: Um disco ou uma buxaina a rolar sobre o cho
// Data de cria: 26-VIII-2004
// Data da ltima actualiza: 2-IX-2004
// Autor: Xos B. Carballeira Rabual


// Para 4 de simula:
//+w200 +h200 +a0.3 -j +kff9600
//+w240 +h240 +a0.3 -j +kff9600


#version 3.6;

//============================================= OPES ===================================================

#declare Freq=1e4;

#declare MassaCorpo=1;
#declare MassaPontas=0;
#declare Raio=1;
#declare ComprmtPontas=1;
#declare GrossRelCorpo=1/20;
#declare GrossRelPontas=1/40;

#declare AltInicial=2;
#declare InclInicial=pi/2+.25;
#declare VelInicial=.5*x;
#declare RotProprInicial=-100;
#declare PrecessInicial=0;
#declare NutInicial=0;

#declare Gravidade=-9.8*y;

#declare RigChao=1e3;
#declare AmortChaoV=1;

#declare AmortChaoHCorpo=0;
#declare CoefAtritoCorpo=.5;
#declare CoefRodaduraCorpo=.1;
#declare CoefPivotamentoCorpo=.1;

#declare AmortChaoHPonta=0;
#declare CoefAtritoPonta=.5;
#declare CoefRodaduraPonta=.1;
#declare CoefPivotamentoPonta=.1;

#declare AltRelCamara=2.2;
#declare AfastCamara=8;
#declare DesvHCamara=0;
#declare MaxAtrasoCamara=.5;
#declare MaxAtrasoMotivo=.2;

#declare TempoTotal=30;

#declare Disperso=on;
#declare LuzExtensa=on;
#declare Fotons=on;


//============================================== CENA ====================================================

global_settings{
  assumed_gamma 1
  ambient_light 1.6
  max_trace_level 20
  #if(Fotons)
    photons{
      spacing max(Raio,ComprmtPontas)*.0005                 // specify the density of photons
      //count 100000               // alternatively use a total number of photons
  
      //gather min, max            // amount of photons gathered during render [20, 100]
      //media max_steps [,factor]  // media photons
      //jitter 1.0                 // jitter phor photon rays
      //max_trace_level 5          // optional separate max_trace_level
      //adc_bailout 1/255          // see global adc_bailout
      //save_file "filename"       // save photons to file
      //load_file "filename"       // load photons from file
      //autostop 0                 // photon autostop option
      //radius 10                  // manually specified search radius
      // (---Adaptive Search Radius---)
      //steps 1
      //expand_thresholds 0.2, 40
    }
  #end
}

#if(mod(frame_number,2)=1)
  #declare Tempo=TempoTotal*clock;
#else
  #declare Tempo=TempoTotal*(clock-clock_delta);
#end

#if(ComprmtPontas=0)
  #declare Inzinhas=array[1][1][7]
#else
  #declare Inzinhas=array[1][4][7]{{{0,0,0,0,0,0,0}{0,0,0,0,0,0,0}{0,0,0,0,0,0,0}{0,0,0,0,0,0,0}}}
#end

#include "metals.inc"
#include "Matematica.inc"
#include "Mecanica.inc"

InerciaEsfera(MassaCorpo, Raio, Inzinhas, 0, 0)
InerciaEscala(Inzinhas, 0, 0, <1,1,GrossRelCorpo>)
InerciaRot(Inzinhas, 0, 0, transform{rotate -degrees(InclInicial)*x})

#declare CentroInicial=Raio*<0,cos(InclInicial),-sin(InclInicial)>+AltInicial*y;
#if(ComprmtPontas=0)
  #declare Nos=array[1]{1}
  #declare Pouxis=array[1][1]{{CentroInicial}}
#else
  InerciaEsfera(MassaPontas, 1, Inzinhas, 0, 3)
  InerciaEscala(Inzinhas, 0, 3, (Raio*GrossRelPontas*<1,1,0>+ComprmtPontas*z))
  InerciaRot(Inzinhas, 0, 3, transform{rotate -degrees(InclInicial)*x})
  #declare Nos=array[1]{4}
  #declare Pouxis=array[1][4]{
    {
      CentroInicial,
      CentroInicial-ComprmtPontas*<0,sin(InclInicial),cos(InclInicial)>,
      CentroInicial+ComprmtPontas*<0,sin(InclInicial),cos(InclInicial)>,
      CentroInicial
    }
  }
#end
#declare NormInic=<0,-sin(InclInicial),-cos(InclInicial)>;
#declare Veixis=array[1]{VelInicial}
#declare Vangis=array[1]{RotProprInicial*NormInic+PrecessInicial*y+NutInicial*vcross(y,NormInic)}

#macro MecXos_Forcas(CorpoActual, ElementoActual, Resultante, Momento)
  #switch(CorpoActual)
    #case(0)
      #local Rotada=function{transform{MecXos_Rotacoes[CorpoActual]}}
      #local Normal=Rotada(0,-sin(InclInicial),-cos(InclInicial));
      #local Centro=MecXos_CMs[CorpoActual];
      #switch(ElementoActual)
        #case(0)  //Corpo
          #if(vlength(vcross(Normal,y))=0)
            #local RaioVector=<0,0,0>;
          #else
            #local RaioVector=Raio*vnormalize(vcross(Normal,vcross(Normal,y)));
          #end
          #local Penetraco=-min(0,Centro.y+RaioVector.y);
          #if(Penetraco>0)
            #local Desliz=MecXos_VelocidadesCM[CorpoActual] +
              vcross(MecXos_VelocidadesAngulares[CorpoActual],RaioVector)
            ;
          #else
            #local Desliz=<0,0,0>;
          #end
          #local Atravess=Desliz.y;
          #local Desliz=Desliz-Desliz.y*y;
          #if(vlength(Desliz)=0)
            #local DirDesliz=<0,0,0>;
          #else
            #local DirDesliz=-vnormalize(Desliz);
          #end
          #local Reacco=RigChao*Penetraco*(y+CoefAtritoCorpo*DirDesliz)-AmortChaoHCorpo*Desliz-AmortChaoV*Atravess*y;
          #declare Resultante=Inzinhas[CorpoActual][ElementoActual][0]*Gravidade+Reacco;
          #local Pivotamento=MecXos_VelocidadesAngulares[CorpoActual].y;
          #if(vlength(RaioVector)=0)
            #if(Pivotamento=0)
              #declare Momento=<0,0,0>;
            #else
              #declare Momento=-( 2/3*CoefAtritoCorpo*Raio*RigChao*Penetraco
                #if(Penetraco>0) + AmortChaoHCorpo*Pivotamento*pow(Raio,2)/2 #end
                ) * Pivotamento/abs(Pivotamento)*y
              ;
            #end
          #else
            #declare Momento=vcross(RaioVector,Reacco);
          #end
          #if(Pivotamento!=0)
            #declare Momento=Momento-CoefPivotamentoCorpo*RigChao*Penetraco*Pivotamento/abs(Pivotamento)*y;
          #end
          #local Rodadura=MecXos_VelocidadesAngulares[CorpoActual]-Pivotamento*y;
          #if(vlength(Rodadura)!=0)
            #declare Momento=Momento-CoefRodaduraCorpo*RigChao*Penetraco*vnormalize(Rodadura);
          #end
        #break
        #range(1,2)  //Extremos das pontas
          #local RaioVector=select(Normal.y,1,-1)*ComprmtPontas*Normal;
          #local Penetraco=-min(0,Centro.y+RaioVector.y);
          #if(Penetraco>0)
            #local Desliz=MecXos_VelocidadesCM[CorpoActual] +
              vcross(MecXos_VelocidadesAngulares[CorpoActual],RaioVector)
            ;
          #else
            #local Desliz=<0,0,0>;
          #end
          #local Atravess=Desliz.y;
          #local Desliz=Desliz-Desliz.y*y;
          #if(vlength(Desliz)=0)
            #local DirDesliz=<0,0,0>;
          #else
            #local DirDesliz=-vnormalize(Desliz);
          #end
          #declare Resultante=RigChao*Penetraco*(y+CoefAtritoPonta*DirDesliz)-AmortChaoHPonta*Desliz-AmortChaoV*Atravess*y;
          #local Pivotamento=MecXos_VelocidadesAngulares[CorpoActual].y;
          #declare Momento=vcross(RaioVector,Resultante);
          #if(Pivotamento!=0)
            #declare Momento=Momento-CoefPivotamentoPonta*RigChao*Penetraco*Pivotamento/abs(Pivotamento)*y;
          #end
          #local Rodadura=MecXos_VelocidadesAngulares[CorpoActual]-Pivotamento*y;
          #if(vlength(Rodadura)!=0)
            #declare Momento=Momento-CoefRodaduraPonta*RigChao*Penetraco*vnormalize(Rodadura);
          #end
        #break
        #case(3)  //Pontas
          #declare Resultante=Inzinhas[CorpoActual][ElementoActual][0]*Gravidade;
        #break
      #end
      #undef Rotada
    #break
  #end
#end

MovSolRigCampoDeForcas("Kukuriku", Nos, Freq, Pouxis, Inzinhas, Veixis, Vangis, Tempo, 0)

light_source{
  1e6*<-20,30,-15> rgb 1.6 parallel point_at 0
  #if(LuzExtensa)
    area_light 2*1e6*x, 2*1e6*y, 4, 4  circular orient
    adaptive 0
  #end
  #if(Fotons)
    photons{
      refraction on
      reflection on
      #if(LuzExtensa) area_light #end
    }
  #end
}




plane{
  y, 0
  texture{
    checker
      texture{
        pigment{rgb<1,.3,0>} normal{agate agate_turb 1}
        finish{diffuse .5 specular .3 roughness .01 irid{.2 thickness 5 turbulence 1}}
      }
      texture{
        pigment{rgb<0,.8,.7>} normal{wrinkles scale .3}
        finish{diffuse .5 specular .6 roughness .002 reflection{.1 .5}}
      }
  }
}

#if(ComprmtPontas!=0)
  union{
#end
  sphere{
    0, Raio scale<1,1,GrossRelCorpo>
    pigment{
      spiral1 6 color_map{
        [0 rgbf<.1,.5,1,1>]
        [.5 rgbf<1,.2,.1,1>]
        [1 rgbf<.3,1,.2,1>]
      }
    }
    finish{diffuse .5 specular .2 roughness .003 reflection{0 .5 fresnel} conserve_energy}
    interior{
      ior 1.5
      #if(Disperso) dispersion 1.012 #end
      #if(!Fotons) caustics .8 #end
    }
    #if(Fotons)
      photons{target 1 refraction on reflection on}
    #end
#if(ComprmtPontas!=0)
  }
  sphere{
    0, 1 scale(Raio*GrossRelPontas*<1,1,0>+ComprmtPontas*z)
    pigment{rgb<0.65,0.50,0.25>}
    finish{
      ambient .1
      brilliance 3
      diffuse .4
      metallic
      specular .7
      roughness 1/60
      reflection .25
    }
    #if(Fotons)
      photons{target 1 refraction off reflection on}
    #end
  }
#end
  translate Raio*y rotate -degrees(InclInicial)*x translate AltInicial*y
  transform MecXos_Movimentos[0]
}

fog{fog_type 1 distance 2*AfastCamara color rgb<0,.001,.02>}


// - - - - - - - - - - - - - - - - - - - - Provas - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

//sphere{0, Raio/10 pigment{rgb<0,1,0>} translate (Raio*<0,cos(InclInicial),-sin(InclInicial)>+MecXos_Despracamentos[0])}
//torus{Raio, Raio/16 rotate (90-degrees(InclInicial))*x transform{MecXos_Rotacoes[0]} translate (Raio*<0,cos(InclInicial),-sin(InclInicial)>+MecXos_Despracamentos[0]) pigment{rgb<1,0,1>}}


//---------------------------------- Defini da cmara ---------------------------------------------------

#declare Estereoscopia_Olho=1-2*mod(frame_number,2);
#declare Estereoscopia_DistOlhos=.06;

#declare Estereoscopia_TipoDeCamara=0;   //Projec cnica (perspectiva), a predeterminada
//#declare Estereoscopia_TipoDeCamara=1;   //Projec ortogonal
//#declare Estereoscopia_TipoDeCamara=2;   //Olho de peixe (esfrica polar)
//#declare Estereoscopia_TipoDeCamara=3;   //Gram angular
//#declare Estereoscopia_TipoDeCamara=4;   //Omnimax
//#declare Estereoscopia_TipoDeCamara=5;   //Panormica
//#declare Estereoscopia_TipoDeCamara=6;   //Cilindro vertical, ponto de vista fixo
//#declare Estereoscopia_TipoDeCamara=7;   //Cilindro horizontal, ponto de vista fixo
//#declare Estereoscopia_TipoDeCamara=8;   //Cilindro vertical, ponto de vista mvel
//#declare Estereoscopia_TipoDeCamara=9;   //Cilindro horizontal, ponto de vista mvel
//#declare Estereoscopia_TipoDeCamara=10;  //Esfrica cartesiana

#declare Roucheixom=function{transform{MecXos_Rotacoes[0]}}
#declare Normy=Roucheixom(0,-sin(InclInicial),-cos(InclInicial));
#undef Roucheixom
#if(Normy.y>0) #declare Normy=-Normy; #end
#declare Sencho=MecXos_CMs[0];
#if(vlength(vcross(Normy,y))=0)
  #declare Ralhus=ComprmtPontas*Normy;
#else
  #declare Ralhus=Raio*vnormalize(vcross(Normy,vcross(Normy,y)));
  #if(Ralhus.y>ComprmtPontas*Normy.y)
    #declare Ralhus=ComprmtPontas*Normy;
  #end
#end

#declare ParamCam=MaxAtrasoCamara/.468914153;
#declare ParamMot=MaxAtrasoMotivo/.468914153;

#declare Estereoscopia_UbiCamara=
  Sencho+Ralhus.y*y +
  < DesvHCamara ,
    AltRelCamara-ParamCam/(ParamCam/(Sencho.y+Ralhus.y)+sinh((Sencho.y+Ralhus.y)/ParamCam)) ,
    -AfastCamara
  >
;
#declare Estereoscopia_Largo=x;
#declare Estereoscopia_Alto=y;
#declare Estereoscopia_Focal=3*z;
#declare Estereoscopia_CeuCamara=y;
//#declare Estereoscopia_AngCamara=25;
//#declare Estereoscopia_AngCamara2=<180,90>;
#declare Estereoscopia_OlhaCara=Sencho-ParamMot/(ParamMot/Sencho.y+sinh(Sencho.y/ParamMot))*y;

//#declare Estereoscopia_TransformaCamara=transform{translate 0}  //Transforma que se impor  cmara
                                                              //sobre todas as opes anteriores

#include "Estereoscopia.inc"