// Arquivo auxiliar para o traador de raios de Persistence of Vision
// Arquivo: Figuras geometricas.inc
// Vers: 3.5
// Descri: Figuras geomtricas
// Data de cria: 4-VI-2003; procede do velho Obxectos de Xose.inc, criado o 17-IV-2002.
// Data da ltima actualiza: _
// Autor: Xos B. Carballeira Rabual

#local versionFigosNhamQueRicos=version;
#version 3.5;

#ifndef(Xose_Mensagens)
  #declare Xose_Mensagens=false;
#end

#if(Xose_Mensagens)
  #debug "    Esta-se a executar o arquivo \"Figuras geometricas.inc\", que define objectos geometricos.\n"
#end

#ifndef(Xose_SemTexturas)
  #declare Xose_SemTexturas=false;
#end

// Serpentino construdo com anacos de toro
#macro SerpentinoAT(Raio,Passo,RaioDoTubo,Comeco,Fim,Preciso)
  union{
    #if (Passo*(Comeco-Fim)>0)
      #local Signo=-1;
    #else
      #local Signo=1;
    #end
    #local Parametro=Comeco+Signo*Passo/(2*Preciso);
    #while((Comeco-Fim)*(Parametro-Fim)>=0)
      intersection{
        torus{Raio+pow(Passo/(2*pi),2)/Raio, RaioDoTubo
          rotate<degrees(atan2(Passo,2*pi*Raio)),-90+360*Parametro/Passo,0>
          translate<-pow(Passo/(2*pi),2)*sin(2*pi*Parametro/Passo)/Raio,Parametro,pow(Passo/(2*pi),2)*cos(2*pi*Parametro/Passo)/Raio>
        }
        plane{-z,0
          rotate<degrees(atan2(Passo,2*pi*Raio)),-90+360*(Parametro+Passo/(1.99*Preciso))/Passo,0>
        }
        plane{z,0
          rotate<degrees(atan2(Passo,2*pi*Raio)),-90+360*(Parametro-Passo/(1.99*Preciso))/Passo,0>
        }
      }
      #declare Parametro=Parametro+Signo*Passo/Preciso;
    #end
  }
#end

// Sphere sweep em forma de serpentino. Convm usar a op Remove_Bounds=off ou -UR.
#macro Serpentino(Raio,Passo,RaioDoTubo,Comeco,Fim,Preciso,Tolerancia)
  #local Numaro=ceil(Preciso*abs(Fim-Comeco)/Passo);
  sphere_sweep{
    cubic_spline
    Numaro+3
    #local ALasPantasDelPipano=-1;
    #while(ALasPantasDelPipano<=Numaro+1)
      , < Raio*cos(2*pi*ALasPantasDelPipano/Preciso), Comeco+ALasPantasDelPipano*(Fim-Comeco)/Numaro,
          Raio*sin(2*pi*ALasPantasDelPipano/Preciso) >, RaioDoTubo
      #declare ALasPantasDelPipano=ALasPantasDelPipano+1;
    #end
    tolerance Tolerancia
    bounded_by{
      cylinder{
        #if(Comeco<=Fim)
          (Comeco-RaioDoTubo*1.001)*y, (Fim+RaioDoTubo*1.001)*y
        #else
          (Comeco+RaioDoTubo*1.001)*y, (Fim-RaioDoTubo*1.001)*y
        #end
        , Raio+RaioDoTubo*1.001
      }
    }
  }
#end


#version versionFigosNhamQueRicos;