// Arquivo principal de cena para o traador de raios de Persistence of Vision
// Arquivo: Fundo estrelecido.pov
// Vers: 3.6
// Descri: Um fundo estrelecido para apresentaes tais como o meu perfil no Lycos
// Data de cria: 22-XI-2005
// Data da ltima actualiza: _
// Autor: Xos Benito Carballeira Rabual

// +w500 +h2500 +a0.3 +kff5 +kc

#version 3.6;

#include "Chou.inc"
#include "Matematica.inc"

global_settings{
  assumed_gamma 1
}

camera{
  orthographic
  location -5*z
  right x
  up 5*y
  direction 3*z
  look_at 0
}

#declare Afu=seed(3);  //  preciso que nengum sol preto quadre nos
#macro CampoDeEstrelas(Cor)               // extremos superior ou inferior da imagem.
  wrinkles scale .03 color_map{
    [.93 rgbt 1]
    [.93 Cor]
  }
  translate 100*<rand(Afu),rand(Afu),rand(Afu)>
#end
plane{-z, -100
  pigment{
    average pigment_map{
      [1 CampoDeEstrelas(rgb 4.1)]
      [1 CampoDeEstrelas(rgb 4.1*<1,0,0>)]
      [1 CampoDeEstrelas(rgb 4.1*<1,1,0>)]
      [1 CampoDeEstrelas(rgb 4.1*<0,0,1>)]
      #declare Marmore=function{pattern{marble}}
      [.1 function{Marmore(x-.3*pow(sin(pi/5*y),2),sin(2*pi/5*y),cos(2*pi/5*y))}
        translate .5*x color_map{
          [.95 rgbt 1]
          [.98 rgbt<.5,.5,.5,.5>]
          [1 rgb<1,.8,.5>]
        }
        warp{turbulence .3 octaves 7 lambda 5 omega .9}
      ]
    }
  }
  finish{diffuse 0 ambient 1}
}

// Sis pretos

#declare RaioDaCoroa=.04;
#declare RaioDaFotosfera=.02;
#declare NoFuros=72;
#declare FracFuro=.1;
#declare QuantosSois=30;

#macro SolPreto()
  union{
    sphere{ 0, RaioDaCoroa hollow
      pigment{rgbt 1}
      interior{
        media{
          scattering{1, 1} method 1 intervals 8+sqrt(NoFuros) samples 1,1
          density{
            spherical color_map{
              [0 rgb 0]
              [(1-RaioDaFotosfera/RaioDaCoroa)/16 rgb .01]
              [(1-RaioDaFotosfera/RaioDaCoroa)/8 rgb .2]
              [(1-RaioDaFotosfera/RaioDaCoroa)/4 rgb 1]
            }
            scale RaioDaCoroa
          }
        }
      }
    }
    #local Coleur=<rand(Afu),rand(Afu),rand(Afu)>;
    sphere{ 0, 1.001*RaioDaFotosfera no_shadow
      pigment{rgbt Coleur+.3*t} finish{ambient .02}
    }
    sphere{ 0, RaioDaFotosfera //hollow
      pigment{rgb Coleur} finish{ambient 0}
      clipped_by{
        union{
          #local Indice=0;
          #while(Indice<NoFuros)
            sphere{RaioDaFotosfera*ChouFrB(Afu), sqrt(FracFuro*4*RaioDaFotosfera*RaioDaFotosfera/NoFuros)}
            #declare Indice=Indice+1;
          #end
          inverse
        }
      }
    }
    light_source{
      0, rgb 5*<rand(Afu),rand(Afu),rand(Afu)> fade_power 2 fade_distance RaioDaFotosfera
      media_attenuation on media_interaction on
    }
    RotVector(2*pi*int(5*(2*rand(Afu)-1))*clock*ChouFrB(Afu))
  }
#end

#declare Rols=0;
#while(Rols<QuantosSois)
  object{SolPreto() translate<rand(Afu)-1/2,5*(rand(Afu)-1/2),0>}
  #declare Rols=Rols+1;
#end