// Arquivo principal de cena para o traador de raios de Persistence of Vision
// Arquivo: Graficas.pov
// Vers: 3.6
// Descri: Representa funcins graficamente.
// Data de cria: 20-XII-2002
// Data da ltima actualizacin: 3-VI-2003
// Autor: Xos B. Carballeira Rabual


#version 3.6;


//============================================== OPES ==========================================================

#declare TipoDeGraf=1;  //Cartesiana explcita: Y em fun de X
//#declare TipoDeGraf=2;  //Cartesiana explcita: X em fun de Y
//#declare TipoDeGraf=3;  //Cartesiana paramtrica
//#declare TipoDeGraf=4;  //Polar explcita: R em fun de Phi
//#declare TipoDeGraf=5;  //Polar explcita: Phi em fun de R
//#declare TipoDeGraf=6;  //Polar paramtrica

//#declare MinIndep=-5;  //Podem-se omitir nas grficas cartesianas explcitas.
//#declare MaxIndep=5;  //

#declare Exactitude=10;  //A curva representada passar por Exactitude + 1 pontos da grfica verdadeira.

#declare TipoDeInterpolaco=1;  //Linear
//#declare TipoDeInterpolaco=2;  //Beta
//#declare TipoDeInterpolaco=3;  //Cbica

#declare ToleranciaVarrido=1e-8;  //1e-6  o usual.

#declare MinX=0;
#declare MaxX=100000;
#declare MinY=0;
#declare MaxY=5000;

#declare CorDaGraf=rgb<.8,.1,.05>;
#declare GrosGraf=.005;
#declare RelevoGraf=1;
#declare CorDoFundo=rgb<.01,0,.05>;
#declare Eixos=true;
#declare CorDosEixos=rgb<.4,.5,.45>;
#declare GrosEixos=.005;
#declare RelevoEixos=.5;
#declare LongFlechas=.05;
#declare GrosFlechas=.01;
#declare RelevoFlechas=.8;

#declare RacoDeAspectoDosPixeles=1;


//---------------------------------------------- Fun -----------------------------------------------------

#include "Matematica.inc"

#macro Funco(X)
  #local Ista=NumerosPrimosAte(X,10+100*log(1+X),5+20*log(1+X))
  #local Ras=dimension_size(Ista,1);
  Ras
#end


/*A macro que define a fun representada deve ter umha das formas seguintes, segundo o tipo de grfica:

//Cartesiana explcita: Y em fun de X
#macro Funco(X)
  //H-de devolver um valor numrico.
#end

//Cartesiana explcita: X em fun de Y
#macro Funco(Y)
  //H-de devolver um valor numrico.
#end

//Cartesiana paramtrica
#macro Funco(Param)
  //H-de assignar valores s variveis X e Y.
#end

//Polar explcita: R em fun de Phi
#macro Funco(Phi)
  //H-de devolver um valor numrico.
#end

//Polar explcita: Phi em fun de R
#macro Funco(R)
  //H-de devolver um valor numrico.
#end

//Polar paramtrica
#macro Funco(Param)
  //H-de assignar valores s variveis R e Phi.
#end*/


//=============================================== CENA =========================================================

global_settings{
  assumed_gamma 1
}

#ifndef(Eixos)
  #declare Eixos=false;
#end

#if(Eixos)
  #declare AltMax=max(RelevoEixos, RelevoFlechas, RelevoGraf);
#else
  #declare AltMax=RelevoGraf;
#end

camera{
  orthographic
  location <(MinX+MaxX)/2, (MinY+MaxY)/2, -5-AltMax>
  right (MaxX-MinX)*x
  up (MaxY-MinY)*y
  direction z
}

background{CorDoFundo}

light_source{-(10+AltMax)*z rgb 1 parallel point_at 0}

//Eixos
#if(Eixos)
  #declare RaioCono=GrosFlechas*(MaxX-MinX);
  union{
    cylinder{MinX*x, (MaxX-.999*LongFlechas*(MaxX-MinX))*x,
      GrosEixos*(MaxY-MinY)*RacoDeAspectoDosPixeles*image_width/image_height
      scale<1, 1, RelevoEixos/(GrosEixos*(MaxY-MinY)*RacoDeAspectoDosPixeles*image_width/image_height)>
    }
    cylinder{MinY*y, ( MaxY -
        .999*LongFlechas*(MaxY-MinY)*RacoDeAspectoDosPixeles*image_width/image_height )*y,
      GrosEixos*(MaxX-MinX)
      scale<1, 1, RelevoEixos/(GrosEixos*(MaxX-MinX))>
    }
    cone{(MaxX-LongFlechas*(MaxX-MinX))*x, GrosFlechas*(MaxY-MinY)*RacoDeAspectoDosPixeles *
        image_width/image_height,
      MaxX*x, 0
      scale<1, 1,
        RelevoFlechas/(GrosFlechas*(MaxY-MinY)*RacoDeAspectoDosPixeles*image_width/image_height)>
    }
    cone{(MaxY-LongFlechas*(MaxY-MinY)*RacoDeAspectoDosPixeles*image_width/image_height)*y,
      GrosFlechas*(MaxX-MinX), MaxY*y, 0
      scale<1, 1, RelevoFlechas/(GrosFlechas*(MaxX-MinX))>
    }
    sphere{0, GrosFlechas*(MaxX-MinX)
      scale<1, (MaxY-MinY)/(MaxX-MinX)*RacoDeAspectoDosPixeles*image_width/image_height,
        RelevoFlechas/(GrosFlechas*(MaxX-MinX))>
    }
    pigment{CorDosEixos}
  }
#end


//-------------------------------------------- Grfica ------------------------------------------------------------

#ifndef(TipoDeGraf)
  #declare TipoDeGraf=1;
#end

#if(TipoDeGraf=1)
  #ifndef(MinIndep)
    #declare MinIndep=MinX;
  #end
  #ifndef(MaxIndep)
    #declare MaxIndep=MaxX;
  #end
#end

#if(TipoDeGraf=2)
  #ifndef(MinIndep)
    #declare MinIndep=MinY;
  #end
  #ifndef(MaxIndep)
    #declare MaxIndep=MaxY;
  #end
#end

#declare Passo=(MaxIndep-MinIndep)/Exactitude;

#ifndef(TipoDeInterpolaco)
  #declare TipoDeInterpolaco=1;
#end

#if(TipoDeInterpolaco=1)
  #declare IndepInic=MinIndep;
  #declare IndepFin=MaxIndep;
#else
  #declare IndepInic=MinIndep-Passo;
  #declare IndepFin=MaxIndep+Passo;
#end

#declare RaioEsfGraf=GrosGraf*(MaxX-MinX);
#declare EscalaVert=(MaxY-MinY)/(MaxX-MinX)*RacoDeAspectoDosPixeles*image_width/image_height;

sphere_sweep{
  #switch(TipoDeInterpolaco)
  #case(1)
    linear_spline Exactitude+1
  #break
  #case(2)
    b_spline Exactitude+3
  #break
  #case(3)
    cubic_spline Exactitude+3
  #end
  #declare Indep=IndepInic;
  #while(Indep<=IndepFin)
    #switch(TipoDeGraf)
    #case(1)
      , <Indep, Funco(Indep), 0>
    #break
    #case(2)
      , <Funco(Indep), Indep, 0>
    #break
    #case(3)
      Funco(Indep)
      , <X,Y,0>
    #break
    #case(4)
      #declare R=Funco(Indep);
      , <R*cos(Indep), R*sin(Indep), 0>
    #break
    #case(5)
      #declare Phi=Funco(Indep);
      , <Indep*cos(Phi), Indep*sin(Phi), 0>
    #break
    #case(6)
      Funco(Indep)
      , <R*cos(Phi), R*sin(Phi), 0>
    #end
      / <1,EscalaVert,1>,
    RaioEsfGraf
    #declare Indep=Indep+Passo;
  #end
  tolerance ToleranciaVarrido
  scale<1, EscalaVert, RelevoGraf/RaioEsfGraf>
  pigment{CorDaGraf}
}