// Arquivo auxiliar para o traador de raios de Persistence of Vision
// Arquivo: Lamina elastica.inc
// Vers: 3.6
// Descri: Simula ondas elsticas bidimensionais de pequena intensidade.
// Data de cria: 15-I-2003
// Data da ltima actualiza: 10-VIII-2004
// Autor: Xos B. Carballeira Rabual


#local versionLamElast=version;
#version 3.6;

//Requer:
//#include "Matematica.inc"

#ifndef(Xose_Mensagens)
  #declare Xose_Mensagens=false;
#end

#if(Xose_Mensagens)
  #debug "    Esta-se a executar o arquivo \"Lamina elastica.inc\", para simular as ondas duma lamina elastica.\n"
#end

#ifndef(Xose_PastaTemp)
  #declare Xose_PastaTemp=""
#end


//  A seguinte macro cria um pigmento LamElast_Pigmento e mais uma fun LamElast_Funco(x,y,z) que
//atribuem (mediante a luminosidade em gris ou com o valor devolto, respectivamente) a cada ponto duma lmina
//elstica rectangular a sua elonga em cada instante, calculada aproximadamente por mtodos numricos.
//A lmina supe-se com um vrtice na origem e com dous lados paralelos aos eixos OX e OZ na parte positiva des-
//tes.
//  Os parmetros s:
//    Nome . O nome do arquivo temporal no que se armazenarm os dados das ondas elsticas.
//    DimsDominio . As dimenses da lmina, em forma de vector de duas componentes: <largo, longo>
//    DensGrelha . Densidade da grelha quadrada que ham de formar os pontos da lmina nos que se calcular
//                   a elonga; igual ao inverso da distncia entre duas filas ou colunas adjacentes.
//                   Nos outros pontos, interpolar-se- bilinearmente. De momento, nenguma das dimenses
//                   da grelha igual ao produto da dimens do domnio que corresponder por DensGrelha
//                   pode ser maior que 256 (mximo nmero de entradas num mapa de cores do POV-Ray), mas
//                   talvez arranje isto algum dia.
//    FreqCalc . Nmero de instantes que se considerarm para o clculo em cada unidade de tempo.
//    Rapidez . Rapidez de propaga das ondas elsticas.
//    Amortecimento . Coeficiente de amortecimento, igual  decelera que o tal efeito provoca sobre
//                  cada ponto dividida pola velocidade deste. O coeficiente de absor (mngua relativa
//                  da intensidade das ondas com o caminho percorrido) relaciona-se com o de amortecimento,
//                  no caso das ondas senoidais de freqncia FreqOnda, do seguinte jeito:

//                                  ___________________________________________________________________
//                                 /                 ________________________________
//                          1     /              (  /    2         2                2                 )
//             Absor =  V 4 pi FreqOnda ( V 4 pi  FreqOnda  + Amortecimento  - 2 pi FreqOnda )
//                       Rapidez

//    RigResorte . Rigidez dum resorte fictcio que segura cada ponto da lmina  origem da sua elonga.
//    QuantosGrausDeLiberdade . Nmero de dimenses nas quais poder oscilar cada ponto da lmina: 1, 2,
//                  3, 4 ou 5.
//    Tempo . Tempo do instante da chamada actual  macro.
//    Tempo0 . Tempo do instante no que se chamou a macro por primeira vez.

//  Cumpre que DensGrelha>=4/LongOnda, e que FreqCalc/DensGrelha>Rapidez, sendo LongOnda a mnima longitu-
//de previsvel para as ondas que se ham-de simular; isto implica, alis, que FreqCalc>4FreqOnda, onde
//chamamos FreqOnda a freqncia mxima que caiba esperar das ondas. Noutro caso, podem surgir erros consi-
//derveis. Devemos reparar em que se se violar a segunda inequa, a lentid do modelo impedir s
//frontes de onda avanarem ao seu passo normal, j que segundo o mtodo empregado, os cmbios que acontecem
//num tempo 1/FreqCalc em cada ponto da grelha s levam na conta os quatro pontos vizinhos, polo que nengu-
//ma perturba se propagar com celeridade maior ca FreqCalc/DensGrelha, isto , as ondas n podem
//percorrer, entre dous clculos, mis distncia que a que separa dous pontos consecutivos da grelha. Se isto
// incompatvel com o valor do parmetro Rapidez, as perturbaes podem-se acumular em determinadas zonas,
//ao n se dar propagado com eficcia, ocasionando que terminem por disparar-se caoticamente os valores abso-
//lutos das elongaes em todo o domnio.
//  Antes de usar LaminaElastica, h que definir a perturba que sofrer cada ponto em cada instante
//com uma macro LamElast_Perturbaco(<PosicoX, PosicoZ>, Tempo), que devolver a acelera adicio-
//nal, forada, do ponto.
//  Para que as elongaes e mais as velocidades iniciais dos pontos da lmina n sejam todas nulas, po-
//dem-se definir:
//    1) As matrizes bidimensionais LamElast_Elongacoes0 e LamElast_Velocidades0, que devem conter as
//        elongaes iniciais e as velocidades iniciais, respectivamente, dos pontos de controlo da lmi-
//        na. Estes repartem-se numa trama rectangular, segundo os parmetros DimsDominio e DensGrelha. Os
//        dados de cada um situar-se-am no elemento de ndices [no_ordem_largo][no_ordem_longo], sendo o pri-
//        meiro o nmero de ordem do ponto na grelha no senso positivo do eixo OX, e o segundo, no do OZ.
//        Ambos os dous comeam em 0.
//    2) Alternativamente a cada uma das matrizes anteriores, as macros LamElast_MacroElongaco0(<x,z>)
//        e LamElast_MacroVelocidade0(<x,z>), que ham de devolver, respectivamente, a elonga e mais a
//        velocidade iniciais do ponto (x,0,z) da lmina. Cada uma destas macros s ser efectiva se
//        n se define a matriz correspondente, pois esta ltima  prioritria ante um conflito.
//  Devido  curiosa feitura dos pigmentos, precisa-se estabelecer uns valores mximo e mnimo para as
//elongaes; fora desse intervalo, truncar-se-am ao extremo mais cercano. As variveis LamElast_MaxElong e
//LamElast_MinElong permitem ajustar estas quantidades; os seus valores predeterminados s 1 e -1, respec-
//tivamente.

//  Podem-se impor condies de contorno s ondas. Para isso cumpre definir a matriz unidimensional
//LamElast_Contorno, com quatro elementos, referentes a cadanseu lado do domnio, segundo os ndices:
//    0) lado x = 0; 1) lado x = DimsDominio.x; 2) lado z = 0; 3) lado z = DimsDominio.y
//  Cada um destes elementos h-de ser um vector com duas componentes booleanas, que indiquem se o lado em
//quest do domnio tem condi de elonga (a primeira componente) ou de pendente normal (a segunda). O
//primeiro caso  o duma verdadeira lmina elstica sujeita polos bordos, nos quais as ondas s inteiramen-
//te reflectidas com oposi de fase; o segundo serve para simular a superfcie livre dum lquido.
//  As condies devem-se estabelecer definindo as macros LamElast_ElongContorno(<PosicoX, PosicoZ>, Tempo)
//e LamElast_PendContorno(<PosicoX, PosicoZ>, Tempo).

//  Tambm se pode forar a integral de superfcie das elongaes sobre o domnio para um valor dado em ca-
//da instante, definindo a macro LamElast_Integral(Tempo); isto ajuda a simular enchimentos e esvazia-
//mentos de depsitos de auga, assim como a manter constante o volume de lquido numa poa.

//  A forma necessariamente rectangular do domnio limita muito as possibilidades, mas espero melhorar
//isto algum dia.

//  Cuido que o aumento indefinido da quantidade de memria ocupada polo POV-Ray enquanto gera uma anima-
// como as Ondinhas se deve a algum erro no programa, que atinge o armazenamento das funes defini-
//das polo usurio; a memria destinada a estas n semelha liberar-se eficazmente entre quadro e quadro.
//Talvez seja boa ideia, logo, destruir sempre manualmente todas as funes empregadas na cena, uma vez
//cumpridos os seus objectivos; para isto engadi, ao remate da defini da macro LaminaElastica, a or-
//dem #undef UnaCabura, desactivada. A prxima vez que se utilizar esta macro, conviria buscar alguma
//melhora com a referida instru activada, e assemade com a linha #undef LamElast_Funco nalgum lugar do
//arquivo principal da cena, posterior ao derradeiro uso de LamElast_Funco; naturalmente, se ese arqui-
//vo ou algum outro que se executar definirem mais funes, cumprir elimin-las todas com #undef para a
//prova ser efectiva.


#macro LaminaElastica(Nome, DimsDominio, DensGrelha, FreqCalc, Rapidez, Amortecimento, RigResorte,
  QuantosGrausDeLiberdade, Tempo, Tempo0)

  #local NoFilas=ceil(DimsDominio.x*DensGrelha)+1;
  #local NoCols=ceil(DimsDominio.y*DensGrelha)+1;

  #if(strlen(Xose_PastaTemp)=0)
    #local NomeDoFilego=concat(Nome, ".xose");
    #warning "Ao no se indicar, mediante a variavel Xose_PastaTemp, uma pasta para armazenar os dados\n  temporais (arquivos \".xose\"), empregar-se-a a actual."
    #warning "V. \"Das variaveis xosianas.txt\""
    #warning "Existe o risco de que esta cambie durante o processo e se perda informaco."
    #warning "Isto pode ocasionar erros na simulaco das ondas."
    #warning "Desaconselha-se, de prosseguir, toda acco que poida mudar a pasta actual, seja ou no com o POV-Ray,\n  ate se completarem todos os quadros."
  #else
    #local NomeDoFilego=concat(Xose_PastaTemp, "\\", Nome, ".xose")
  #end

  #ifndef(LamElast_Contorno) #declare LamElast_Contorno=array[4]{<0,0> <0,0> <0,0> <0,0>} #end
  #if(NoFilas=1)  //Se houver apenas uma fila, n se poderm impor condies de pendente normal
    #declare LamElast_Contorno[0]=LamElast_Contorno[0]*<1,0>;                //nos lados x=cte
    #declare LamElast_Contorno[1]=LamElast_Contorno[1]*<1,0>;
  #end
  #if(NoCols=1)  //Se houver apenas uma coluna, n se poderm impor condies de pendente normal
    #declare LamElast_Contorno[2]=LamElast_Contorno[2]*<1,0>;                //nos lados z=cte
    #declare LamElast_Contorno[3]=LamElast_Contorno[3]*<1,0>;
  #end
  #switch(QuantosGrausDeLiberdade)
    #case(1)
      #local Zero=0;
    #break
    #case(2)
      #local Zero=<0,0>;
    #break
    #case(3)
      #local Zero=<0,0,0>;
    #break
    #case(4)
      #local Zero=<0,0,0,0>;
    #break
    #case(5)
      #local Zero=<0,0,0,0,0>;
    #break
    #else
      #warning "O numero de graus de liberdade das ondas deve ser um inteiro\n  compreendido entre 1 e 5, ambos inclusive."
      #warning "O valor fornecido no serve; considerar-se a um so grau de liberdade."
      #local Zero=0;
  #end
  #ifdef(LamElast_Integral) #local SomaElongacoes=Zero; #end

  #if(Tempo=Tempo0)

    //Instante inicial
    #if(Xose_Mensagens)
      #debug concat("    Estam-se a guardar as condicoes iniciais da lamina elastica \"", Nome, "\".\n")
    #end
    #declare LamElast_Elongacoes=array[2][NoFilas][NoCols]
    #local Fila=0;
    #while(Fila<NoFilas)
      #local Col=0;
      #while(Col<NoCols)
        #ifdef(LamElast_Elongacoes0)
          #declare LamElast_Elongacoes[1][Fila][Col]=LamElast_Elongacoes0[Fila][Col];
        #else
          #ifdef(LamElast_MacroElongaco0)
            #declare LamElast_Elongacoes[1][Fila][Col] =
              LamElast_MacroElongaco0(<Fila/DensGrelha, Col/DensGrelha>);
          #else
            #declare LamElast_Elongacoes[1][Fila][Col]=Zero;
          #end
        #end
        #ifdef(LamElast_Integral)
          #declare SomaElongacoes=SomaElongacoes+LamElast_Elongacoes[1][Fila][Col];
        #end
        #ifdef(LamElast_Velocidades0)
          #declare LamElast_Elongacoes[0][Fila][Col]=LamElast_Elongacoes[1][Fila][Col] -
            LamElast_Velocidades0[Fila][Col]/FreqCalc;
        #else
          #ifdef(LamElast_MacroVelocidade0)
            #declare LamElast_Elongacoes[0][Fila][Col]=LamElast_Elongacoes[1][Fila][Col] -
              LamElast_MacroVelocidade0(<Fila/DensGrelha, Col/DensGrelha>)/FreqCalc;
          #else
            #declare LamElast_Elongacoes[0][Fila][Col]=LamElast_Elongacoes[1][Fila][Col];
          #end
        #end
        #declare Col=Col+1;
      #end
      #declare Fila=Fila+1;
    #end
    #ifdef(LamElast_Integral) #declare LamElast_ElongacoMedia=SomaElongacoes/(NoFilas*NoCols); #end
    #declare LamElast_Vez=0;

  #else

    #if(Xose_Mensagens)
      #debug concat("    Estam-se a calcular as pequenas deformacoes da lamina elastica \"", Nome, "\".\n")
    #end
    #include NomeDoFilego
    #local Tempus=LamElast_Tempo;
    #local FilaInf=(LamElast_Contorno[0].u? 1: 0)+(LamElast_Contorno[0].v? 1: 0);
    #local FilaSup=NoFilas-(LamElast_Contorno[1].u? 1: 0)-(LamElast_Contorno[1].v? 1: 0);
    #local ColInf=(LamElast_Contorno[2].u? 1: 0)+(LamElast_Contorno[2].v? 1: 0);
    #local ColSup=NoCols-(LamElast_Contorno[3].u? 1: 0)-(LamElast_Contorno[3].v? 1: 0);

    #while(Tempus<Tempo)

      // Pontos interiores
      
      #local Fila=max(FilaInf,1);
      #while(Fila<min(FilaSup,NoFilas-1))
        #local Col=max(ColInf,1);
        #while(Col<min(ColSup,NoCols-1))
          #declare LamElast_Elongacoes[LamElast_Vez][Fila][Col] =
            pow(Rapidez*DensGrelha/FreqCalc,2) *
              ( LamElast_Elongacoes[!LamElast_Vez][Fila-1][Col] +
                LamElast_Elongacoes[!LamElast_Vez][Fila+1][Col] +
                LamElast_Elongacoes[!LamElast_Vez][Fila][Col+1] +
                LamElast_Elongacoes[!LamElast_Vez][Fila][Col-1] -
                4*LamElast_Elongacoes[!LamElast_Vez][Fila][Col]
              ) +
            LamElast_Perturbaco(<Fila, Col>/DensGrelha, Tempus)/pow(FreqCalc,2) +
            LamElast_Elongacoes[!LamElast_Vez][Fila][Col]*(1-RigResorte/pow(FreqCalc,2)) +
            ( LamElast_Elongacoes[!LamElast_Vez][Fila][Col] -
              LamElast_Elongacoes[LamElast_Vez][Fila][Col] ) * (1-Amortecimento/FreqCalc)
            #ifdef(LamElast_Integral)
              + LamElast_Integral(Tempus)/(DimsDominio.x*DimsDominio.y)-LamElast_ElongacoMedia;
              #declare SomaElongacoes=SomaElongacoes+LamElast_Elongacoes[LamElast_Vez][Fila][Col];
            #else
              ;
            #end
          #declare Col=Col+1;
        #end
        #declare Fila=Fila+1;
      #end

      // Condies de contorno
      #local Fila=FilaInf;
      #while(Fila<FilaSup)
        #if(NoCols>1)
          #if(LamElast_Contorno[3].u)
            #declare LamElast_Elongacoes[LamElast_Vez][Fila][NoCols-1] =
              LamElast_ElongContorno(<Fila, NoCols-1>/DensGrelha, Tempus);
            #if(LamElast_Contorno[3].v)  
              #declare LamElast_Elongacoes[LamElast_Vez][Fila][NoCols-2] =
                LamElast_Elongacoes[LamElast_Vez][Fila][NoCols-1] -
                LamElast_PendContorno(<Fila, NoCols-1>/DensGrelha, Tempus)/DensGrelha;
              #ifdef(LamElast_Integral)
                #declare SomaElongacoes=SomaElongacoes+LamElast_Elongacoes[LamElast_Vez][Fila][NoCols-2];
              #end
            #end
          #else
            #if(LamElast_Contorno[3].v)  
              #declare LamElast_Elongacoes[LamElast_Vez][Fila][NoCols-1] =
                LamElast_Elongacoes[LamElast_Vez][Fila][NoCols-2] +
                LamElast_PendContorno(<Fila, NoCols-1>/DensGrelha, Tempus)/DensGrelha;
            #else
              #declare LamElast_Elongacoes[LamElast_Vez][Fila][NoCols-1] =
                #if((Fila>0)&(Fila<NoFilas-1))
                  pow(Rapidez*DensGrelha/FreqCalc,2) *
                    ( LamElast_Elongacoes[!LamElast_Vez][Fila-1][NoCols-1] +
                      LamElast_Elongacoes[!LamElast_Vez][Fila+1][NoCols-1] -
                      2*LamElast_Elongacoes[!LamElast_Vez][Fila][NoCols-1]
                    ) +
                #end
                LamElast_Perturbaco(<Fila, NoCols-1>/DensGrelha, Tempus)/pow(FreqCalc,2) +
                LamElast_Elongacoes[!LamElast_Vez][Fila][NoCols-1]*(1-RigResorte/pow(FreqCalc,2)) +
                ( LamElast_Elongacoes[!LamElast_Vez][Fila][NoCols-1] -
                  LamElast_Elongacoes[LamElast_Vez][Fila][NoCols-1] ) * (1-Amortecimento/FreqCalc)
                #ifdef(LamElast_Integral)
                  + LamElast_Integral(Tempus)/(DimsDominio.x*DimsDominio.y)-LamElast_ElongacoMedia
                #end
              ;
            #end
          #end
          #ifdef(LamElast_Integral)
            #declare SomaElongacoes=SomaElongacoes+LamElast_Elongacoes[LamElast_Vez][Fila][NoCols-1];
          #end
        #end
        #if(LamElast_Contorno[2].u)
          #declare LamElast_Elongacoes[LamElast_Vez][Fila][0] =
            LamElast_ElongContorno(<Fila/DensGrelha, 0>, Tempus);
          #if(LamElast_Contorno[2].v)  
            #declare LamElast_Elongacoes[LamElast_Vez][Fila][1] =
              LamElast_Elongacoes[LamElast_Vez][Fila][0] +
              LamElast_PendContorno(<Fila/DensGrelha, 0>, Tempus)/DensGrelha;
            #ifdef(LamElast_Integral)
              #declare SomaElongacoes=SomaElongacoes+LamElast_Elongacoes[LamElast_Vez][Fila][1];
            #end
          #end
        #else
          #if(LamElast_Contorno[2].v)  
            #declare LamElast_Elongacoes[LamElast_Vez][Fila][0] =
              LamElast_Elongacoes[LamElast_Vez][Fila][1] -
              LamElast_PendContorno(<Fila/DensGrelha, 0>, Tempus)/DensGrelha;
          #else
            #declare LamElast_Elongacoes[LamElast_Vez][Fila][0] =
              #if((Fila>0)&(Fila<NoFilas-1))
                pow(Rapidez*DensGrelha/FreqCalc,2) *
                  ( LamElast_Elongacoes[!LamElast_Vez][Fila-1][0] +
                    LamElast_Elongacoes[!LamElast_Vez][Fila+1][0] -
                    2*LamElast_Elongacoes[!LamElast_Vez][Fila][0]
                  ) +
              #end
              LamElast_Perturbaco(<Fila/DensGrelha, 0>, Tempus)/pow(FreqCalc,2) +
              LamElast_Elongacoes[!LamElast_Vez][Fila][0]*(1-RigResorte/pow(FreqCalc,2)) +
              ( LamElast_Elongacoes[!LamElast_Vez][Fila][0] -
                LamElast_Elongacoes[LamElast_Vez][Fila][0] ) * (1-Amortecimento/FreqCalc)
              #ifdef(LamElast_Integral)
                + LamElast_Integral(Tempus)/(DimsDominio.x*DimsDominio.y)-LamElast_ElongacoMedia
              #end
            ;
          #end
        #end
        #ifdef(LamElast_Integral)
          #declare SomaElongacoes=SomaElongacoes+LamElast_Elongacoes[LamElast_Vez][Fila][0];
        #end
        #declare Fila=Fila+1;
      #end
      #local Col=0;
      #while(Col<NoCols)
        #if(NoFilas>1)
          #if(LamElast_Contorno[1].u)
            #declare LamElast_Elongacoes[LamElast_Vez][NoFilas-1][Col] =
              LamElast_ElongContorno(<NoFilas-1, Col>/DensGrelha, Tempus);
            #ifdef(LamElast_Integral)
              #declare SomaElongacoes=SomaElongacoes+LamElast_Elongacoes[LamElast_Vez][NoFilas-1][Col];
            #end
            #if(LamElast_Contorno[1].v)  
              #declare LamElast_Elongacoes[LamElast_Vez][NoFilas-2][Col] =
                LamElast_Elongacoes[LamElast_Vez][NoFilas-1][Col] -
                LamElast_PendContorno(<NoFilas-1, Col>/DensGrelha, Tempus)/DensGrelha;
              #ifdef(LamElast_Integral)
                #declare SomaElongacoes=SomaElongacoes+LamElast_Elongacoes[LamElast_Vez][NoFilas-2][Col];
              #end
            #end
          #else
            #if(LamElast_Contorno[1].v)  
              #declare LamElast_Elongacoes[LamElast_Vez][NoFilas-1][Col] =
                LamElast_Elongacoes[LamElast_Vez][NoFilas-2][Col] +
                LamElast_PendContorno(<NoFilas-1, Col>/DensGrelha, Tempus)/DensGrelha;
              #ifdef(LamElast_Integral)
                #declare SomaElongacoes=SomaElongacoes+LamElast_Elongacoes[LamElast_Vez][NoFilas-1][Col];
              #end
            #else
              #if((Col>max(0,ColInf-1))&(Col<min(NoCols-1,ColSup)))
                #declare LamElast_Elongacoes[LamElast_Vez][NoFilas-1][Col] =
                  pow(Rapidez*DensGrelha/FreqCalc,2) *
                    ( LamElast_Elongacoes[!LamElast_Vez][NoFilas-1][Col+1] +
                      LamElast_Elongacoes[!LamElast_Vez][NoFilas-1][Col-1] -
                      2*LamElast_Elongacoes[!LamElast_Vez][NoFilas-1][Col]
                    ) +
                  LamElast_Perturbaco(<NoFilas-1, Col>/DensGrelha, Tempus)/pow(FreqCalc,2) +
                  LamElast_Elongacoes[!LamElast_Vez][NoFilas-1][Col]*(1-RigResorte/pow(FreqCalc,2)) +
                  ( LamElast_Elongacoes[!LamElast_Vez][NoFilas-1][Col] -
                    LamElast_Elongacoes[LamElast_Vez][NoFilas-1][Col] ) * (1-Amortecimento/FreqCalc)
                  #ifdef(LamElast_Integral)
                    + LamElast_Integral(Tempus)/(DimsDominio.x*DimsDominio.y)-LamElast_ElongacoMedia;
                    #declare SomaElongacoes=SomaElongacoes+LamElast_Elongacoes[LamElast_Vez][NoFilas-1][Col];
                  #else
                    ;
                  #end
              #end
            #end
          #end
        #end
        #if(LamElast_Contorno[0].u)
          #declare LamElast_Elongacoes[LamElast_Vez][0][Col] =
            LamElast_ElongContorno(<0, Col/DensGrelha>, Tempus);
          #ifdef(LamElast_Integral)
            #declare SomaElongacoes=SomaElongacoes+LamElast_Elongacoes[LamElast_Vez][0][Col];
          #end
          #if(LamElast_Contorno[0].v)  
            #declare LamElast_Elongacoes[LamElast_Vez][1][Col] =
              LamElast_Elongacoes[LamElast_Vez][0][Col] +
              LamElast_PendContorno(<0, Col/DensGrelha>, Tempus)/DensGrelha;
            #ifdef(LamElast_Integral)
              #declare SomaElongacoes=SomaElongacoes+LamElast_Elongacoes[LamElast_Vez][1][Col];
            #end
          #end
        #else
          #if(LamElast_Contorno[0].v)  
            #declare LamElast_Elongacoes[LamElast_Vez][0][Col] =
              LamElast_Elongacoes[LamElast_Vez][1][Col] -
              LamElast_PendContorno(<0, Col/DensGrelha>, Tempus)/DensGrelha;
            #ifdef(LamElast_Integral)
              #declare SomaElongacoes=SomaElongacoes+LamElast_Elongacoes[LamElast_Vez][0][Col];
            #end
          #else
            #if((Col>max(0,ColInf-1))&(Col<min(NoCols-1,ColSup)))
              #declare LamElast_Elongacoes[LamElast_Vez][0][Col] =
                pow(Rapidez*DensGrelha/FreqCalc,2) *
                  ( LamElast_Elongacoes[!LamElast_Vez][0][Col+1] +
                    LamElast_Elongacoes[!LamElast_Vez][0][Col-1] -
                    2*LamElast_Elongacoes[!LamElast_Vez][0][Col]
                  ) +
                LamElast_Perturbaco(<0, Col/DensGrelha>, Tempus)/pow(FreqCalc,2) +
                LamElast_Elongacoes[!LamElast_Vez][0][Col]*(1-RigResorte/pow(FreqCalc,2)) +
                ( LamElast_Elongacoes[!LamElast_Vez][0][Col] -
                  LamElast_Elongacoes[LamElast_Vez][0][Col] ) * (1-Amortecimento/FreqCalc)
                #ifdef(LamElast_Integral)
                  + LamElast_Integral(Tempus)/(DimsDominio.x*DimsDominio.y)-LamElast_ElongacoMedia;
                  #declare SomaElongacoes=SomaElongacoes+LamElast_Elongacoes[LamElast_Vez][0][Col];
                #else
                  ;
                #end
            #end
          #end
        #end
        #declare Col=Col+1;
      #end

      #ifdef(LamElast_Integral)
        #declare LamElast_ElongacoMedia=SomaElongacoes/(NoFilas*NoCols);
      #end
      #declare LamElast_Vez=!LamElast_Vez;
      #declare Tempus=Tempus+1/FreqCalc;
    #end
    #local Tempo=Tempus;

  #end

  #if(Xose_Mensagens)
    #debug concat("    Estam-se a guardar as condicoes actuais da lamina elastica \"", Nome, "\".\n")
  #end
  #fopen Filego NomeDoFilego write
  #write( Filego,
    "// Arquivo de dados temporais para o \"Lamina elastica.inc\" de Xose\n\n#declare LamElast_Elongacoes=array[2][",
    str(NoFilas,0,0),"][",str(NoCols,0,0),"]{\n"
  )
  #local Toque=0;
  #while(Toque<2)
    #write(Filego,"  {\n")
    #local Fila=0;
    #while(Fila<NoFilas)
      #write(Filego,"    {")
      #local Col=0;
      #while(Col<NoCols)
        #write(Filego,
          #if(QuantosGrausDeLiberdade=1)
            NotacoCientifica(LamElast_Elongacoes[Toque][Fila][Col],17)
          #else
            NotacoCientificaVector(QuantosGrausDeLiberdade,LamElast_Elongacoes[Toque][Fila][Col],17)
          #end
          ,",")
        #declare Col=Col+1;
      #end
      #write(Filego,"}\n")
      #declare Fila=Fila+1;
    #end
    #write(Filego,"  }\n")
    #declare Toque=Toque+1;
  #end
  #write(Filego,"}\n#declare LamElast_Vez=",str(LamElast_Vez,0,0),";\n#declare LamElast_Tempo=",
    NotacoCientifica(Tempo,17),";")
  #ifdef(LamElast_Integral)
    #write(Filego,"\n#declare LamElast_ElongacoMedia=",
      #if(QuantosGrausDeLiberdade=1)
        NotacoCientifica(LamElast_ElongacoMedia,17)
      #else
        NotacoCientificaVector(QuantosGrausDeLiberdade,LamElast_ElongacoMedia,17)
      #end
      ,";");
  #end
  #fclose Filego

  #if(Xose_Mensagens)
    #debug "    Estam-se a definir LamElast_Pigmento e LamElast_Funco.\n"
  #end

  #ifndef(LamElast_MaxElong)
    #declare LamElast_MaxElong=1;
  #end

  #ifndef(LamElast_MinElong)
    #declare LamElast_MinElong=-1;
  #end

/*  // Instruces para superar o limite de 256  256 pontos na grelha, ainda n prontas.
  #local NivelMaxX=floor(log(NoFilas)/log(256))+1;
  #local NivelMaxZ=floor(log(NoCols)/log(256))+1;
  #local NivelX=NivelMaxX;
  #local NivelZ=NivelMaxZ;
  #declare LamElast_Pigmento=pigment{
    #local Fila=0;
    #while(Fila<NoFilas)
      #if(mod(Fila,pow(256,NivelX)))
        gradient x pigment_map{
      #end
      #declare Fila=Fila+1;
    #end
  }
*/

  #declare LamElast_Pigmento=pigment{
    gradient x pigment_map{
      #local Fila=0;
      #while(Fila<NoFilas)
        [Fila/(NoFilas-1)
          gradient z colour_map{
            #local Col=0;
            #while(Col<NoCols)
              [Col/(NoCols-1)
                rgb min( max( (LamElast_Elongacoes[!LamElast_Vez][Fila][Col]-LamElast_MinElong) /
                  (LamElast_MaxElong-LamElast_MinElong), 0), 1)
              ]
              #declare Col=Col+1;
            #end
          }
        ]
        #declare Fila=Fila+1;
      #end
    }
    scale<(NoFilas-1)/DensGrelha,1,(NoCols-1)/DensGrelha>
  }

  #local UnaCabura=function{pattern{pigment_pattern{LamElast_Pigmento}}}
  #declare LamElast_Funco =
    function{UnaCabura(x,y,z)*(LamElast_MaxElong-LamElast_MinElong)+LamElast_MinElong}
  //#undef UnaCabura

#end


#version versionLamElast;