// Arquivo auxiliar para o traador de raios de Persistence of Vision
// Arquivo: Lei de Coulomb.inc
// Vers: 3.5
// Descri: Facilita o desenho de cargas elctricas com os seus vectores posi, fora elctrica e
    //intensidade de campo elctrico.
// Data de cria: 18-IX-2002; aproveitei umha cena construda em fevereiro.
// Autor: Xos B. Carballeira Rabual

#local versionLeiCoulomb=version;
#version 3.5;

//Requere:
//#include "Figurinhas de Matematicas, Fisica e Quimica.inc"


//----------------------------------------- Ferramentas ---------------------------------------------------

#macro DesenhaVectorPosico(Origem, Extremo)
  object{FlechaRecta(Origem, Extremo,.0015,.006) #if(!Xose_SemTexturas) pigment{rgbf<.8,.5,0,.7>} #end}
#end

#macro VectorCampoElectrico(Ponto)
  #local ContaCargas=0;
  #local CampoAcumulado=<0,0,0>;
  #while (ContaCargas<NumeroDeCargas)
    #declare CampoAcumulado=CampoAcumulado +
      1/(4*pi*ConstanteDielectrica)*Cargas[ContaCargas]/pow(vlength(Ponto-Posicoes[ContaCargas]),3) *
      (Ponto-Posicoes[ContaCargas]);
    #declare ContaCargas=ContaCargas+1;
  #end
  CampoAcumulado
#end

#macro DesenhaVectorCampoElectrico(Ponto)
  FlechaRecta(Ponto, Ponto+EscalaDasIntensidadesDeCampo*VectorCampoElectrico(Ponto),.002,.008)
#end

#macro VectorForcaElectrica(NumeroDaCarga)
  #local ContaCargas=0;
  #local ForcaAcumulada=<0,0,0>;
  #while (ContaCargas<NumeroDeCargas)
    #if (ContaCargas!=NumeroDaCarga)
      #declare ForcaAcumulada=ForcaAcumulada +
        Cargas[NumeroDaCarga]/(4*pi*ConstanteDielectrica) *
        Cargas[ContaCargas]/pow(vlength(Posicoes[NumeroDaCarga]-Posicoes[ContaCargas]),3) *
        (Posicoes[NumeroDaCarga]-Posicoes[ContaCargas]);
    #end
    #declare ContaCargas = ContaCargas+1;
  #end
  ForcaAcumulada
#end

#macro DesenhaVectorForcaElectrica(NumeroDaCarga)
  object{
    FlechaRecta(Posicoes[NumeroDaCarga],
      Posicoes[NumeroDaCarga]+EscalaDasForcas*VectorForcaElectrica(NumeroDaCarga),.002,.008)
    #if(!Xose_SemTexturas) pigment{rgbf<.8,0,.5,.7>} #end
  }
#end

#declare ConstanteDielectricaDoBaleiro=1/(4*pi*9e9);          //Constante dielctrica do vazio em C^2*N^(-2)*m^(-2)


//------------------------------------- Representa das cargas --------------------------------------------------

#ifndef(DesenhaCargas)
  #declare DesenhaCargas=yes;
#end
#if(DesenhaCargas & !defined(AspeitoDaCarga))
  #declare AspeitoDaCarga=object{Bola scale .003}
#end

#if(DesenhaCargas)
  #declare ContaCargas=0;                                  //Desenhamos bolinhas que representam as cargas.
  #while (ContaCargas < NumeroDeCargas)
    object{AspeitoDaCarga translate Posicoes[ContaCargas]}
    #declare ContaCargas=ContaCargas+1;
  #end
  #undef ContaCargas
#end


#version versionLeiCoulomb;