// Arquivo auxiliar para o traador de raios de Persistence of Vision
// Arquivo: Matematica.inc
// Vers: 3.6
// Descri: Complementos matemticos para as nossas cenas
// Data da cria: 16-IV-2002
// Data da ltima actualiza: 31-X-2005
// Autor: Xos B. Carballeira Rabual


#local versionMatametika=version;
#version 3.6;

#ifndef(Xose_Mensagens)
  #declare Xose_Mensagens=false;
#end

#if(Xose_Mensagens)
  #debug "    Esta-se a executar o arquivo \"Matematica.inc\", que fornece complementos matematicos.\n"
#end

//===================================== ESCRITA DE NMEROS E VECTORES =======================================================

#macro NotacoCientifica(Numero, QuantasCifrasSignificativas)
  #if(Numero=0)
    #local Expoente=0;
  #else
    #local Expoente=floor(log(abs(Numero)));
  #end
  #if(((Expoente<QuantasCifrasSignificativas)&(Expoente>=0))|((Expoente<1)&(Expoente>-3)))
    str(Numero,0,QuantasCifrasSignificativas-Expoente-1)
  #else
    concat(str(Numero/pow(10,Expoente),0,QuantasCifrasSignificativas-1),"e",str(Expoente,0,0))
  #end
#end

#macro NotacoCientificaVector(Dimenso, Vector, QuantasCifrasSignificativas)
  #local Texto=concat("<", NotacoCientifica(Vector.x, QuantasCifrasSignificativas), ",",
    NotacoCientifica(Vector.y, QuantasCifrasSignificativas));
  #if(Dimenso>2)
    #declare Texto=concat(Texto, ",", NotacoCientifica(Vector.z, QuantasCifrasSignificativas));
    #if(Dimenso>3)
      #declare Texto=concat(Texto, ",", NotacoCientifica(Vector.t, QuantasCifrasSignificativas));
      #if(Dimenso>4)
        #declare Texto =
          concat(Texto, ",", NotacoCientifica(Vector.transmit, QuantasCifrasSignificativas));
      #end
    #end
  #end
  concat(Texto, ">")
#end

//========================================== TEORIA DOS NMEROS =======================================================

#macro NumerosPrimos(Quantos) // Devolve uma tbua linear com os primeiros nmeros primos,
                                                                            // na quantidade pedida.
  #local Inici=array[1]{2}
  MaisNumerosPrimos(Inici,Quantos)
#end

#macro MaisNumerosPrimos(PrimosPrev, QuantosTotal) // Amplia uma tbua dada PrimosPrev com os primeiros
                                        // nmeros primos at obter QuantosTotal nmeros primos.
  #local QuantosLevamos=dimension_size(PrimosPrev,1);
  #local Primos=array[QuantosTotal]
  #local Index=0;
  #while(Index<QuantosLevamos)
    #declare Primos[Index]=PrimosPrev[Index];
    #declare Index=Index+1;
  #end
  #if((Index=1)&(QuantosTotal>1))
    #declare Primos[1]=3;
    #declare Index=2;
  #end
  #local Ensaio=Primos[Index-1];
  #while(Index<QuantosTotal)
    #local EhPrimo=false;
    #while(!EhPrimo)
      #declare Ensaio=Ensaio+2;
      #local Contagem=1;
      #declare EhPrimo=true;
      #while((Primos[Contagem]<=sqrt(Ensaio))&EhPrimo)
        #if(mod(Ensaio/Primos[Contagem],1)=0)
          #declare EhPrimo=false;
        #end
        #declare Contagem=Contagem+1;
      #end
    #end
    #declare Primos[Index]=Ensaio;
    #declare Index=Index+1;
  #end
  Primos
#end

//  A seguinte macro devolve uma tbua linear com todos os nmeros primos menores ou iguais que Cota,
// achando no comeo QuantInicial nmeros primos, e engadindo uma quantidade deles igual a Passo em cada
// itera.
#macro NumerosPrimosAte(Cota, QuantInicial, Passo)
  #local lPasso=ceil(Passo);
  #if(Cota<2)
    #local Resuu=array[1]//{2}  // Provoca um erro; n sei por qu :(
    #declare Resuu[0]=2;
    Resuu
  #else
    #local Quant=int(QuantInicial);
    #local Acu=NumerosPrimos(Quant)
    #while(Acu[Quant-1]<Cota)
      #declare Quant=Quant+lPasso;
      #declare Acu=MaisNumerosPrimos(Acu,Quant)
    #end
    #local Indice=Quant-1;
    #while(Acu[Indice]>Cota)
      #declare Indice=Indice-1;
    #end
    #if(Indice<Quant-1)
      #local Resul=array[Indice+1]
      #local Gago=0;
      #while(Gago<=Indice)
        #declare Resul[Gago]=Acu[Gago];
        #declare Gago=Gago+1;
      #end
      Resul
    #else
      Acu
    #end
  #end
#end

#macro DescompFactPrim(Ene)  // Devolve uma tbua linear com os expoentes da descomposi do nmero
                                       // natural no nulo Ene em factores primos.
  #if(Ene<2)
    #local Expys=array[1]{0}
  #else
    #local Prix=NumerosPrimosAte(Ene, Ene/2, Ene/10)
    #local Num=dimension_size(Prix,1);
    #local Expzs=array[Num]
    #local Indice=0;
    #while(Indice<Num)
      #declare Expzs[Indice]=0;
      #declare Indice=Indice+1;
    #end
    #local Indice=0;
    #local Enada=int(Ene);
    #while(Enada>1)
      #if(mod(Enada,Prix[Indice])=0)
        #declare Enada=Enada/Prix[Indice];
        #declare Expzs[Indice]=Expzs[Indice]+1;
      #else
        #declare Indice=Indice+1;
      #end
    #end
    #local Expys=array[Indice+1]
    #local Indy=0;
    #while(Indy<=Indice)
      #declare Expys[Indy]=Expzs[Indy];
      #declare Indy=Indy+1;
    #end
  #end
  Expys
#end

//============================================ FUNES VRIAS =======================================================

#macro Factorial(Ene)  // Factorial dum nmero natural
  #local Prod=1;
  #local Fact=2;
  #while(Fact<=Ene)
    #declare Prod=Prod*Fact;
    #declare Fact=Fact+1;
  #end
  Prod
#end


//================================================ SRIES ===========================================================

// Soma parcial duma srie. Os termos da sucess devem se definir mediante uma macro MatXos_Sucesso(Indice)
#macro SomaParcial(Indice)
  #local Soma=0;
  #local Qual=0;
  #while(Qual<=Indice)
    #declare Soma=Soma+MatXos_Sucesso(Qual);
    #declare Qual=Qual+1;
  #end
  Soma
#end

//   Soma da srie dada pola macro MatXos_Sucesso(Indice), aproximada pola primeira soma parcial
// tal que a soma dos valores absolutos dos seus Margem derradeiros termos for menor ou igual que Epsilon,
// abrangendo n menos de IndiceMin termos
#macro SomaAprox(Epsilon, Margem, IndiceMin)
  #local Indice=0;
  #local Soma=0;
  #local Buffer=array[Margem]
  #local Continua=true;
  #while(Continua)
    #local Termo=MatXos_Sucesso(Indice);
    #declare Soma=Soma+Termo;
    #declare Buffer[mod(Indice,Margem)]=abs(Termo);
    #declare Indice=Indice+1;
    #if(Indice>=Margem)
      #local Voluptuous=0;
      #local OMeuDedo=0;
      #while(OMeuDedo<Margem)
        #declare Voluptuous=Voluptuous+Buffer[OMeuDedo];
        #declare OMeuDedo=OMeuDedo+1;
      #end
      #declare Continua=((Indice<IndiceMin)|(Voluptuous>Epsilon));
    #end
  #end
  Soma
#end

//   Soma parcial da srie de potncias com coeficientes dados por MatXos_Sucesso(Indice), avaliada no
// valor introduzido da varivel
#macro SomaParcialPots(Indice, Variavel)
  #local Soma=MatXos_Sucesso(0);
  #local Qual=1;
  #while(Qual<=Indice)
    #local Coeficiente=MatXos_Sucesso(Qual);
    #if(Coeficiente!=0)
      #declare Soma=Soma+Coeficiente/abs(Coeficiente)*pow(pow(abs(Coeficiente),1/Qual)*Variavel,Qual);
    #end
    #declare Qual=Qual+1;
  #end
  Soma
#end

//   Soma da srie de potncias com coeficientes dados pola macro MatXos_Sucesso(Indice), aproximada pola
// primeira soma parcial tal que a soma dos valores absolutos dos seus Margem derradeiros termos for menor
// ou igual que Epsilon, abrangendo n menos de IndiceMin termos, e avaliada no valor introduzido da
// varivel
#macro SomaAproxPots(Epsilon, Margem, IndiceMin, Variavel)
  #local Indice=1;
  #local Soma=MatXos_Sucesso(0);
  #local Buffer=array[Margem]
  #declare Buffer[0]=abs(Soma);
  #local Continua=true;
  #while(Continua)
    #local Coeficiente=MatXos_Sucesso(Indice);
    #if(Coeficiente=0)
      #local Termo=0;
    #else
      #local Termo=Coeficiente/abs(Coeficiente)*pow(pow(abs(Coeficiente),1/Indice)*Variavel,Indice);
      #declare Soma=Soma+Termo;
    #end
    #declare Buffer[mod(Indice,Margem)]=abs(Termo);
    #declare Indice=Indice+1;
    #if(Indice>=Margem)
      #local Voluptuous=0;
      #local OMeuDedo=0;
      #while(OMeuDedo<Margem)
        #declare Voluptuous=Voluptuous+Buffer[OMeuDedo];
        #declare OMeuDedo=OMeuDedo+1;
      #end
      #declare Continua=((Indice<IndiceMin)|(Voluptuous>Epsilon));
    #end
  #end
  Soma
#end

//   Soma parcial da srie de potncias com coeficientes dados por
// MatXos_Sucesso(Indice)/Factorial(Indice), avaliada no valor introduzido da varivel
#macro SomaParcialTaylor(Indice, Variavel)
  #local Soma=MatXos_Sucesso(0);
  #local Qual=1;
  #while(Qual<=Indice)
    #local Coeficiente=MatXos_Sucesso(Qual);
    #if(Coeficiente!=0)
      #local Proud=Variavel;
      #local ImosPolo=2;
      #while(ImosPolo<=Qual)
        #declare Proud=Proud*Variavel/ImosPolo;
        #declare ImosPolo=ImosPolo+1;
      #end
      #declare Soma=Soma+Coeficiente*Proud;
    #end
    #declare Qual=Qual+1;
  #end
  Soma
#end

//   Soma da srie de potncias com coeficientes dados por MatXos_Sucesso(Indice)/Factorial(Indice),
// aproximada pola primeira soma parcial tal que a soma dos valores absolutos dos seus derradeiros Margem
// termos for menor ou igual que Epsilon, abrangendo n menos de IndiceMin termos, e avaliada no valor
// introduzido da varivel
#macro SomaAproxTaylor(Epsilon, Margem, IndiceMin, Variavel)
  #local Indice=1;
  #local Soma=MatXos_Sucesso(0);
  #local Buffer=array[Margem]
  #declare Buffer[0]=abs(Soma);
  #local Continua=true;
  #while(Continua)
    #local Coeficiente=MatXos_Sucesso(Indice);
    #if(Coeficiente=0)
      #local Termo=0;
    #else
      #local Proud=Variavel;
      #local ImosPolo=2;
      #while(ImosPolo<=Indice)
        #declare Proud=Proud*Variavel/ImosPolo;
        #declare ImosPolo=ImosPolo+1;
      #end
      #local Termo=Coeficiente*Proud;
      #declare Soma=Soma+Termo;
    #end
    #declare Buffer[mod(Indice,Margem)]=abs(Termo);
    #declare Indice=Indice+1;
    #if(Indice>=Margem)
      #local Voluptuous=0;
      #local OMeuDedo=0;
      #while(OMeuDedo<Margem)
        #declare Voluptuous=Voluptuous+Buffer[OMeuDedo];
        #declare OMeuDedo=OMeuDedo+1;
      #end
      #declare Continua=((Indice<IndiceMin)|(Voluptuous>Epsilon));
    #end
  #end
  Soma
#end


//============================================= TRANSFORMACES ======================================================

#macro RotEuler(Psi, Theta, Phi)  //Rota arredor da origem, dados os seus ngulos de Euler em radianos
  rotate<0,0,degrees(Psi)>
  rotate<degrees(Theta),0,0>
  rotate<0,0,degrees(Phi)>
#end

#macro RotVector(VectRot)  //Rota arredor do eixo que passa pola origem, com a direc de VectRot, na
  #local Phi=vlength(VectRot); //que o ngulo de rota, em radianos,  o comprimento de VectRot; o senso
  #if(Phi)                         //obedece a regra do parafuso, de rosca esquerda se o sistema
    #if(VectRot.z|VectRot.y)                  //de referncia  levgiro.
      #local Ajuste=atan2(VectRot.z,VectRot.y);
    #else
      #local Ajuste=0;
    #end
    rotate<degrees(-Ajuste),0,degrees(-acos(VectRot.x/Phi))>
    rotate<degrees(Phi),0,degrees(acos(VectRot.x/Phi))>
    rotate<degrees(Ajuste),0,0>
  #end
#end

#macro RotAntDep(Antes,Depois)  //Rota que leva o vector Antes no Depois polo caminho mais curto.
  #local Uvecros=vcross(Antes,Depois);
  #if(vlength(Uvecros)=0)
    #local Uvecros=vcross(x,Antes);
    #if(vlength(Uvecros)=0)
      #local Uvecros=vcross(y,Antes);
    #end
  #end
  RotVector(acos(vdot(vnormalize(Antes),vnormalize(Depois)))*vnormalize(Uvecros))
#end

#macro MatrizTransform(Transformaco, ComTransl)  //Matriz associada  transforma
  #local Fonfy=function{transform{Transformaco}}
  #local Trasl=Fonfy(0,0,0);
  #local Ecs=Fonfy(1,0,0)-Trasl;
  #local Guai=Fonfy(0,1,0)-Trasl;
  #local Sed=Fonfy(0,0,1)-Trasl;
  #if(ComTransl)
    array[3][4]{
      {Ecs.x, Guai.x, Sed.x , Trasl.x}
      {Ecs.y, Guai.y, Sed.y , Trasl.y}
      {Ecs.z, Guai.z, Sed.z , Trasl.z}
    }
  #else
    array[3][3]{
      {Ecs.x, Guai.x, Sed.x}
      {Ecs.y, Guai.y, Sed.y}
      {Ecs.z, Guai.z, Sed.z}
    }
  #end
#end

#macro TransformMatriz(Matriz)  //Transforma associada  matriz
  #if(dimensions(Matriz)!=2)
    #error "Requer-se umha matriz bidimensional de 3 filas e 3 ou 4 colunas."
  #end
  #if(dimension_size(Matriz,1)!=3) #error "Requer-se umha matriz de 3 filas." #end
  #switch(dimension_size(Matriz,2))
    #case(3) #local HaTransl=false; #break
    #case(4) #local HaTransl=true; #break
    #else #error "Requer-se umha matriz de 3 ou 4 colunas."
  #end
  matrix< Matriz[0][0], Matriz[1][0], Matriz[2][0],
          Matriz[0][1], Matriz[1][1], Matriz[2][1],
          Matriz[0][2], Matriz[1][2], Matriz[2][2],
    #if(HaTransl)
          Matriz[0][3], Matriz[1][3], Matriz[2][3]
    #else            0,            0,            0
    #end                                           >
#end

//  Lembre-se que para fazer umha reflex ou rota na qual o espelho ou o eixo, respectivamente, passem
// polo ponto (x0,y0,z0), avonda:

//  translate<-x0,-y0,-z0>  //Translada o elemento central at a origem.
//  /*rota ou reflex respeito do elemento central transladado*/
//  translate<x0,y0,z0>     //Desfai a transla inicial.

//  Quando o elemento central passa pola origem, mas n tem a orienta de nengum eixo ou plano coordena-
// do, segundo corresponder, pode-se fazer: 1) rota que leve o elemento central nalgum coordenado, 2)
// rota ou reflex respeito do elemento central rotado, e 3) rota inversa da inicial.
//  Se o elemento central n passa pola origem nem tem orienta coordenada, podem-se combinar os dous
// procedimentos anteriores: 1) translada-se o elemento central at a origem, 2) rota-se at que coinci-
// dir com um elemento coordenado, 3) efectua-se a rota ou reflex arredor do elemento central transfor-
// mado, 4) desfai-se a rota do passo 2, e 5) desfai-se a trasla do comeo.


//================================================ MATRIZES ===========================================================================

//------------------------------------------------ Operaes ---------------------------------------------------------------------------

#macro SomaM(A,B)
  #if((dimension_size(A,1)!=dimension_size(B,1))|(dimension_size(A,2)!=dimension_size(B,2)))
    #error "As matrizes sumandos devem ter as mesmas dimensoes."
  #end
  #local Soma=array[dimension_size(A,1)][dimension_size(A,2)]
  #local Fila=0;
  #while(Fila<dimension_size(A,1))
    #local Coluna=0;
    #while(Coluna<dimension_size(A,2))
      #declare Soma[Fila][Coluna]=A[Fila][Coluna]+B[Fila][Coluna];
      #declare Coluna=Coluna+1;
    #end
    #declare Fila=Fila+1;
  #end
  Soma
#end

#macro ProdEscM(Alfa,A)
  #local Produto=array[dimension_size(A,1)][dimension_size(A,2)]
  #local Fila=0;
  #while(Fila<dimension_size(A,1))
    #local Coluna=0;
    #while(Coluna<dimension_size(A,2))
      #declare Produto[Fila][Coluna]=Alfa*A[Fila][Coluna];
      #declare Coluna=Coluna+1;
    #end
    #declare Fila=Fila+1;
  #end
  Produto
#end

#macro ProdM(A,B)
  #if(dimension_size(A,2)!=dimension_size(B,1))
    #error "A matriz multiplicando deve ter tantas colunas como filas possua a matriz multiplicador."
  #end
  #local Produto=array[dimension_size(A,1)][dimension_size(B,2)]
  #local Fila=0;
  #while(Fila<dimension_size(A,1))
    #local Coluna=0;
    #while(Coluna<dimension_size(B,2))
      #declare Produto[Fila][Coluna]=0;
      #local Conta=0;
      #while(Conta<dimension_size(A,2))
        #declare Produto[Fila][Coluna]=Produto[Fila][Coluna]+A[Fila][Conta]*B[Conta][Coluna];
        #declare Conta=Conta+1;
      #end
      #declare Coluna=Coluna+1;
    #end
    #declare Fila=Fila+1;
  #end
  Produto
#end

#macro ProdPonto(A,B)  // Produto de ponto de dous sistemas ordenados de nmeros,
                            // na forma de tbuas lineares
  #if(dimension_size(A,1)!=dimension_size(B,1))
    #error "So existe o produto de ponto de dous sistemas ordenados de numeros com o mesmo tamanho."
  #end
  #local Soma=0;
  #local Tamanho=dimension_size(A,1);
  #local Indice=0;
  #while(Indice<Tamanho)
    #declare Soma=Soma+A[Indice]*B[Indice];
    #declare Indice=Indice+1;
  #end
  Soma
#end

#macro PotM(A,Expoente)  // Potncia de expoente inteiro
  #if(dimension_size(A,1)!=dimension_size(A,2))
    #error "No existem potencias de matrizes rectangulares."
  #else
    #declare Ordem=dimension_size(A,1);
  #end
  #if(Expoente=0)
    #local Resultado=MUnidade(Ordem,Ordem)
  #else
    #local Ene=abs(Expoente);
    #if(Expoente<0)
      #local Acu=MInversa(A)
    #else
      #local Acu=A
    #end
    #local Iter=floor(log(Ene)/log(2));
    #local Expu=pow(2,Iter);
    #local Rasuls=array[Iter+1]
    #declare Rasuls[0]=Acu
    #local Conta=1;
    #while(Conta<=Iter)
      #declare Rasuls[Conta]=ProdM(Rasuls[Conta-1],Rasuls[Conta-1])
      #declare Conta=Conta+1;
    #end
    #local Resultado=Rasuls[Iter]
    #declare Conta=Iter-1;
    #while(Expu<Ene)
      #if(Expu+pow(2,Conta)<=Ene)
        #declare Resultado=ProdM(Resultado,Rasuls[Conta])
        #declare Expu=Expu+pow(2,Conta);
      #end
      #declare Conta=Conta-1;
    #end
  #end
  Resultado
#end

//  Raiz Pe-sima aproximada da matriz quadrada A, por uma tentativa de generalizar o mtodo de
// Newton-Raphson. Se Proj=false, empregar-se- Estim0 como primeira aproxima da raiz procurada;
// se Proj=true, essa primeira estima ser a matriz diagonal com as razes Pe-simas dos valores absolu-
// tos dos elementos diagonais de A; se algum desses elementos for nulo, substituir-se- por 1.
//  O processo continuar at o erro estimado ser menor ou igual que Epsilon, ficando completas n menos
// de IndiceMin iteraes. Se o erro estimado superar nalguma itera CoutaDivg, a macro supor divergen-
// te a sucess de Newton-Raphson, e deter o clculo.

#macro RaizMNewtonRaphson(Epsilon, CoutaDivg, IndiceMin, A, Pe, Estim0, Proj)
  #if(dimension_size(A,1)!=dimension_size(A,2))
    #error "As matrizes rectangulares no tenhem raizes."
  #end
  #local Ordem=dimension_size(A,1);
  #if(Proj)
    #local Orbit=A
    #local Fila=0;
    #while(Fila<Ordem)
      #local Col=0;
      #while(Col<Ordem)
        #if(Col=Fila)
          #local Darjoum=Orbit[Fila][Col];
          #if(Darjoum=0)
            #declare Orbit[Fila][Col]=1;
          #else
            #declare Orbit[Fila][Col]=pow(abs(Darjoum),1/Pe);
          #end
        #else
          #declare Orbit[Fila][Col]=0;
        #end
        #declare Col=Col+1;
      #end
      #declare Fila=Fila+1;
    #end
  #else
    #local Orbit=Estim0
  #end
  #local Buffer=array[4]
  #local OrbitVelho=Orbit
  #local Indice=1;
  #local Continua=true;
  #while(Continua)
    #local Brinc=ProdEscM(1/Pe,ProdM(A,PotM(Orbit,1-Pe)))
    #declare Orbit=SomaM(ProdEscM(1-1/Pe,Orbit),Brinc)
    #declare Buffer[mod(Indice,4)]=NrmFrobenius(SomaM(Orbit,ProdEscM(-1,OrbitVelho)));
    #declare Indice=Indice+1;
    #if(Indice>4)
      #local OMeuDedo=0;
      #local Vulture=0;
      #while(OMeuDedo<4)
        #declare Vulture=Vulture+Buffer[OMeuDedo];
        #declare OMeuDedo=OMeuDedo+1;
      #end
    #else
      #local Vulture=Buffer[Indice-1];
    #end
    #if(Vulture>CoutaDivg)
      #declare Continua=false;
      #warning "A extenso do algoritmo de Newton-Raphson empregada para procurar uma raiz Pe-esima da matriz dada parece divergir com a estimaco inicial fornecida."
      #warning "Devolver-se-a o derradeiro iterante calculado."
    #else
      #declare Continua=((Vulture>Epsilon)|(Indice<IndiceMin));
    #end
    #declare OrbitVelho=Orbit
  #end
  Orbit
#end

//---------------------------------------------- Funes notveis ---------------------------------------------------------------------------

#macro Traca(A)
  #local Soma=0;
  #local Conta=0;
  #while(Conta<min(dimension_size(A,1),dimension_size(A,2)))
    #declare Soma=Soma+A[Conta][Conta];
    #declare Conta=Conta+1;
  #end
  Soma
#end

#macro Determinante(A)  // Emprega o mtodo de Gauss em cada coluna, e a seguir, o desenvolvimento por
                                                                                             //adjuntos.
  #if(dimension_size(A,1)=dimension_size(A,2))
    #local Ordem=dimension_size(A,1);
  #else
    #error "Apenas as matrizes quadradas tenhem determinante."
  #end
  #switch(Ordem)
    #case(1)
      #local Resultado=A[0][0];
    #break;
    #case(2)
      #local Resultado=A[0][0]*A[1][1]-A[1][0]*A[0][1];
    #break;
    #case(3)
      #local Resultado=A[0][0]*A[1][1]*A[2][2]+A[1][0]*A[2][1]*A[0][2]+A[2][0]*A[0][1]*A[1][2] -
        A[2][0]*A[1][1]*A[0][2]-A[0][0]*A[2][1]*A[1][2]-A[1][0]*A[0][1]*A[2][2];
    #break;
  #else
    #local lA=A;
    #local Fila=0;
    #local Pivote=-1;
    #while(Fila<Ordem)
      #if(Pivote=-1)
        #if(lA[Fila][0]!=0) #declare Pivote=Fila; #end
      #else
        #if(lA[Fila][0]!=0)
          #local Factor=-lA[Fila][0]/lA[Pivote][0];
          #local Col=0;
          #while(Col<Ordem)
            #declare lA[Fila][Col]=lA[Fila][Col]+Factor*lA[Pivote][Col];
            #declare Col=Col+1;
          #end
        #end
      #end
      #declare Fila=Fila+1;
    #end
    #if(Pivote=-1)
      #local Resultado=0;
    #else
      #local B=array[Ordem-1][Ordem-1]
      #local Fila=0;
      #while(Fila<Ordem-1)
        #local FilaOrg=((Fila<Pivote)? Fila: Fila+1);
        #local Col=0;
        #while(Col<Ordem-1)
          #declare B[Fila][Col]=lA[FilaOrg][Col+1];
          #declare Col=Col+1;
        #end
        #declare Fila=Fila+1;
      #end
      #local Factor=lA[Pivote][0];
      #undef lA
      #local Resultado=pow(-1,Pivote)*Factor*Determinante(B);
    #end
  #end
  Resultado
#end

#macro NrmEuclidea(A)  // Norma eucldea duma tbua linear
  #local Acu=0;
  #local Conta=0;
  #while(Conta<dimension_size(A,1))
    #local Elmt=A[Conta];
    #declare Acu=Acu+Elmt*Elmt;
    #declare Conta=Conta+1;
  #end
  #declare Acu=sqrt(Acu);
  Acu
#end

#macro NrmP(A,Pe)  // Norma Pe duma tbua linear, sendo Pe um nmero real maior ou igual que 1
  #local Acu=0;
  #local Conta=0;
  #while(Conta<dimension_size(A,1))
    #declare Acu=Acu+pow(abs(A[Conta]),Pe);
    #declare Conta=Conta+1;
  #end
  #declare Acu=pow(Acu,1/Pe);
  Acu
#end

#macro NrmInf(A)  // Norma infinito duma tbua linear
  #local Acu=0;
  #local Conta=0;
  #while(Conta<dimension_size(A,1))
    #declare Acu=max(Acu,abs(A[Conta]));
    #declare Conta=Conta+1;
  #end
  Acu
#end

#macro NrmFrobenius(A)  // Norma de Frobenius duma matriz
  #local Acu=0;
  #local Fila=0;
  #while(Fila<dimension_size(A,1))
    #local Col=0;
    #while(Col<dimension_size(A,2))
      #local Elmt=A[Fila][Col];
      #declare Acu=Acu+Elmt*Elmt;
      #declare Col=Col+1;
    #end
    #declare Fila=Fila+1;
  #end
  #declare Acu=sqrt(Acu);
  Acu
#end

#macro NrmFilas(A)  // Norma de filas duma matriz
  #local Max=0;
  #local Fila=0;
  #while(Fila<dimension_size(A,1))
    #local Acu=0;
    #local Col=0;
    #while(Col<dimension_size(A,2))
      #declare Acu=Acu+abs(A[Fila][Col]);
      #declare Col=Col+1;
    #end
    #declare Max=max(Max,Acu);
    #declare Fila=Fila+1;
  #end
  Max
#end

#macro NrmCols(A)  // Norma de colunas duma matriz
  #local Max=0;
  #local Col=0;
  #while(Col<dimension_size(A,2))
    #local Acu=0;
    #local Fila=0;
    #while(Fila<dimension_size(A,1))
      #declare Acu=Acu+abs(A[Fila][Col]);
      #declare Fila=Fila+1;
    #end
    #declare Max=max(Max,Acu);
    #declare Col=Col+1;
  #end
  Max
#end

#macro RangoM(A, Epsilon)  // Epsilon  o menor valor absoluto que h-de superar um elemento da matriz
                               // para n ser tratado como zero no clculo do rango.
  #local lA=A
  #local Conta=0;
  #local Coluna=0;
  #while((Coluna<dimension_size(A,2))&(Conta<dimension_size(A,1)))
    #local HaPivote=false;
    #local Fila=Conta;
    #while(Fila<dimension_size(A,1))
      #if(HaPivote)
        #local Factor=-lA[Fila][Coluna]/lA[Conta][Coluna];
        #local Col=Coluna;
        #while(Col<dimension_size(A,2))
          #declare lA[Fila][Col]=lA[Fila][Col]+Factor*lA[Conta][Col];
          #declare Col=Col+1;
        #end
      #else
        #if(abs(lA[Fila][Coluna])>Epsilon)
          #declare HaPivote=true;
          #if(Fila!=Conta)
            #local Col=Coluna;
            #while(Col<dimension_size(A,2))
              #local Acu=lA[Conta][Col];
              #declare lA[Conta][Col]=lA[Fila][Col];
              #declare lA[Fila][Col]=Acu;
              #declare Col=Col+1;
            #end
          #end
        #end
      #end
      #declare Fila=Fila+1;
    #end
    #if(HaPivote)
      #declare Conta=Conta+1;
    #end
    #declare Coluna=Coluna+1;
  #end
  Conta
#end

#macro CondCols(A)
  #if(dimension_size(A,1)!=dimension_size(A,2))
    #error "As matrizes rectangulares no tenhem numero de condico de colunas."
  #end
  #local Resultado=NrmCols(A)*NrmCols(MInversa(A));
  Resultado
#end

#macro CondFilas(A)
  #if(dimension_size(A,1)!=dimension_size(A,2))
    #error "As matrizes rectangulares no tenhem numero de condico de filas."
  #end
  #local Resultado=NrmFilas(A)*NrmFilas(MInversa(A));
  Resultado
#end

#macro Transposta(A)
  #local Rasulty=array[dimension_size(A,2)][dimension_size(A,1)]
  #local Fila=0;
  #while(Fila<dimension_size(A,1))
    #local Coluna=0;
    #while(Coluna<dimension_size(A,2))
      #declare Rasulty[Coluna][Fila]=A[Fila][Coluna];
      #declare Coluna=Coluna+1;
    #end
    #declare Fila=Fila+1;
  #end
  Rasulty
#end

#macro MInversa(A)  // Emprega o mtodo de Gauss-Jordan.
  #if(dimension_size(A,1)=dimension_size(A,2))
    #local Ordem=dimension_size(A,1);
  #else
    #error "As matrizes rectangulares no podem ter inversa."
  #end
  #local lA=A;
  #local B=MUnidade(Ordem,Ordem)
  #local Coluna=0;
  #while(Coluna<Ordem)
    #local Fila=Coluna;
    #local HaPivote=false;
    #while(Fila<Ordem)
      #if(HaPivote)
        #if(lA[Fila][Coluna]!=0)
          #local Factor=-lA[Fila][Coluna];
          #local Col=0;
          #while(Col<Ordem)
            #if(Col>=Coluna)
              #declare lA[Fila][Col]=lA[Fila][Col]+Factor*lA[Coluna][Col];
            #end
            #declare B[Fila][Col]=B[Fila][Col]+Factor*B[Coluna][Col];
            #declare Col=Col+1;
          #end
        #end
      #else
        #if(lA[Fila][Coluna]!=0)
          #declare HaPivote=true;
          #declare Factor=1/lA[Fila][Coluna];
          #if(Fila!=Coluna)
            #local Col=0;
            #while(Col<Ordem)
              #if(Col>=Coluna)
                #local Elmt=Factor*lA[Fila][Col];
                #declare lA[Fila][Col]=lA[Coluna][Col];
                #declare lA[Coluna][Col]=Elmt;
              #end
              #local Elmt=Factor*B[Fila][Col];
              #declare B[Fila][Col]=B[Coluna][Col];
              #declare B[Coluna][Col]=Elmt;
              #declare Col=Col+1;
            #end
          #else
            #local Col=0;
            #while(Col<Ordem)
              #if(Col>=Coluna)
                #declare lA[Fila][Col]=Factor*lA[Fila][Col];
              #end
              #declare B[Fila][Col]=Factor*B[Fila][Col];
              #declare Col=Col+1;
            #end
          #end
        #end
      #end
      #declare Fila=Fila+1;
    #end
    #if(!HaPivote)
      #error "A matriz dada e singular: no possui inversa."
    #end
    #declare Coluna=Coluna+1;
  #end
  #local Coluna=Ordem-1;
  #while(Coluna>0)
    #local Fila=Coluna-1;
    #while(Fila>=0)
      #local Factor=-lA[Fila][Coluna];
      #local Col=0;
      #while(Col<Ordem)
        #declare B[Fila][Col]=B[Fila][Col]+Factor*B[Coluna][Col];
        #declare Col=Col+1;
      #end
      #declare Fila=Fila-1;
    #end
    #declare Coluna=Coluna-1;
  #end
  B
#end


//  Inversa aproximada da matriz quadrada A, por uma tentativa de generalizar o mtodo de
// Newton-Raphson. Se Proj=false, empregar-se- Estim0 como primeira aproxima da inversa procurada;
// se Proj=true, essa primeira estima ser a matriz diagonal com os inversos dos elementos diagonais de
// A; se algum desses elementos for nulo, a estima inicial conservar um zero no seu lugar. Se a matriz
// A for singular ou rectangular, a macro procurar uma sua possvel pseudo-inversa; em tal caso, a esti-
// ma inicial dever ter as mesmas dimenses que a TRANSPOSTA de A
//  O processo continuar at o erro estimado ser menor ou igual que Epsilon, ficando completas n menos
// de IndiceMin iteraes. Se o erro estimado superar nalguma itera CoutaDivg, a macro supor divergen-
// te a sucess de Newton-Raphson, e deter o clculo.

#macro MInversaNewtonRaphson(Epsilon, CoutaDivg, IndiceMin, A, Estim0, Proj)
  #if(Proj)
    #local Orbit=Transposta(A)
    #local Fila=0;
    #while(Fila<dimension_size(A,2))
      #local Col=0;
      #while(Col<dimension_size(A,1))
        #if(Col=Fila)
          #local Darjoum=Orbit[Fila][Col];
          #if(Darjoum!=0)
            #declare Orbit[Fila][Col]=1/Darjoum;
          #end
        #else
          #declare Orbit[Fila][Col]=0;
        #end
        #declare Col=Col+1;
      #end
      #declare Fila=Fila+1;
    #end
  #else
    #local Orbit=Estim0
  #end
  #local Buffer=array[4]
  #local OrbitVelho=Orbit
  #local Indice=1;
  #local Continua=true;
  #while(Continua)
    #local Lanc=ProdM(Orbit,ProdM(A,Orbit))
    #declare Orbit=SomaM(ProdEscM(2,Orbit),ProdEscM(-1,Lanc))
    #declare Buffer[mod(Indice,4)]=NrmFrobenius(SomaM(Orbit,ProdEscM(-1,OrbitVelho)));
    #declare Indice=Indice+1;
    #if(Indice>4)
      #local OMeuDedo=0;
      #local Vulture=0;
      #while(OMeuDedo<4)
        #declare Vulture=Vulture+Buffer[OMeuDedo];
        #declare OMeuDedo=OMeuDedo+1;
      #end
    #else
      #local Vulture=Buffer[Indice-1];
    #end
    #if(Vulture>CoutaDivg)
      #declare Continua=false;
      #warning "A extenso do algoritmo de Newton-Raphson empregada para procurar a inversa ou pseudo-inversa da matriz dada parece divergir com a estimaco inicial fornecida."
      #warning "Devolver-se-a o derradeiro iterante calculado."
    #else
      #declare Continua=((Vulture>Epsilon)|(Indice<IndiceMin));
    #end
    #declare OrbitVelho=Orbit
  #end
  Orbit
#end

#macro MPseudoInversa(A, Epsilon)  // As colunas de A cuja norma eucldea for menor ou igual que Epsilon,
                                      // tratar-se-m como nulas.
  #if(NrmFrobenius(A)<=Epsilon)
    #local Resultado=MNula(dimension_size(A,1),dimension_size(A,2))
  #else
    #local Q=0; #local R=0;
    FactQR(A,Q,R,Epsilon)
    #local Resultado=ProdM(Transposta(R),ProdM(MInversa(ProdM(R,Transposta(R))),Transposta(Q)))
  #end
  Resultado
#end

#macro AvalPolM(Polinomio, A)  // Avalia o polinomio dado, na forma duma tbua unidimensional com os seus
                                      // coeficientes nos lugares indicados polas suas ordens, na matriz A
  #if(dimension_size(A,1)!=dimension_size(A,2))
    #error "No se pode avaliar um polinomio numa matriz rectangular."
  #end
  #local Ordem=dimension_size(A,1);
  #local Acu=MEscalar(Polinomio[dimension_size(Polinomio,1)-1],Ordem,Ordem)
  #local Indice=dimension_size(Polinomio,1)-2;  
  #while(Indice>=0)
    #declare Acu=ProdM(Acu,A)
    #declare Acu=SomaM(Acu,MEscalar(Polinomio[Indice],Ordem,Ordem))
    #declare Indice=Indice-1;
  #end
  Acu
#end

//  Esta macro obtm os coeficientes do polinmio caracterstico da matriz dada, na forma duma tbua uni-
// dimensional, com cada coeficiente no lugar de ndice igual  sua ordem.  uma optimiza de
// PolinomioDet(A,B) para B = - I
#macro PolCarM(A)
  #if(dimension_size(A,1)!=dimension_size(A,2))
    #error "No existe o polinomio caracteristico duma matriz rectangular."
  #else
    #local Ordem=dimension_size(A,1);
  #end
  #switch(Ordem)
    #case(1)
      #local Resultado=array[2]{A[0][0], -1}
    #break
    #case(2)
      #local Resultado=array[3]{A[0][0]*A[1][1]-A[1][0]*A[0][1], -A[0][0]-A[1][1], 1}
    #break
    #case(3)
      #local Resultado=array[4]{
        A[0][0]*A[1][1]*A[2][2] + A[1][0]*A[2][1]*A[0][2] + A[2][0]*A[0][1]*A[1][2]
          - A[1][0]*A[0][1]*A[2][2] - A[0][0]*A[2][1]*A[1][2] - A[2][0]*A[1][1]*A[0][2] ,
        -A[0][0]*A[1][1]+A[1][0]*A[0][1] - A[0][0]*A[2][2]+A[2][0]*A[0][2]
          - A[1][1]*A[2][2]+A[2][1]*A[1][2] ,
        A[0][0]+A[1][1]+A[2][2] ,
        -1
      }
    #break
  #else
    #local Resultado=array[Ordem+1]
    #local A1=A
    #local B1=SuprCols(MEscalar(-1,Ordem,Ordem),array[1]{0})
    #local Fila=0;
    #local Pivote=-1;
    #while(Fila<Ordem)
      #if(Pivote=-1)
        #if(A1[Fila][0]!=0)
          #declare Pivote=Fila;
          #local Alpha=A1[Fila][0];
        #end
      #else
        #local Factor=-A1[Fila][0]/Alpha;
        SomaFilaMul(A1,Fila,Pivote,Factor)
        #if(Pivote!=0)
          #declare B1[Fila][Pivote-1]=B1[Fila][Pivote-1]-Factor;
        #end
      #end
      #declare Fila=Fila+1;
    #end
    #if(Pivote=-1)
      #local Conta=0;
      #while(Conta<=Ordem)
        #declare Resultado[Conta]=0;
        #declare Conta=Conta+1;
      #end
    #else
      #declare A1=SuprFilasECols(A1, array[1]{Pivote}, array[1]{0})
      #declare B1=SuprFilas(B1, array[1]{Pivote})
      #declare Alpha=pow(-1,Pivote)*Alpha;
      #local Guar=PolinomioDet(A1,B1)
      #local Conta=0;
      #while(Conta<Ordem)
        #declare Resultado[Conta]=Alpha*Guar[Conta];
        #declare Conta=Conta+1;
      #end
      #declare Resultado[Ordem]=0;
    #end
    #local A2=SuprFilasECols(A, array[1]{0}, array[1]{0})
    #local Guar=PolCarM(A2)
    #local Conta=1;
    #while(Conta<=Ordem)
      #declare Resultado[Conta]=Resultado[Conta]-Guar[Conta-1];
      #declare Conta=Conta+1;
    #end
  #end
  Resultado
#end


//  A macro PolinomioDet(A, B) obtm os coeficientes do polinmio det ( A + x B ), onde A e B s duas
// matrizes quadradas fornecidas, de dimenses n  n. Repare-se em que:
//                                         -1
//    det ( A + x B ) = det ( B ) det ( A B  + x I ) = - det ( B ) q     ( x )
//                                                                      -1
//                                                                  - A B
// para toda B regular; se A for regular cumprir-se-, para toda B:
//                                               -1
//    det ( A + x B ) = det ( A ) det ( I + x B A  )
//
// Em x = 0, isto  igual a det ( A ); para todo x n nulo:
//                             -1                                 -1
//    det ( A ) det ( I + x B A  ) = det ( A ) x det ( 1/x I + B A  ) = - det ( A ) x q     ( 1/x )
//                                                                                         -1
//                                                                                     - B A

#macro PolinomioDet(A, B)
  #if(
    (dimension_size(A,1)!=dimension_size(A,2)) |
    (dimension_size(B,1)!=dimension_size(B,2)) |
    (dimension_size(A,1)!=dimension_size(B,1))
  )
    #error "Os argumentos da macro PolinomioDet(A, B) devem ser matrizes quadradas do mesmo tamanho."
  #end
  #local Ordem=dimension_size(A,1);
  #switch(Ordem)
    //  Cada coeficiente  a soma de todos os determinantes que se podem armar substituindo um nmero de
    // colunas de A igual ao seu ndice, polas correspondentes de B.
    #case(1)
      #local Poly=array[2]{A[0][0], B[0][0]}
    #break
    #case(2)
      #local Poly=array[3]{A[0][0]*A[1][1]-A[1][0]*A[0][1], A[0][0]*B[1][1]-A[1][0]*B[0][1] + B[0][0]*A[1][1]-B[1][0]*A[0][1], B[0][0]*B[1][1]-B[1][0]*B[0][1]}
    #break
    #case(3)
      #local Poly=array[4]{
        A[0][0]*A[1][1]*A[2][2] + A[1][0]*A[2][1]*A[0][2] + A[0][1]*A[1][2]*A[2][0]
          - A[2][0]*A[1][1]*A[0][2] - A[0][0]*A[2][1]*A[1][2] - A[1][0]*A[0][1]*A[2][2] ,
        A[0][0]*A[1][1]*B[2][2] + A[1][0]*A[2][1]*B[0][2] + A[0][1]*B[1][2]*A[2][0]
          - A[2][0]*A[1][1]*B[0][2] - A[0][0]*A[2][1]*B[1][2] - A[1][0]*A[0][1]*B[2][2] +
        A[0][0]*B[1][1]*A[2][2] + A[1][0]*B[2][1]*A[0][2] + B[0][1]*A[1][2]*A[2][0]
          - A[2][0]*B[1][1]*A[0][2] - A[0][0]*B[2][1]*A[1][2] - A[1][0]*B[0][1]*A[2][2] +
        B[0][0]*A[1][1]*A[2][2] + B[1][0]*A[2][1]*A[0][2] + A[0][1]*A[1][2]*B[2][0]
          - B[2][0]*A[1][1]*A[0][2] - B[0][0]*A[2][1]*A[1][2] - B[1][0]*A[0][1]*A[2][2] ,
        A[0][0]*B[1][1]*B[2][2] + A[1][0]*B[2][1]*B[0][2] + B[0][1]*B[1][2]*A[2][0]
          - A[2][0]*B[1][1]*B[0][2] - A[0][0]*B[2][1]*B[1][2] - A[1][0]*B[0][1]*B[2][2] +
        B[0][0]*B[1][1]*A[2][2] + B[1][0]*B[2][1]*A[0][2] + B[0][1]*A[1][2]*B[2][0]
          - B[2][0]*B[1][1]*A[0][2] - B[0][0]*B[2][1]*A[1][2] - B[1][0]*B[0][1]*A[2][2] +
        B[0][0]*A[1][1]*B[2][2] + B[1][0]*A[2][1]*B[0][2] + A[0][1]*B[1][2]*B[2][0]
          - B[2][0]*A[1][1]*B[0][2] - B[0][0]*A[2][1]*B[1][2] - B[1][0]*A[0][1]*B[2][2] ,
        B[0][0]*B[1][1]*B[2][2] + B[1][0]*B[2][1]*B[0][2] + B[0][1]*B[1][2]*B[2][0]
          - B[2][0]*B[1][1]*B[0][2] - B[0][0]*B[2][1]*B[1][2] - B[1][0]*B[0][1]*B[2][2]
      }
    #break
  #else
    //  Calcula-se det ( A + x B ) como a soma de dous determinantes, um dos quais tem como primeira
    // coluna a de A, e o outro, a de x B, aplicando o mtodo de Gauss para que cada um deles fique de
    // novo como um mltiplo do determinante dum binmio matricial, e repetindo o processo at a ordem
    // dos determinantes envoltos no clculo se reduzir a 3.
    #local Poly=array[Ordem+1]
    #local A1=A
    #local A2=SuprCols(A,array[1]{0})
    #local B1=SuprCols(B,array[1]{0})
    #local B2=B
    #local Fila=0;
    #local Pivote=-1;
    #while(Fila<Ordem)
      #if(Pivote=-1)
        #if(A1[Fila][0]!=0)
          #declare Pivote=Fila;
          #local Alpha=A1[Fila][0];
        #end
      #else
        #local Factor=-A1[Fila][0]/Alpha;
        SomaFilaMul(A1,Fila,Pivote,Factor)
        SomaFilaMul(B1,Fila,Pivote,Factor)
      #end
      #declare Fila=Fila+1;
    #end
    #if(Pivote=-1)
      #local Conta=0;
      #while(Conta<=Ordem)
        #declare Poly[Conta]=0;
        #declare Conta=Conta+1;
      #end
    #else
      #declare A1=SuprFilasECols(A1, array[1]{Pivote}, array[1]{0})
      #declare B1=SuprFilas(B1, array[1]{Pivote})
      #declare Alpha=pow(-1,Pivote)*Alpha;
      #local Guar=PolinomioDet(A1,B1)
      #local Conta=0;
      #while(Conta<Ordem)
        #declare Poly[Conta]=Alpha*Guar[Conta];
        #declare Conta=Conta+1;
      #end
      #declare Poly[Ordem]=0;
    #end
    #local Fila=0;
    #local Pivote=-1;
    #while(Fila<Ordem)
      #if(Pivote=-1)
        #if(B2[Fila][0]!=0)
          #declare Pivote=Fila;
          #local Beta=B2[Fila][0];
        #end
      #else
        #local Factor=-B2[Fila][0]/Beta;
        SomaFilaMul(B2,Fila,Pivote,Factor)
        SomaFilaMul(A2,Fila,Pivote,Factor)
      #end
      #declare Fila=Fila+1;
    #end
    #if(Pivote!=-1)
      #declare B2=SuprFilasECols(B2, array[1]{Pivote}, array[1]{0})
      #declare A2=SuprFilas(A2, array[1]{Pivote})
      #declare Beta=pow(-1,Pivote)*Beta;
      #local Guar=PolinomioDet(A2,B2)
      #local Conta=1;
      #while(Conta<=Ordem)
        #declare Poly[Conta]=Poly[Conta]+Beta*Guar[Conta-1];
        #declare Conta=Conta+1;
      #end
    #end
  #end
  Poly
#end


//------------------------------ Funes escalares avaliadas nas matrizes quadradas ---------------------------------------------------------

//  As seguintes macros, ags as que calculam o logaritmo neperiano, aproximam a fun correspondente atra-
// vs da sua srie de Taylor centrada na matriz escalar mais achegada a respeito da norma de Frobenius
// ao argumento, desde que chega avaliar a sucess polinmica assim definida na matriz. Embora n seja tam
// trivial, poder-se-ia utilizar uma srie centrada na melhor aproxima diagonal claramente, a matriz com
// os mesmos elementos diagonais, e os restantes zerados da matriz dada, e j que logo, melhorar a
// convergncia?
//  Doutra banda, talvez se possa tirar mais sumo do mtodo de Newton-Raphson. Cumpre investigar ;)

#macro ExpM(Epsilon, IndiceMin, A)  // Exponencial aproximada da matriz quadrada A
  #if(dimension_size(A,1)!=dimension_size(A,2))
    #error "As matrizes rectangulares no tenhem exponencial."
  #end
  #macro MatXos_Sucesso(Indice) 1 #end
  #local Ordem=dimension_size(A,1);
  #local Proj=Traca(A)/Ordem;     // Acelera a convergncia centrando a srie de Taylor na projec
  #local Acum=SomaM(A,MEscalar(-Proj,Ordem,Ordem))  // de A sobre o subespao das matrizes escalares.                                         
  #declare Acum=ProdEscM(exp(Proj),SomaAproxTaylorM(Epsilon, 2, IndiceMin, Acum))
  Acum
#end

//  Logaritmo neperiano da matriz regular A, aproximado polo seu desenvolvimento em srie de Taylor
// arredor da matriz escalar julgada com menor probabilidade de gerar uma srie divergente.
#macro LnM(Epsilon, IndiceMin, A)
  #if(dimension_size(A,1)!=dimension_size(A,2))
    #error "As matrizes rectangulares no tenhem logaritmo neperiano."
  #end
  #local Ordem=dimension_size(A,1);
  #if(RangoM(A,Epsilon)<Ordem)
    #error "As matrizes singulares no possuem logaritmo neperiano."
  #end
  #macro MatXos_Sucesso(Indice)
    #if(Indice=0)
      #local Coef=ln(Sombra);
    #else
      //#local Coef=pow(-1,Indice-1)/Indice;
      #local Coef=((mod(Indice,2)=0)? -1: 1)/Indice;
    #end
    Coef
  #end
  #local Sombra=Traca(ProdM(A,Transposta(A)))/Traca(A);
  #local Acu=SomaM(ProdEscM(1/Sombra,A),MEscalar(-1,Ordem,Ordem))
  #local Norma=min(NrmFilas(Acu),NrmCols(Acu),NrmFrobenius(Acu));
  #if((Norma<1)&(Sombra>0))
    #declare Acu=SomaAproxPotsM(Epsilon, 4, IndiceMin, Acu)
  #else
    #local Invi=MInversa(A)
    #local SombraBis=Traca(ProdM(Invi,Transposta(Invi)))/Traca(Invi);
    #local AcuBis=SomaM(ProdEscM(1/SombraBis,Invi),MEscalar(-1,Ordem,Ordem))
    #undef Invi
    #local NormaBis=min(NrmFilas(AcuBis),NrmCols(AcuBis),NrmFrobenius(AcuBis));
    #if((NormaBis<1)&(SombraBis>0))
      #declare Sombra=SombraBis;
      #declare Acu=SomaAproxPotsM(Epsilon, 4, IndiceMin, AcuBis)
      #undef AcuBis
      #declare Acu=ProdEscM(-1,Acu)
    #else
      #if((Sombra<0)&(SombraBis<0))
        #error "O desenvolvimento em serie de Taylor no fornece o logaritmo neperiano da matriz dada."
      #end
      #warning "O desenvolvimento em serie de Taylor do logaritmo neperiano pode no convergir na matriz dada. Calcular-se-a apenas a soma dos IndiceMin primeiros termos."
      #if(((Norma>NormaBis)&(SombraBis>0))|(Sombra<0))
        #declare Sombra=SombraBis;
        #declare Acu=SomaParcialPotsM(IndiceMin, AcuBis)
        #undef AcuBis
        #declare Acu=ProdEscM(-1,Acu)
      #else
        #undef AcuBis
        #declare Acu=SomaParcialPotsM(IndiceMin, Acu)
      #end
    #end
  #end
  Acu
#end

//  Logaritmo neperiano aproximado da matriz quadrada A, por uma tentativa de generalizar o mtodo de
// Newton-Raphson. Se Proj=false, empregar-se- Estim0 como primeira aproxima do logaritmo procurado;
// se Proj=true, essa primeira estima ser a matriz diagonal com os logaritmos neperianos dos valores
// absolutos dos elementos diagonais de A; se algum desses elementos for nulo, a estima inicial portar
// um 1 no seu lugar.
//  O processo continuar at o erro estimado ser menor ou igual que Epsilon, ficando completas n menos
// de IndiceMin iteraes. Se o erro estimado superar nalguma itera CoutaDivg, a macro supor divergen-
// te a sucess de Newton-Raphson, e deter o clculo.

#macro LnMNewtonRaphson(Epsilon, CoutaDivg, IndiceMin, A, Estim0, Proj)
  #if(dimension_size(A,1)!=dimension_size(A,2))
    #error "As matrizes rectangulares no tenhem logaritmo neperiano."
  #end
  #local Ordem=dimension_size(A,1);
  #if(RangoM(A,Epsilon)<Ordem)
    #error "As matrizes singulares no possuem logaritmo neperiano."
  #end
  #if(Proj)
    #local Orbit=A
    #local Fila=0;
    #while(Fila<Ordem)
      #local Col=0;
      #while(Col<Ordem)
        #if(Col=Fila)
          #local Darjoum=Orbit[Fila][Col];
          #if(Darjoum=0)
            #declare Orbit[Fila][Col]=1;
          #else
            #declare Orbit[Fila][Col]=ln(abs(Darjoum));
          #end
        #else
          #declare Orbit[Fila][Col]=0;
        #end
        #declare Col=Col+1;
      #end
      #declare Fila=Fila+1;
    #end
  #else
    #local Orbit=Estim0
  #end
  #local Buffer=array[4]
  #local OrbitVelho=Orbit
  #local Indice=1;
  #local Continua=true;
  #while(Continua)
    #local Tortus=ExpM(Epsilon,IndiceMin,ProdEscM(-1/2,Orbit))
    #declare Orbit=SomaM(Orbit,MEscalar(-1,Ordem,Ordem))
    #declare Orbit=SomaM(Orbit,ProdM(Tortus,ProdM(A,Tortus)))
    #declare Buffer[mod(Indice,4)]=NrmFrobenius(SomaM(Orbit,ProdEscM(-1,OrbitVelho)));
    #declare Indice=Indice+1;
    #if(Indice>4)
      #local OMeuDedo=0;
      #local Vulture=0;
      #while(OMeuDedo<4)
        #declare Vulture=Vulture+Buffer[OMeuDedo];
        #declare OMeuDedo=OMeuDedo+1;
      #end
    #else
      #local Vulture=Buffer[Indice-1];
    #end
    #if(Vulture>CoutaDivg)
      #declare Continua=false;
      #warning "A extenso do algoritmo de Newton-Raphson empregada para procurar o logaritmo neperiano da matriz dada parece divergir com a estimaco inicial fornecida."
      #warning "Devolver-se-a o derradeiro iterante calculado."
    #else
      #declare Continua=((Vulture>Epsilon)|(Indice<IndiceMin));
    #end
    #declare OrbitVelho=Orbit
  #end
  Orbit
#end

#macro SenM(Epsilon, IndiceMin, A)  // Seno aproximado da matriz quadrada A
  #if(dimension_size(A,1)!=dimension_size(A,2))
    #error "As matrizes rectangulares no tenhem seno."
  #end
  #local Ordem=dimension_size(A,1);
  #local Proj=Traca(A)/Ordem;     // Acelera a convergncia centrando a srie de Taylor na projec
  #macro MatXos_Sucesso(Indice)  // de A sobre o subespao das matrizes escalares.
    #if(mod(Indice,2)=0)
      #local Coef=sin(Proj)*((mod(Indice/2,2)=0)? 1: -1);
    #else
      #local Coef=cos(Proj)*((mod((Indice-1)/2,2)=0)? 1: -1);
    #end
    Coef
  #end
  #local Acum=SomaM(A,MEscalar(-Proj,Ordem,Ordem))                                         
  #declare Acum=SomaAproxTaylorM(Epsilon, 4, IndiceMin, Acum)
  Acum
#end

#macro CosM(Epsilon, IndiceMin, A)  // Cosseno aproximado da matriz quadrada A
  #if(dimension_size(A,1)!=dimension_size(A,2))
    #error "As matrizes rectangulares no tenhem cosseno."
  #end
  #local Ordem=dimension_size(A,1);
  #local Proj=Traca(A)/Ordem;     // Acelera a convergncia centrando a srie de Taylor na projec
  #macro MatXos_Sucesso(Indice)  // de A sobre o subespao das matrizes escalares.
    #if(mod(Indice,2)=0)
      #local Coef=cos(Proj)*((mod(Indice/2,2)=0)? 1: -1);
    #else
      #local Coef=sin(Proj)*((mod((Indice-1)/2,2)=0)? -1: 1);
    #end
    Coef
  #end
  #local Acum=SomaM(A,MEscalar(-Proj,Ordem,Ordem))                                         
  #declare Acum=SomaAproxTaylorM(Epsilon, 4, IndiceMin, Acum)
  Acum
#end

#macro ShM(Epsilon, IndiceMin, A)  // Seno hiperblico aproximado da matriz quadrada A
  #if(dimension_size(A,1)!=dimension_size(A,2))
    #error "As matrizes rectangulares no tenhem seno hiperbolico."
  #end
  #local Ordem=dimension_size(A,1);
  #local Proj=Traca(A)/Ordem;     // Acelera a convergncia centrando a srie de Taylor na projec
  #macro MatXos_Sucesso(Indice)  // de A sobre o subespao das matrizes escalares.
    #if(mod(Indice,2)=0)
      #local Coef=sinh(Proj);
    #else
      #local Coef=cosh(Proj);
    #end
    Coef
  #end
  #local Acum=SomaM(A,MEscalar(-Proj,Ordem,Ordem))                                         
  #declare Acum=SomaAproxTaylorM(Epsilon, 4, IndiceMin, Acum)
  Acum
#end

#macro ChM(Epsilon, IndiceMin, A)  // Cosseno hiperblico aproximado da matriz quadrada A
  #if(dimension_size(A,1)!=dimension_size(A,2))
    #error "As matrizes rectangulares no tenhem cosseno hiperbolico."
  #end
  #local Ordem=dimension_size(A,1);
  #local Proj=Traca(A)/Ordem;     // Acelera a convergncia centrando a srie de Taylor na projec
  #macro MatXos_Sucesso(Indice)  // de A sobre o subespao das matrizes escalares.
    #if(mod(Indice,2)=0)
      #local Coef=cosh(Proj);
    #else
      #local Coef=sinh(Proj);
    #end
    Coef
  #end
  #local Acum=SomaM(A,MEscalar(-Proj,Ordem,Ordem))                                         
  #declare Acum=SomaAproxTaylorM(Epsilon, 4, IndiceMin, Acum)
  Acum
#end


//----------------------------------------------- Factorizaes -----------------------------------------------------------------------------

#macro FactLU(A, L, U)// Obtm a factoriza LU da matriz quadrada A, e guarda os factores nas
                                   // variveis fornecidas atravs dos parmetros L e U
                                   // Se a factoriza n for possvel, devolver a matriz nula no canto
                                   // da L
  #if(dimension_size(A,1)!=dimension_size(A,2))
    #error "As matrizes rectangulares no admitem factorizaco LU."
  #end
  #local Ordem=dimension_size(A,1);
  #declare L=MUnidade(Ordem,Ordem)
  #declare U=A
  #local Coluna=0;
  #while(Coluna<Ordem-1)
    #if(U[Coluna][Coluna]=0)  // A n admite factoriza LU.
      #declare L=MNula(Ordem,Ordem)
      #declare Coluna=Ordem;
    #else
      #local Fila=Coluna+1;
      #while(Fila<Ordem)
        #local Factor=-U[Fila][Coluna]/U[Coluna][Coluna];
        #local Conta=Coluna;
        #while(Conta<Ordem)
          #declare U[Fila][Conta]=U[Fila][Conta]+Factor*U[Coluna][Conta];
          #declare Conta=Conta+1;
        #end
        SomaFilaMul(L,Fila,Coluna,Factor)
        #declare Fila=Fila+1;
      #end
      #declare Coluna=Coluna+1;
    #end
  #end
  #if(Coluna<Ordem)
    #declare L=MInversa(L)
  #end
#end

#macro FactCholeski(A, L)  // Obtm o factor de Choleski da matriz simtrica              t
                               //                                                   ( A + A ) / 2
                               // de jeito que
                               //                                t
                               //                       A = L  L
                               // Se a factoriza n for possvel, devolver a matriz nula no canto da L
  #if(dimension_size(A,1)!=dimension_size(A,2))
    #error "As matrizes rectangulares no admitem factorizaco de Choleski."
  #end
  #local Ordem=dimension_size(A,1);
  #declare L=MUnidade(Ordem,Ordem)
  #local Acu=ProdEscM(1/2,SomaM(A,Transposta(A)))
  #local Coluna=0;
  #while(Coluna<Ordem)
    #if(Acu[Coluna][Coluna]<=0)
      #declare L=MNula(Ordem,Ordem)
      #declare Coluna=Ordem+1;
    #else
      #local Factor=1/sqrt(Acu[Coluna][Coluna]);
      #declare Acu[Coluna][Coluna]=1;
      #local Conta=Coluna+1;
      #while(Conta<Ordem)
        #declare Acu[Coluna][Conta]=Factor*Acu[Coluna][Conta];
        #declare Acu[Conta][Coluna]=Factor*Acu[Conta][Coluna];
        #declare Conta=Conta+1;
      #end
      MulFila(L,Coluna,Factor)
      #local Fila=Coluna+1;
      #while(Fila<Ordem)
        #local Factor=-Acu[Fila][Coluna];
        #local Conta=Coluna;
        #while(Conta<Ordem)
          #declare Acu[Fila][Conta]=Acu[Fila][Conta]+Factor*Acu[Coluna][Conta];
          #declare Conta=Conta+1;
        #end
        SomaFilaMul(L,Fila,Coluna,Factor)
        #local Conta=Coluna;
        #while(Conta<Ordem)
          #declare Acu[Conta][Fila]=Acu[Conta][Fila]+Factor*Acu[Conta][Coluna];
          #declare Conta=Conta+1;
        #end
        #declare Fila=Fila+1;
      #end
      #declare Coluna=Coluna+1;
    #end
  #end
  #if(Coluna<Ordem+1)
    #declare L=MInversa(L)
  #end
#end

#macro FactQR(A, Q, R, Epsilon)  // Obtm a factoriza QR da matriz A, e guarda os factores nas
                                   // variveis fornecidas atravs dos parmetros Q e R; Epsilon
                                   //  o mnimo valor absoluto que haver de atingir um elemento
                                   // de A para no ser considerado nulo no clculo do rango, que
                                   // coincidir com o nmero de colunas de Q e de filas de R
  #declare Q=GrammSchmidtM(A,Epsilon)
  #declare R=ProdM(Transposta(Q),A)
#end

//-------------------------------------------- Altera de matrizes -------------------------------------------------------------------------

#macro FilaM(A,Fila)  // Extrai a fila indicada da matriz A, na forma duma tbua unidimensional.
  #local Resultado=array[dimension_size(A,2)]
  #local Conta=0;
  #while(Conta<dimension_size(A,2))
    #declare Resultado[Conta]=A[Fila][Conta];
    #declare Conta=Conta+1;
  #end
  Resultado
#end

#macro SuprFilas(A, Filas)  // Suprime as filas indicadas mediante uma tbua unidimensional.
  #local Resultado=array[dimension_size(A,1)-dimension_size(Filas,1)][dimension_size(A,2)]
  #local Fila=0;
  #local QuantasVamSuprimidas=0;
  #while(Fila<dimension_size(A,1))
    #local EstaSim=true;
    #local Conta=0;
    #while(Conta<dimension_size(Filas,1))
      #if(Filas[Conta]=Fila)
        #declare EstaSim=false;
        #declare QuantasVamSuprimidas=QuantasVamSuprimidas+1;
      #end
      #declare Conta=Conta+1;
    #end
    #if(EstaSim)
      #local Col=0;
      #while(Col<dimension_size(A,2))
        #declare Resultado[Fila-QuantasVamSuprimidas][Col]=A[Fila][Col];
        #declare Col=Col+1;
      #end
    #end
    #declare Fila=Fila+1;
  #end
  Resultado
#end

#macro ColM(A,Col)  // Extrai a coluna indicada da matriz A, na forma duma tbua unidimensional.
  #local Resultado=array[dimension_size(A,1)]
  #local Conta=0;
  #while(Conta<dimension_size(A,1))
    #declare Resultado[Conta]=A[Conta][Col];
    #declare Conta=Conta+1;
  #end
  Resultado
#end

#macro SuprCols(A, Colunas)  // Suprime as colunas indicadas mediante uma tbua unidimensional.
  #local Resultado=array[dimension_size(A,1)][dimension_size(A,2)-dimension_size(Colunas,1)]
  #local Col=0;
  #local QuantasVamSuprimidas=0;
  #while(Col<dimension_size(A,2))
    #local EstaSim=true;
    #local Conta=0;
    #while(Conta<dimension_size(Colunas,1))
      #if(Colunas[Conta]=Col)
        #declare EstaSim=false;
        #declare QuantasVamSuprimidas=QuantasVamSuprimidas+1;
      #end
      #declare Conta=Conta+1;
    #end
    #if(EstaSim)
      #local Fila=0;
      #while(Fila<dimension_size(A,1))
        #declare Resultado[Fila][Col-QuantasVamSuprimidas]=A[Fila][Col];
        #declare Fila=Fila+1;
      #end
    #end
    #declare Col=Col+1;
  #end
  Resultado
#end

#macro SuprFilasECols(A, Filas, Colunas)  // Suprime as filas e colunas indicadas mediante senlhas tbuas uni-
                                          // dimensionais.
  #local Resultado=array[dimension_size(A,1)-dimension_size(Filas,1)][dimension_size(A,2)-dimension_size(Colunas,1)]
  #local Fila=0;
  #local QuantasFilasSuprimidas=0;
  #while(Fila<dimension_size(A,1))
    #local EstaSim=true;
    #local Conta=0;
    #while(Conta<dimension_size(Filas,1))
      #if(Filas[Conta]=Fila)
        #declare EstaSim=false;
        #declare QuantasFilasSuprimidas=QuantasFilasSuprimidas+1;
      #end
      #declare Conta=Conta+1;
    #end
    #if(EstaSim)
      #local Col=0;
      #local QuantasColsSuprimidas=0;
      #while(Col<dimension_size(A,2))
        #local EstaSim=true;
        #local Conta=0;
        #while(Conta<dimension_size(Colunas,1))
          #if(Colunas[Conta]=Col)
            #declare EstaSim=false;
            #declare QuantasColsSuprimidas=QuantasColsSuprimidas+1;
          #end
          #declare Conta=Conta+1;
        #end
        #if(EstaSim)
          #declare Resultado[Fila-QuantasFilasSuprimidas][Col-QuantasColsSuprimidas]=A[Fila][Col];
        #end
        #declare Col=Col+1;
      #end
    #end
    #declare Fila=Fila+1;
  #end
  Resultado
#end

#macro GrammSchmidtM(A,Epsilon)  // Obtem uma matriz de colunas ortonormais equivalente  dada, polo processo de
                                  // Gramm-Schmidt. Considerar-se-m nulas, no algoritmo, todas as colunas
                                  // cuja norma eucldea n superar Epsilon
  #local Resultado=A
  #local Col=0;
  #while(Col<dimension_size(Resultado,2))
    #local Cascarolo=0;
    #while(Cascarolo<Col)
      SomaColMul(Resultado,Col,Cascarolo,-ProdPonto(ColM(Resultado,Col),ColM(Resultado,Cascarolo)))
      #declare Cascarolo=Cascarolo+1;
    #end
    #local Comprimento=NrmEuclidea(ColM(Resultado,Col));
    #if(Comprimento<=Epsilon)
      #if(dimension_size(Resultado,2)=1)
        #error "No se pode ortonormalizar uma matriz nula."
      #end
      #local Supr=array[1]{Col}
      #declare Resultado=SuprCols(Resultado,Supr)
    #else
      MulCol(Resultado,Col,1/Comprimento)
      #declare Col=Col+1;
    #end
  #end
  Resultado
#end


//------------------------------------------- Operaes elementares --------------------------------------------------------------------------

#macro PermFilas(A,Fila1,Fila2)  // Permuta duas filas da matriz A
  #local Col=0;
  #while(Col<dimension_size(A,2))
    #local Acu=A[Fila1][Col];
    #declare A[Fila1][Col]=A[Fila2][Col];
    #declare A[Fila2][Col]=Acu;
    #declare Col=Col+1;
  #end
#end

#macro MulFila(A,Fila,Escalar)  // Multiplica uma fila da matriz A por um escalar.
  #local Col=0;
  #while(Col<dimension_size(A,2))
    #declare A[Fila][Col]=Escalar*A[Fila][Col];
    #declare Col=Col+1;
  #end
#end

#macro SomaFilaMul(A,FilaDest,FilaOrg,Escalar)  // Soma  fila destino da matriz A o produto do escalar
                                                     // pola fila origem.
  #local Col=0;
  #while(Col<dimension_size(A,2))
    #declare A[FilaDest][Col]=A[FilaDest][Col]+Escalar*A[FilaOrg][Col];
    #declare Col=Col+1;
  #end
#end

#macro PermCols(A,Col1,Col2)  // Permuta duas colunas da matriz A
  #local Fila=0;
  #while(Fila<dimension_size(A,1))
    #local Acu=A[Fila][Col1];
    #declare A[Fila][Col1]=A[Fila][Col2];
    #declare A[Fila][Col2]=Acu;
    #declare Fila=Fila+1;
  #end
#end

#macro MulCol(A,Col,Escalar)  // Multiplica uma coluna da matriz A por um escalar.
  #local Fila=0;
  #while(Fila<dimension_size(A,1))
    #declare A[Fila][Col]=Escalar*A[Fila][Col];
    #declare Fila=Fila+1;
  #end
#end

#macro SomaColMul(A,ColDest,ColOrg,Escalar)  // Soma  coluna destino da matriz A o produto do escalar
                                                     // pola coluna origem.
  #local Fila=0;
  #while(Fila<dimension_size(A,1))
    #declare A[Fila][ColDest]=A[Fila][ColDest]+Escalar*A[Fila][ColOrg];
    #declare Fila=Fila+1;
  #end
#end


//--------------------------------------------- Matrizes especiais ---------------------------------------------------------------------------

#macro MUnidade(NoFilas,NoCols)
  MEscalar(1,NoFilas,NoCols)
#end

#macro MEscalar(Escalar,NoFilas,NoCols)
  #declare NoFilas=int(NoFilas);
  #declare NoCols=int(NoCols);
  #if((NoFilas<=0)|(NoCols<=0))
    #error "As dimensoes da matriz devem ser numeros naturais no nulos."
  #end
  #local Resultado=array[NoFilas][NoCols]
  #local Fila=0;
  #while(Fila<NoFilas)
    #local Coluna=0;
    #while(Coluna<NoCols)
      #declare Resultado[Fila][Coluna]=((Fila=Coluna)? Escalar: 0);
      #declare Coluna=Coluna+1;
    #end
    #declare Fila=Fila+1;
  #end
  Resultado
#end

#macro MNula(NoFilas,NoCols)
  MConst(0,NoFilas,NoCols)
#end

#macro MConst(Escalar,NoFilas,NoCols)
  #declare NoFilas=int(NoFilas);
  #declare NoCols=int(NoCols);
  #if((NoFilas<=0)|(NoCols<=0))
    #error "As dimensoes da matriz devem ser numeros naturais no nulos."
  #end
  #local Resultado=array[NoFilas][NoCols]
  #local Fila=0;
  #while(Fila<NoFilas)
    #local Coluna=0;
    #while(Coluna<NoCols)
      #declare Resultado[Fila][Coluna]=Escalar;
      #declare Coluna=Coluna+1;
    #end
    #declare Fila=Fila+1;
  #end
  Resultado
#end

#macro MDiag(A)  // Obtem a matriz com a diagonal duma matriz dada, e com os restantes elementos nulos.
  #local Rasy=A
  #local Fila=0;
  #while(Fila<dimension_size(A,1))
    #local Col=0;
    #while(Col<dimension_size(A,2))
      #if(Col!=Fila)
        #declare Rasy[Fila][Col]=0;
      #end
      #declare Col=Col+1;
    #end
    #declare Fila=Fila+1;
  #end
  Rasy
#end


//-------------------------------- Sries de potncias de matrizes ---------------------------------------------------------

//   Soma parcial da srie de potncias com coeficientes dados por MatXos_Sucesso(Indice), avaliada na
// matriz quadrada introduzida
#macro SomaParcialPotsM(Indice, Matriz)
  #if(dimension_size(Matriz,1)!=dimension_size(Matriz,2))
    #error "No se pode avaliar uma serie de potencias numa matriz rectangular."
  #end
  #local Soma=MEscalar(MatXos_Sucesso(0),dimension_size(Matriz,1),dimension_size(Matriz,2))
  #local Qual=1;
  #while(Qual<=Indice)
    #local Coeficiente=MatXos_Sucesso(Qual);
    #if(Coeficiente!=0)
      #declare Soma = SomaM( Soma,
        ProdEscM( Coeficiente/abs(Coeficiente), PotM(ProdEscM(pow(abs(Coeficiente),1/Qual),Matriz),Qual) )
      )
    #end
    #declare Qual=Qual+1;
  #end
  Soma
#end

//   Soma da srie de potncias com coeficientes dados pola macro MatXos_Sucesso(Indice), avaliada na
// matriz quadrada introduzida, e aproximada pola primeira soma parcial tal que a soma das normas de
// Frobenius dos seus Margem derradeiros termos for menor ou igual que Epsilon, abrangendo n menos de
// IndiceMin termos
#macro SomaAproxPotsM(Epsilon, Margem, IndiceMin, Matriz)
  #if(dimension_size(Matriz,1)!=dimension_size(Matriz,2))
    #error "No se pode avaliar uma serie de potencias numa matriz rectangular."
  #end
  #local Ordem=dimension_size(Matriz,1);
  #local Indice=1;
  #local Coeficiente=MatXos_Sucesso(0);
  #local Soma=MEscalar(Coeficiente,Ordem,Ordem)
  #local Buffer=array[Margem]
  #declare Buffer[0]=sqrt(Ordem)*abs(Coeficiente);
  #local Continua=true;
  #while(Continua)
    #local Coeficiente=MatXos_Sucesso(Indice);
    #if(Coeficiente=0)
      #local Bulk=0;
    #else
      #local Termo = ProdEscM( Coeficiente/abs(Coeficiente),
        PotM( ProdEscM( pow(abs(Coeficiente),1/Indice), Matriz ), Indice )
      )
      #declare Soma=SomaM(Soma,Termo)
      #local Bulk=NrmFrobenius(Termo);
    #end
    #declare Buffer[mod(Indice,Margem)]=Bulk;
    #declare Indice=Indice+1;
    #if(Indice>=Margem)
      #local Voluptuous=0;
      #local OMeuDedo=0;
      #while(OMeuDedo<Margem)
        #declare Voluptuous=Voluptuous+Buffer[OMeuDedo];
        #declare OMeuDedo=OMeuDedo+1;
      #end
      #declare Continua=((Indice<IndiceMin)|(Voluptuous>Epsilon));
    #end
  #end
  Soma
#end

//   Soma parcial da srie de potncias com coeficientes dados por
// MatXos_Sucesso(Indice)/Factorial(Indice), avaliada na matriz quadrada introduzida
#macro SomaParcialTaylorM(Indice, Matriz)
  #if(dimension_size(Matriz,1)!=dimension_size(Matriz,2))
    #error "No se pode avaliar uma serie de potencias numa matriz rectangular."
  #end
  #local Soma=MEscalar(MatXos_Sucesso(0),dimension_size(Matriz,1),dimension_size(Matriz,2))
  #local Qual=1;
  #while(Qual<=Indice)
    #local Coeficiente=MatXos_Sucesso(Qual);
    #if(Coeficiente!=0)
      #local Proud=Matriz
      #local ImosPolo=2;
      #while(ImosPolo<=Qual)
        #declare Proud=ProdM(Proud,Matriz)
        #declare Proud=ProdEscM(1/ImosPolo,Proud)
        #declare ImosPolo=ImosPolo+1;
      #end
      #declare Soma=SomaM(Soma,ProdEscM(Coeficiente,Proud))
    #end
    #declare Qual=Qual+1;
  #end
  Soma
#end

//   Soma da srie de potncias com coeficientes dados por MatXos_Sucesso(Indice)/Factorial(Indice),
// avaliada na matriz quadrada introduzida, e aproximada pola primeira soma parcial tal que a soma das
// normas de Frobenius dos seus derradeiros Margem termos for menor ou igual que Epsilon, abrangendo n
// menos de IndiceMin termos
#macro SomaAproxTaylorM(Epsilon, Margem, IndiceMin, Matriz)
  #if(dimension_size(Matriz,1)!=dimension_size(Matriz,2))
    #error "No se pode avaliar uma serie de potencias numa matriz rectangular."
  #end
  #local Ordem=dimension_size(Matriz,1);
  #local Indice=1;
  #local Coeficiente=MatXos_Sucesso(0);
  #local Soma=MEscalar(Coeficiente,Ordem,Ordem)
  #local Buffer=array[Margem]
  #declare Buffer[0]=sqrt(Ordem)*abs(Coeficiente);
  #local Continua=true;
  #while(Continua)
    #local Coeficiente=MatXos_Sucesso(Indice);
    #if(Coeficiente=0)
      #local Bulk=0;
    #else
      #local Proud=Matriz
      #local ImosPolo=2;
      #while(ImosPolo<=Indice)
        #declare Proud=ProdM(Proud,Matriz)
        #declare Proud=ProdEscM(1/ImosPolo,Proud)
        #declare ImosPolo=ImosPolo+1;
      #end
      #local Termo=ProdEscM(Coeficiente,Proud)
      #declare Soma=SomaM(Soma,Termo)
      #local Bulk=NrmFrobenius(Termo);
    #end
    #declare Buffer[mod(Indice,Margem)]=Bulk;
    #declare Indice=Indice+1;
    #if(Indice>=Margem)
      #local Voluptuous=0;
      #local OMeuDedo=0;
      #while(OMeuDedo<Margem)
        #declare Voluptuous=Voluptuous+Buffer[OMeuDedo];
        #declare OMeuDedo=OMeuDedo+1;
      #end
      #declare Continua=((Indice<IndiceMin)|(Voluptuous>Epsilon));
    #end
  #end
  Soma
#end


//J continuaremos...


//======================== NMEROS COMPLEXOS E HIPERCOMPLEXOS E QUATRNIOS DE HAMILTON ======================================================

//------------------------------------------- Nmeros complexos ------------------------------------------------------------------------

#macro ProdC(A,B)  //Produto
  <A.x*B.x-A.y*B.y,A.x*B.y+A.y*B.x>
#end

#macro ConjC(A)  //Conjugado
  <A.x,-A.y>
#end

#macro InvC(A)  //Inverso
  #if(vlength(A)=0)
    #error "O numero complexo 0 no tem inverso."
  #end
  ( ConjC(A)/pow(vlength(A),2) )
#end

#macro CocC(A,B)  //Cociente
  ProdC(A,InvC(B))
#end

#macro QuadrC(A)  //Quadrado
  <pow(A.x,2)-pow(A.y,2),2*A.x*A.y>
#end

#macro RaizQuadrC(A)  //Raiz quadrada de parte real maior ou igual que zero; para os nmeros reais negativos,
  <sqrt((vlength(A)+A.x)/2),sqrt((vlength(A)-A.x)/2)*select(A.y,-1,1)>  //escolhemos as razes de parte
#end                                                                               //imaginria positiva.

#macro ExpC(A)  //Exponencial
  ( exp(A.x)*<cos(A.y),sin(A.y)> )
#end

#macro LnC(A)  //Logaritmo neperiano: escolhemos o que tenha a parte imaginria compreendida entre -pi e pi;
  #if(vlength(A)=0)  //para os nmeros reais negativos tomamos o logaritmo de parte imaginria pi
    #error "O numero complexo 0 carece de logaritmo neperiano."
  #end
  <ln(vlength(A)),atan2(A.y,A.x)>
#end

#macro PotC(A,B)  //A elevado a B
  ExpC(ProdC(LnC(A),B))
#end

#macro SenC(A)  //Seno
  <sin(A.x)*cosh(A.y), cos(A.x)*sinh(A.y)>
#end

#macro ArcSenC(A)  //Arco seno
  ProdC(<0,1>,ArgShC(<A.y,-A.x>))
#end

#macro CosC(A)  //Cosseno
  <cos(A.x)*cosh(A.y),-sin(A.x)*sinh(A.y)>
#end

#macro ArcCosC(A)  //Arco cosseno
  ProdC(<0,select(A.y,1,-1)>,ArgChC(A))
#end


#macro TanC(A)  //Tangente
  #if((cos(A.x)=0)&(A.y=0))
    #error "Os numeros reais de cosseno nulo carecen de tangente."
  #else
    ( <sin(A.x)*cos(A.x),sinh(A.y)*cosh(A.y)>/(pow(cos(A.x)*cosh(A.y),2)+pow(sin(A.x)*sinh(A.y),2)) )
  #end
#end

#macro ArcTanC(A)  //Arco tangente
  #if(A.y=0)
    <atan(A.x),0>
  #else
    #if((abs(A.y)=1)&(A.x=0))
      #error "No existem numeros complexos com tangente i nem -i"
    #else
      ProdC(<0,1>,ArgThC(<A.y,-A.x>))
    #end
  #end
#end

#macro ShC(A)  //Seno hiperblico
  <sinh(A.x)*cos(A.y),cosh(A.x)*sin(A.y)>
#end

#macro ArgShC(A)  //Argumento seno hiperblico
  LnC(A+RaizQuadrC(QuadrC(A)+<1,0>))
#end

#macro ChC(A)  //Cosseno hiperblico
  <cosh(A.x)*cos(A.y),sinh(A.x)*sin(A.y)>
#end

#macro ArgChC(A)  //Argumento cosseno hiperblico
  LnC(A+RaizQuadrC(QuadrC(A)-<1,0>))
#end

#macro ThC(A)  //Tangente hiperblica
  #if((A.x=0)&(cos(A.y)=0))
    #error "Os numeros imaginarios puros tais que o cosseno da sua parte imaginaria e nulo, carecem de tangente hiperbolica."
  #else
    ( <sinh(A.x)*cosh(A.x),sin(A.y)*cos(A.y)> /
      (pow(cosh(A.x)*cos(A.y),2)+pow(sinh(A.x)*sin(A.y),2)) )
  #end
#end

#macro ArgThC(A)  //Argumento tangente hiperblica
  #if((A.y=0)&(abs(A.x)=1))
    #error "No existem numeros complexos de tangente hiperbolica 1 nem -1"
  #else
    ( LnC(<1-pow(vlength(A),2),2*A.y>/(1+pow(vlength(A),2)-2*A.x))/2 )
  #end
#end


//--------------------------------------- Quatrnios de Hamilton ------------------------------------------------------------------------

#macro EspX(A)  //Parte n real
  <A.y,A.z,A.t>
#end

#macro ProdX(A,B)  //A por B, nesta orde; a multiplica de quatrnios de Hamilton n  comutativa.
  #local a=EspX(A);
  #local b=EspX(B);
  #local vecty=A.x*b+B.x*a+vcross(a,b);
  <A.x*B.x-vdot(a,b),vecty.x,vecty.y,vecty.z>
#end

#macro ConjX(A)  //Conjugado
  <A.x,-A.y,-A.z,-A.t>
#end

#macro InvX(A)  //Inverso
  #if(vlength(A)=0)
    #error "O quaternio nulo no tem inverso."
  #end
  ( ConjX(A)/pow(vlength(A),2) )
#end

#macro QuadrX(A)  //Quadrado
  #local a=EspX(A);
  #local vecty=2*A.x*a;
  <pow(A.x,2)-vdot(a,a),vecty.x,vecty.y,vecty.z>
#end

#macro RaizQuadrX(A)  //Raiz quadrada de parte real maior ou igual que zero; para os nmeros reais negativos,
  #local a=EspX(A);                                  //escolhemos as razes que s produto de i por um
  #if(vlength(a)=0)                                                                //nmero real positivo.
    #if(A.x>=0)
      <sqrt(A.x),0,0,0>
    #else
      <0,sqrt(-A.x),0,0>
    #end
  #else
    #local vecty=sqrt((vlength(A)-A.x)/2)*vnormalize(a);
    <sqrt((vlength(A)+A.x)/2),vecty.x,vecty.y,vecty.z>
  #end
#end

#macro ExpX(A)  //Exponencial
  #local a=EspX(A);
  #if(vlength(a)=0)
    #local vecty=<0,0,0>;
  #else
    #local vecty=sin(vlength(a))*vnormalize(a);
  #end
  ( exp(A.x)*<cos(vlength(a)),vecty.x,vecty.y,vecty.z> )
#end

#macro LnX(A)  //Logaritmo neperiano: escolhemos o que tenha o mdulo da parte n real menor ou igual que
  #if(vlength(A)=0)    //pi; para os nmeros reais negativos, optamos polo logaritmo de parte n real pii
    #error "O quaternio nulo carece de logaritmo neperiano."
  #else
    #local a=EspX(A);
    #if(vlength(a)=0)
      #if(A.x>0)
        #local vecty=<0,0,0>;
      #else
        #local vecty=<pi,0,0>;
      #end
    #else
      #local vecty=acos(A.x/vlength(A))*vnormalize(a);
    #end
  #end
  <ln(vlength(A)),vecty.x,vecty.y,vecty.z>
#end

#macro PotX(A,B)  //A elevado a B
  ExpX(ProdX(LnX(A),B))
#end

#macro SenX(A)  //Seno
  #local a=EspX(A);
  #if(vlength(a)=0)
    #local vecty=<0,0,0>;
  #else
    #local vecty=cos(A.x)*sinh(vlength(a))*vnormalize(a);
  #end
  <sin(A.x)*cosh(vlength(a)),vecty.x,vecty.y,vecty.z>
#end

#macro ArcSenX(A)  //Arco seno
  #local a=EspX(A);
  #if(vlength(a)=0)
    #if((A.x>=-1)&(A.x<=1))
      <asin(A.x),0,0,0>
    #else
      ProdX(<0,1,0,0>,ArgShX(<0,-A.x,0,0>))
    #end
  #else
    #declare a=<0,a.x,a.y,a.z>;
    ProdX(vnormalize(a),ArgShX(<vlength(a),0,0,0>-A.x*vnormalize(a)))
  #end
#end

#macro CosX(A)  //Cosseno
  #local a=EspX(A);
  #if(vlength(a)=0)
    #local vecty=<0,0,0>;
  #else
    #local vecty=-sin(A.x)*sinh(vlength(a))*vnormalize(a);
  #end
  <cos(A.x)*cosh(vlength(a)),vecty.x,vecty.y,vecty.z>
#end

#macro ArcCosX(A)  //Arco cosseno
  #local a=EspX(A);
  #if(vlength(a)=0)
    #if((A.x>=-1)&(A.x<=1))
      <acos(A.x),0,0,0>
    #else
      ProdX(<0,-1,0,0>,ArgChX(A))
    #end
  #else
    #declare a=<0,a.x,a.y,a.z>;
    ProdX(-vnormalize(a),ArgChX(A))
  #end
#end

#macro TanX(A)  //Tangente
  #local a=EspX(A);
  #if(vlength(a)=0)
    #if(cos(A.x)=0)
      #error "Os numeros reais de cosseno nulo carecem de tangente."
    #else
      #local vecty=<0,0,0>;
    #end
  #else
    #local vecty=sinh(vlength(a))*cosh(vlength(a))*vnormalize(a);
  #end
  ( <sin(A.x)*cos(A.x),vecty.x,vecty.y,vecty.z> /
    (pow(cos(A.x)*cosh(vlength(a)),2)+pow(sin(A.x)*sinh(vlength(a)),2)) )
#end

#macro ArcTanX(A)  //Arco tangente
  #local a=EspX(A);
  #if(vlength(a)=0)
    <atan(A.x),0,0,0>
  #else
    #if((vlength(a)=1)&(A.x=0))
      #error "No existem quaternios com tangente pura e unitaria."
    #else
      #declare a=<0,a.x,a.y,a.z>;
      ProdX(vnormalize(a),ArgThX(<vlength(a),0,0,0>-A.x*vnormalize(a)))
    #end
  #end
#end

#macro ShX(A)  //Seno hiperblico
  #local a=EspX(A);
  #if(vlength(a)=0)
    #local vecty=<0,0,0>;
  #else
    #local vecty=cosh(A.x)*sin(vlength(a))*vnormalize(a);
  #end
  <sinh(A.x)*cos(vlength(a)),vecty.x,vecty.y,vecty.z>
#end

#macro ArgShX(A)  //Argumento seno hiperblico
  LnX(A+RaizQuadrX(QuadrX(A)+<1,0,0,0>))
#end

#macro ChX(A)  //Cosseno hiperblico
  #local a=EspX(A);
  #if(vlength(a)=0)
    #local vecty=<0,0,0>;
  #else
    #local vecty=sinh(A.x)*sin(vlength(a))*vnormalize(a);
  #end
  <cosh(A.x)*cos(vlength(a)),vecty.x,vecty.y,vecty.z>
#end

#macro ArgChX(A)  //Argumento cosseno hiperblico
  LnX(A+RaizQuadrX(QuadrX(A)-<1,0,0,0>))
#end

#macro ThX(A)  //Tangente hiperblica
  #local a=EspX(A);
  #if(vlength(a)=0)
    #local vecty=<0,0,0>;
  #else
    #if((A.x=0)&(cos(vlength(a))=0))
      #error "Os quaternios puros tais que o cosseno do modulo da sua parte no real e nulo, carecem de tangente hiperbolica."
    #else
      #local vecty=sin(vlength(a))*cos(vlength(a))*vnormalize(a);
    #end
  #end
  ( <sinh(A.x)*cosh(A.x),vecty.x,vecty.y,vecty.z> /
    (pow(cosh(A.x)*cos(vlength(a)),2)+pow(sinh(A.x)*sin(vlength(a)),2)) )
#end

#macro ArgThX(A)  //Argumento tangente hiperblica
  #if((vlength(EspX(A))=0)&(abs(A.x)=1))
    #error "No existem quaternios de tangente hiperbolica 1 nem -1"
  #else
    ( LnX(<1-pow(vlength(A),2),2*A.y,2*A.z,2*A.t>/(1+pow(vlength(A),2)-2*A.x))/2 )
  #end
#end


//----------------------------------------- Nmeros hipercomplexos ------------------------------------------------------------------------

//Outro dia comeo...


#version versionMatametika;