// Arquivo auxiliar para o traador de raios de Persistence of Vision
// Arquivo: Mecanica.inc
// Vers: 3.6
// Descri: Ajuda a calcular alguns movimentos dos corpos. Empregam-se unidades do SI.
// Data de cria: XII-2002
// Data da ltima actualiza: 30-VIII-2004
// Autor: Xos B. Carballeira Rabual

//  Provavelmente interesse modificar a macro MovSolRigCampoDeForcas para que calcule as rotaes median-
// te os quatrnios de Hamilton, a partir do vector de giro, no canto de usar os ngulos de Euler, como faz
// agora. Alis, h muito que limpar por a...

//  NOTA (24-VIII-2005). Lembremos que dado um quatrnio q representativo dum acontecimento no espao-tem-
// po (de jeito que o tempo corresponder  sua parte real), ao rotar a parte espacial arredor do eixo do
// quatrnio espacial puro phi, no sentido indicado pola regra do parafuso a respeito do sentido de phi,
// um ngulo igual a |phi|, se obter o quatrnio:
//                                     phi/2   -phi/2
//                               q' = e     q e

#local versionMecXos=version;
#version 3.6;

//Requer:
//#include "Matematica.inc"

#ifndef(Xose_Mensagens)
  #declare(Xose_Mensagens)=false;
#end

#if(Xose_Mensagens)
  #debug "    Esta-se a executar o arquivo \"Mecanica.inc\", que ajuda a calcular alguns movimentos dos objectos.\n"
#end

#ifndef(Xose_SRDextrogiro)
  #declare Xose_SRDextrogiro=false;
  #if(Xose_Mensagens)
    #debug "    Por omisso, considera-se o sistema de referencia levogiro.\n"
  #end
#end

#ifndef(Xose_PastaTemp)
  #declare Xose_PastaTemp=""
#end

#declare MecXos_Gravidade=-9.8*(Xose_SRDextrogiro? z: y);


//------------------------------------------ Cinemtica ------------------------------------------------------------------------------------

//Movimento uniformemente acelerado
#macro MUA(vInicial, Aceleraco, Tempo)
  #declare MecXos_Despracamento=vInicial*Tempo+Aceleraco*pow(Tempo,2)/2;
  translate MecXos_Despracamento
#end


//------------------------------------------ Inrcias ----------------------------------------------------------------------------------------

// As seguintes macros ajudam a definir a matriz Inercias0 requerida por MovSolRigCampoDeForcas como pa-
//rmetro.

// Esta recolhe os dados inerciais dum elemento, situados no lugar indicado duma matriz, e sobre-escreve-
//-os com os valores do mesmo elemento rotado.
// O quarto parmetro deve ser uma rota. Qualquer transforma ser aceite, mas se ela alterar o tama-
//nho ou a forma do elemento, o resultado n ser correcto. As translaes n alteram o resultado, j que
//o centro de massas tambm se move nelas, mas reduzem a precis.
#macro InerciaRot(Matriz, Corpo, Elemento, Rotaco)
  #local Primi=array[3][3]{
    { Matriz[Corpo][Elemento][1], -Matriz[Corpo][Elemento][4], -Matriz[Corpo][Elemento][5]}
    {-Matriz[Corpo][Elemento][4],  Matriz[Corpo][Elemento][2], -Matriz[Corpo][Elemento][6]}
    {-Matriz[Corpo][Elemento][5], -Matriz[Corpo][Elemento][6],  Matriz[Corpo][Elemento][3]}
  }
  #local Choufa=MatrizTransform(Rotaco,false)
  #local Resul=ProdM(ProdM(Choufa,Primi),Transposta(Choufa))
  #declare Matriz[Corpo][Elemento][1]=Resul[0][0];
  #declare Matriz[Corpo][Elemento][2]=Resul[1][1];
  #declare Matriz[Corpo][Elemento][3]=Resul[2][2];
  #declare Matriz[Corpo][Elemento][4]=-Resul[1][0];
  #declare Matriz[Corpo][Elemento][5]=-Resul[2][0];
  #declare Matriz[Corpo][Elemento][6]=-Resul[2][1];
#end

// Esta recolhe os dados inerciais dum elemento, situados no lugar indicado duma matriz, e sobre-escreve-
//-os com os valores do mesmo elemento a escala, supondo a massa se conservar.
#macro InerciaEscala(Matriz, Corpo, Elemento, CoefsEscala)
  #local A=pow(CoefsEscala.x,2)/2;
  #local B=pow(CoefsEscala.y,2)/2;
  #local C=pow(CoefsEscala.z,2)/2;
  #local Resul=array[6]{
    (B+C)*Matriz[Corpo][Elemento][1]+(-B+C)*Matriz[Corpo][Elemento][2]+(B-C)*Matriz[Corpo][Elemento][3],
    (-A+C)*Matriz[Corpo][Elemento][1]+(A+C)*Matriz[Corpo][Elemento][2]+(A-C)*Matriz[Corpo][Elemento][3],
    (-A+B)*Matriz[Corpo][Elemento][1]+(A-B)*Matriz[Corpo][Elemento][2]+(A+B)*Matriz[Corpo][Elemento][3],
    CoefsEscala.x*CoefsEscala.y*Matriz[Corpo][Elemento][4],
    CoefsEscala.x*CoefsEscala.z*Matriz[Corpo][Elemento][5],
    CoefsEscala.y*CoefsEscala.z*Matriz[Corpo][Elemento][6]
  }
  #declare Matriz[Corpo][Elemento][1]=Resul[0];
  #declare Matriz[Corpo][Elemento][2]=Resul[1];
  #declare Matriz[Corpo][Elemento][3]=Resul[2];
  #declare Matriz[Corpo][Elemento][4]=Resul[3];
  #declare Matriz[Corpo][Elemento][5]=Resul[4];
  #declare Matriz[Corpo][Elemento][6]=Resul[5];
#end

// A seguir, vrias macros que escrevem no lugar assinalado duma matriz os valores inerciais de
//certos corpos geomtricos:

//Corpo nulo, sem massa
#macro InerciaNulo(Matriz, Corpo, Elemento)
  #declare Matriz[Corpo][Elemento][0]=0;
  #declare Matriz[Corpo][Elemento][1]=0;
  #declare Matriz[Corpo][Elemento][2]=0;
  #declare Matriz[Corpo][Elemento][3]=0;
  #declare Matriz[Corpo][Elemento][4]=0;
  #declare Matriz[Corpo][Elemento][5]=0;
  #declare Matriz[Corpo][Elemento][6]=0;
#end

//Ortoedro macio homogneo de arestas paralelas aos eixos de coordenadas
#macro InerciaCaixa(Massa, ArestaX, ArestaY, ArestaZ, Matriz, Corpo, Elemento)
  #declare Matriz[Corpo][Elemento][0]=Massa;
  #declare Matriz[Corpo][Elemento][1]=Massa*(pow(ArestaY,2)+pow(ArestaZ,2))/12;
  #declare Matriz[Corpo][Elemento][2]=Massa*(pow(ArestaX,2)+pow(ArestaZ,2))/12;
  #declare Matriz[Corpo][Elemento][3]=Massa*(pow(ArestaX,2)+pow(ArestaY,2))/12;
  #declare Matriz[Corpo][Elemento][4]=0;
  #declare Matriz[Corpo][Elemento][5]=0;
  #declare Matriz[Corpo][Elemento][6]=0;
#end

//Cilindro macio homogneo de eixo paralelo ao OX
#macro InerciaCilindroX(Massa, Raio, Altura, Matriz, Corpo, Elemento)
  #local I=Massa*(3*pow(Raio,2)+pow(Altura,2))/12;
  #declare Matriz[Corpo][Elemento][0]=Massa;
  #declare Matriz[Corpo][Elemento][1]=Massa*pow(Raio,2)/2;
  #declare Matriz[Corpo][Elemento][2]=I;
  #declare Matriz[Corpo][Elemento][3]=I;
  #declare Matriz[Corpo][Elemento][4]=0;
  #declare Matriz[Corpo][Elemento][5]=0;
  #declare Matriz[Corpo][Elemento][6]=0;
#end
//Cilindro macio homogneo
#macro InerciaCilindro(Massa, Raio, Altura, VectEixo, Matriz, Corpo, Elemento)
  InerciaCilindroX(Massa, Raio, Altura, Matriz, Corpo, Elemento)
  InerciaRot(Matriz, Corpo, Elemento, transform{RotAntDep(x,VectEixo)})
#end

//Cono macio homogneo de eixo paralelo ao OX
#macro InerciaConoX(Massa, Raio, Altura, Matriz, Corpo, Elemento)
  #local I=Massa*(12*pow(Raio,2)+3*pow(Altura,2))/80;
  #declare Matriz[Corpo][Elemento][0]=Massa;
  #declare Matriz[Corpo][Elemento][1]=3/10*Massa*pow(Raio,2);
  #declare Matriz[Corpo][Elemento][2]=I;
  #declare Matriz[Corpo][Elemento][3]=I;
  #declare Matriz[Corpo][Elemento][4]=0;
  #declare Matriz[Corpo][Elemento][5]=0;
  #declare Matriz[Corpo][Elemento][6]=0;
#end
//Cono macio homogneo
#macro InerciaCono(Massa, Raio, Altura, VectEixo, Matriz, Corpo, Elemento)
  InerciaConoX(Massa, Raio, Altura, Matriz, Corpo, Elemento)
  InerciaRot(Matriz, Corpo, Elemento, transform{RotAntDep(x,VectEixo)})
#end
//  Lembremos que a distncia do centro de massas dum cono circular recto homogneo
// sua base  1/4 da sua altura.

//Esfera macia homognea
#macro InerciaEsfera(Massa, Raio, Matriz, Corpo, Elemento)
  #local I=2/5*Massa*pow(Raio,2);
  #declare Matriz[Corpo][Elemento][0]=Massa;
  #declare Matriz[Corpo][Elemento][1]=I;
  #declare Matriz[Corpo][Elemento][2]=I;
  #declare Matriz[Corpo][Elemento][3]=I;
  #declare Matriz[Corpo][Elemento][4]=0;
  #declare Matriz[Corpo][Elemento][5]=0;
  #declare Matriz[Corpo][Elemento][6]=0;
#end

//Toro macio homogneo de eixo de revolu paralelo ao OX
#macro InerciaToroX(Massa, RaioPpal, RaioTubo, Matriz, Corpo, Elemento)
  #local I=Massa*(pow(RaioPpal,2)+3/4*pow(RaioTubo,2))/2;
  #declare Matriz[Corpo][Elemento][0]=Massa;
  #declare Matriz[Corpo][Elemento][1]=Massa*(pow(RaioPpal,2)+pow(RaioTubo,2)/4);
  #declare Matriz[Corpo][Elemento][2]=I;
  #declare Matriz[Corpo][Elemento][3]=I;
  #declare Matriz[Corpo][Elemento][4]=0;
  #declare Matriz[Corpo][Elemento][5]=0;
  #declare Matriz[Corpo][Elemento][6]=0;
#end
//Toro macio homogneo
#macro InerciaToro(Massa, RaioPpal, RaioTubo, VectEixo, Matriz, Corpo, Elemento)
  InerciaToroX(Massa, RaioPpal, RaioTubo, Matriz, Corpo, Elemento)
  InerciaRot(Matriz, Corpo, Elemento, transform{RotAntDep(x,VectEixo)})
#end


//----------------------------------------- Dinmica do slido rgido -----------------------------------------------------------------------------

//  Movimento aproximado dum conjunto de slidos rgidos submetidos a um campo de foras. Cada corpo est
//formado por um ou mais elementos ou componentes, os quais se tratarm como pontos materiais nalguns cl-
//culos.

//  Supe-se definido o campo de foras mediante uma macro:
//    MecXos_Forcas(CorpoActual, ElementoActual, Resultante, Momento)
//  O campo devolve a fora resultante sobre cada elemento, e o momento total respeito do centro de massas
//deste mediante os parmetros Resultante e Momento, respectivamente.
//  Nome (v. cabeceira da macro MovSolRigCampoDeForcas, mais abaixo)  uma seqncia de caracteres que
//identifica o arquivo no que se guardarm os dados mecnicos do sistema de corpos; h-se de usar, pois, um
//nome distinto para cada sistema no caso de haver vrios assemade. QuantosElementos  uma matriz unidimen-
//sional com o nmero de elementos de cada corpo, estes ltimos na mesma ordem das demais matrizes. FreqCalc
// o nmero de vezes que se calcularm as foras cada unidade de tempo. Posicoes0  uma matriz bidimen-
//sional com as posies iniciais dos centros de massas de todos os elementos; o primeiro ndice refere-se
//ao corpo ao que cada um pertence, e o segundo identifica o componente. Este segundo ndice deve tomar, ao
//menos, tantos valores como elementos possua o corpo que mais tiver; as posies que n corresponderem a
//nengum elemento, n se levarm na conta. As restantes matrizes comportar-se-am de jeito similar, e a or-
//dem dos elementos, como a dos corpos, ser a mesma em todas. Inercias0  uma matriz tridimensional; cada
//corpo identifica-se polo valor adequado do primeiro ndice, e nele, o segundo ndice designa um elemento;
//dados estes dous, Inercias0 guarda o seguinte para cada valor do terceiro ndice:
// 0) a massa do elemento;
// 1) o seu momento de inrcia inicial respeito do eixo paralelo ao OX polo seu centro de massas;
// 2) "  "     "    "     "       "       "     "   "      "     "  OY  "    "    "    "    "  ;
// 3) "  "     "    "     "       "       "     "   "      "     "  OZ  "    "    "    "    "  ;
// 4) "  "  produto de inercia    "       "     dos eixos paralelos aos OX e OY polo seu centro de massas;
// 5) "  "     "     "     "      "       "      "    "       "      "  OX e OZ  "    "    "    "    "  ;
// 6) "  "     "     "     "      "       "      "    "       "      "  OY e OZ  "    "    "    "    "  .
//VelocidadesCM0  uma matriz unidimensional com a velocidade inicial do centro de massas de cada corpo, e
//VelocidadesAngulares0, outra com as suas velocidades angulares iniciais de rota. Tempo  a varivel
//fsica temporal, e Tempo0 o valor tomado por aquela na primeira chamada a MovSolRigCampoDeForcas com o
//Nome actual.
//  Podem-se definir a matriz unidimensional MecXos_QuantosPontosFixos e mais a bidimensional
//MecXos_PontosFixos para impedir o movimento dalgum ponto dos corpos; a primeira contm o nmero de pon-
//tos fixos de cada slido, e a segunda, as posies destes; em MecXos_PontosFixos, o primeiro ndice re-
//fere o corpo, e o segundo, cada ponto fixo; s se considerar para cada corpo um nmero de posies
//igual ao que lhe assigna MecXos_QuantosPontosFixos; o resto n corresponde a ponto fixo algum. As ma-
//trizes de trs dimenses MecXos_MassasAgentes e MecXos_VectsAgentes0, tambm opcionais, nas que os dous
//primeiros ndices significam o mesmo que em Inercias0, permitem assignar vrios tipos (um por cada valor
//do terceiro ndice) de massas agentes como a carga para o campo eltrico e de vectores agentes o
//produto de intensidade de corrente por longitude para o campo magntico distintos a cada componente.
//  Quando se comear o trao dum subconjunto dos quadros duma anima que empregue
//MovSolRigCampoDeForcas, esta macro buscar o arquivo de dados correspondente ao quadro prvio; se o
//topa, l-lo- e continuar a simula normalmente; do contrrio, reiniciar os clculos desde Tempo0,
//o qual provocar a mido uma gr demora. Definindo a varivel MecXos_ReiniciaMSRCF com o valor 1 antes
//da chamada a MovSolRigCampoDeForcas, far-se- este reincio mesmo com o arquivo temporal presente. Con-
//sidere-se que, por mor da acumula de erros, os resultados de duas simulaes quase idnticas no prin-
//cpio, podem diferir fortemente a longo prazo (Teoria do Caos), de jeito que estes reincios s vezes
//produzirm saltos na anima, sobretudo se ocorrerem numa fase serdia dela.

//  MovSolRigCampoDeForcas fornece os seguintes dados:
// 1) a matriz de transformaes MecXos_Movimentos;
// 2) a matriz de transformaes MecXos_Rotacoes;
// 3) a matriz de transformaes MecXos_TensInercias;
// 4) a matriz de transformaes MecXos_InvTensInercias;
// 5) a matriz de vectores MecXos_CMs;
// 6) a matriz de vectores MecXos_Posicoes;
// 7) a matriz de vectores MecXos_Despracamentos;
// 8) a matriz de vectores MecXos_Angulos;
// 9) a matriz de vectores MecXos_VelocidadesCM;
// 10) a matriz de vectores MecXos_Velocidades, e mais
// 11) a matriz de vectores MecXos_VelocidadesAngulares.

//  H que proteger MovSolRigCampoDeForcas para quando algum tensor de inrcia for singular, e melhorar
//se  possvel o comportamento quando se usa um ponto fixo a uma distncia considervel do centro de
//massas.

#macro MovSolRigCampoDeForcas(Nome, QuantosElementos, FreqCalc, Posicoes0, Inercias0, VelocidadesCM0,
    VelocidadesAngulares0, Tempo, Tempo0)

  #local QuantosCorpos=dimension_size(QuantosElementos,1);
  #local ContaCorpos=0;
  #local NoMaxElmts=0;
  #while(ContaCorpos<QuantosCorpos)
    #if(NoMaxElmts<QuantosElementos[ContaCorpos])
      #declare NoMaxElmts=QuantosElementos[ContaCorpos];
    #end
    #declare ContaCorpos=ContaCorpos+1;
  #end
  #ifndef(MecXos_QuantosPontosFixos)
    #if(Xose_Mensagens)
      #debug "    Por omisso, no se fixaram pontos.\n"
    #end
    #declare MecXos_QuantosPontosFixos=array[QuantosCorpos]
    #local ContaCorpos=0;
    #while(ContaCorpos<QuantosCorpos)
      #declare MecXos_QuantosPontosFixos[ContaCorpos]=0;
      #declare ContaCorpos=ContaCorpos+1;
    #end
  #end
  #if(strlen(Xose_PastaTemp)=0)
    #local NomeDoFilio=concat(Nome, ".xose");
    #warning "Ao no se indicar, mediante a variavel Xose_PastaTemp, uma pasta para armazenar os dados\n  temporais (arquivos \".xose\"), empregar-se-a a actual."
    #warning "V. \"Das variaveis xosianas.txt\"."
    #warning "Existe o risco de que esta mude durante o processo e se perca informaco."
    #warning "Isto pode ocasionar que os objectos movidos com MovSolRigCampoDeForcas saltem bruscamente, perden-\n  do-se de vista, ou mesmo causando um erro no traco."
    #warning "Desaconselha-se, de prosseguir, toda aco que possa mudar a pasta actual, seja ou no com o POV-Ray,\n  ate que se terminem todos os quadros."
  #else
    #local NomeDoFilio=concat(Xose_PastaTemp, "\\", Nome, ".xose")
  #end
  #if(Xose_Mensagens)
    #debug concat("    Gravar-se-am os dados do movimento dos solidos no arquivo \"", NomeDoFilio, "\".\n")
  #end
  #local IncouderMA=!defined(MecXos_MassasAgentes);
  #local IncouderVA=!defined(MecXos_VectsAgentes0);
  #if(IncouderMA)
    #declare MecXos_MassasAgentes=array[QuantosCorpos][NoMaxElmts][1];
    #if(Xose_Mensagens)
      #debug "    Por omisso, os solidos careceram de massas agentes.\n"
    #end
  #end
  #if(IncouderVA)
    #declare MecXos_VectsAgentes0=array[QuantosCorpos][NoMaxElmts][1];
    #if(Xose_Mensagens)
      #debug "    Por omisso, os solidos careceram de \"vectores agentes\".\n"
    #end
  #end
  #if(IncouderMA|IncouderVA)
    #local ContaCorpos=0;
    #while(ContaCorpos<QuantosCorpos)
      #local MaisInfernoVermelho=0;
      #while(MaisInfernoVermelho<QuantosElementos[ContaCorpos])
        #if(IncouderMA) #declare MecXos_MassasAgentes[ContaCorpos][MaisInfernoVermelho][0]=0; #end
        #if(IncouderVA) #declare MecXos_VectsAgentes0[ContaCorpos][MaisInfernoVermelho][0]=<0,0,0>; #end
        #declare MaisInfernoVermelho=MaisInfernoVermelho+1;
      #end
      #declare ContaCorpos=ContaCorpos+1;
    #end
  #end
  #declare MecXos_Movimentos=array[QuantosCorpos]
  #declare MecXos_Rotacoes=array[QuantosCorpos]
  #declare MecXos_TensInercias=array[QuantosCorpos]
  #declare MecXos_InvTensInercias=array[QuantosCorpos]
  #declare MecXos_CMs=array[QuantosCorpos]
  #declare MecXos_Posicoes=array[QuantosCorpos][NoMaxElmts]
  #declare MecXos_Velocidades=array[QuantosCorpos][NoMaxElmts]
  #declare MecXos_VectsAgentes=array[QuantosCorpos][NoMaxElmts][dimension_size(MecXos_VectsAgentes0,3)]

  #if(Tempo=Tempo0)

    // Primeiro quadro da simula

    MecXos_ComecaSimul()

  #else

    // Quadros sucessivos

    #ifndef(MecXos_ReiniciaMSRCF)
      #declare MecXos_ReiniciaMSRCF=false;
    #end
    #if(file_exists(NomeDoFilio)&!(MecXos_ReiniciaMSRCF&(frame_number=initial_frame)))
      #include NomeDoFilio  //Recolhemos os dados do quadro prvio.
    #else
      MecXos_ComecaSimul()
      #declare MecXos_Tempo=Tempo0
    #end

    //  Definimos os centros de massas dos corpos, as posies dos elementos, as velocidades destes e mais
    //os seus vectores agentes j rotados.
    #local ContaCorpos=0;
    #while(ContaCorpos<QuantosCorpos)
      #if(Xose_Mensagens)
        #debug concat( "\n  --------------------------------- Corpo numero ",
          str(ContaCorpos,0,0), " de ", str(QuantosCorpos,0,0), " ---------------------------------\n" )
      #end
      #if(MecXos_QuantosPontosFixos[ContaCorpos]=0)
        #local PtoRef=MecXos_CMs0[ContaCorpos];
      #else
        #local PtoRef=MecXos_PontosFixos[ContaCorpos][0];
        #if(MecXos_QuantosPontosFixos[ContaCorpos]=2)
          #local uEixoFixo =
            vnormalize(MecXos_PontosFixos[ContaCorpos][1]-MecXos_PontosFixos[ContaCorpos][0]);
        #end
      #end
      MecXos_ActualizaInfoCorpo()
      #declare ContaCorpos=ContaCorpos+1;
    #end

    // Clculo do movimento
    #local Tempus=MecXos_Tempo;
    #while(Tempus<Tempo)
      #if(Xose_Mensagens)
        #debug concat("\n  **************************** Agora Tempo = ", NotacoCientifica(Tempus,6),
          " ****************************\n")
        #debug concat("\n  ********************** Tracar-se-a o vindeiro quadro quando Tempo = ",
          NotacoCientifica(Tempo,6),
          " **********************\n")
      #end
      #local ContaCorpos=0;
      #while(ContaCorpos<QuantosCorpos)
        #if(Xose_Mensagens)
          #debug concat( "\n  --------------------------------- Corpo numero ",
            str(ContaCorpos,0,0), " de ", str(QuantosCorpos,0,0), " ---------------------------------\n" )
        #end
        #if(MecXos_QuantosPontosFixos[ContaCorpos]=0)
          #local PtoRef=MecXos_CMs[ContaCorpos];
          #local vRef=MecXos_VelocidadesCM[ContaCorpos];
          #local SomaForcas=<0,0,0>;
        #else
          #local PtoRef=MecXos_PontosFixos[ContaCorpos][0];
          #local vRef=<0,0,0>;
          #if(MecXos_QuantosPontosFixos[ContaCorpos]=2)
            #local uEixoFixo =
              vnormalize(MecXos_PontosFixos[ContaCorpos][1]-MecXos_PontosFixos[ContaCorpos][0]);
          #end
        #end
        #local ContaElmts=0;
        #local SomaMomentos=<0,0,0>;
        #while(ContaElmts<QuantosElementos[ContaCorpos])
          #local Forcinha=<0,0,0>;
          #local Momentinho=<0,0,0>;
          MecXos_Forcas(ContaCorpos, ContaElmts, Forcinha, Momentinho)
          #if(Xose_Mensagens)
            #debug concat("    Forca resultante sobre o elemento numero ", str(ContaElmts,0,0), ": ",
              NotacoCientificaVector(3,Forcinha,6), "\n")
            #debug concat("    Momento total sobre o elemento numero ", str(ContaElmts,0,0),
              ", respeito do seu centro de massas: ", NotacoCientificaVector(3,Momentinho,6), "\n")
          #end
          #if(MecXos_QuantosPontosFixos[ContaCorpos]=0)
            #declare SomaForcas=SomaForcas+Forcinha;
          #end
          #declare SomaMomentos = SomaMomentos +
            vcross(MecXos_Posicoes[ContaCorpos][ContaElmts]-PtoRef,Forcinha) + Momentinho;
          #declare ContaElmts=ContaElmts+1;
        #end
        #if(MecXos_QuantosPontosFixos[ContaCorpos]<2)
          #if(MecXos_QuantosPontosFixos[ContaCorpos]=0)
            #local aCM=SomaForcas/MecXos_InerciasDosCorpos0[ContaCorpos][0];
            #declare MecXos_Despracamentos[ContaCorpos] = MecXos_Despracamentos[ContaCorpos] +
              MecXos_VelocidadesCM[ContaCorpos]/FreqCalc + aCM/(2*pow(FreqCalc,2));
            #declare MecXos_VelocidadesCM[ContaCorpos]=MecXos_VelocidadesCM[ContaCorpos]+aCM/FreqCalc;
            #if(Xose_Mensagens)
              #debug concat("\n    Forca resultante: ", NotacoCientificaVector(3,SomaForcas,6), "\n")
              #debug concat("    Quantidade de movimento: ", NotacoCientificaVector( 3,
                MecXos_InerciasDosCorpos0[ContaCorpos][0]*MecXos_VelocidadesCM[ContaCorpos],6), "\n" )
              #debug concat("    Aceleraco do centro de massas: ", NotacoCientificaVector(3,aCM,6), "\n")
              #debug concat("    Velocidade do centro de massas: ", NotacoCientificaVector( 3,
                MecXos_VelocidadesCM[ContaCorpos],6), "\n" )
              #debug concat("    Despracamento acumulado do centro de massas: ",
                NotacoCientificaVector(3,MecXos_Despracamentos[ContaCorpos],6),"\n")
              #local EcTransl=MecXos_InerciasDosCorpos0[ContaCorpos][0] *
                pow(vlength(MecXos_VelocidadesCM[ContaCorpos]),2)/2;
              #debug concat("    Energia cinetica de translaco: ", NotacoCientifica(EcTransl,6), "\n")
            #end
          #end
          #local Omega=MecXos_VelocidadesAngulares[ContaCorpos];
          #local YaTieneTodoLoEso =
            SomaMomentos-vcross(Omega, MecXos_TensInercias[ContaCorpos](Omega.x,Omega.y,Omega.z));
          #local Gamma =
            MecXos_InvTensInercias[ContaCorpos](YaTieneTodoLoEso.x,YaTieneTodoLoEso.y,YaTieneTodoLoEso.z);
          #local OmegaMd=Omega+Gamma/(2*FreqCalc);
          #declare MecXos_VelocidadesAngulares[ContaCorpos]=Omega+Gamma/FreqCalc;
          #local DeltaTheta = (OmegaMd.x*cos(MecXos_Angulos[ContaCorpos].z) +
            OmegaMd.y*sin(MecXos_Angulos[ContaCorpos].z)) / FreqCalc;
          #if(OmegaMd.x*sin(MecXos_Angulos[ContaCorpos].z)=OmegaMd.y*cos(MecXos_Angulos[ContaCorpos].z))
            #local DeltaPsi=0;
          #else
            #local DeltaPsi=(OmegaMd.x*sin(MecXos_Angulos[ContaCorpos].z)-OmegaMd.y*cos(MecXos_Angulos[ContaCorpos].z)) /
              ((sin(MecXos_Angulos[ContaCorpos].y+DeltaTheta/2)=0)? ((sin(MecXos_Angulos[ContaCorpos].y)=0)? 1e-6:sin(MecXos_Angulos[ContaCorpos].y)):
                sin(MecXos_Angulos[ContaCorpos].y+DeltaTheta/2))/FreqCalc;
          #end
          #local DeltaPhi=OmegaMd.z/FreqCalc-DeltaPsi*cos(MecXos_Angulos[ContaCorpos].y+DeltaTheta/2);
          #declare MecXos_Angulos[ContaCorpos] = < mod(MecXos_Angulos[ContaCorpos].x+DeltaPsi,2*pi),
            mod(MecXos_Angulos[ContaCorpos].y+DeltaTheta,2*pi),
            mod(MecXos_Angulos[ContaCorpos].z+DeltaPhi,2*pi) >;
          #if(Xose_Mensagens)
            #local Omega=MecXos_VelocidadesAngulares[ContaCorpos];
            #local MomCin=MecXos_TensInercias[ContaCorpos](Omega.x,Omega.y,Omega.z);
            #debug concat("\n    Momento das forcas: ", NotacoCientificaVector(3,SomaMomentos,6), "\n")
            #debug concat("    Momento cinetico: ", NotacoCientificaVector(3,MomCin,6), "\n")
            #debug concat("    Aceleraco angular: ", NotacoCientificaVector(3,Gamma,6), "\n")
            #debug concat("    Velocidade angular: ", NotacoCientificaVector(3,Omega,6), "\n")
            #debug concat("    Angulo de rotaco propria de Euler: ", NotacoCientifica(MecXos_Angulos[ContaCorpos].x,6), "\n")
            #debug concat("    Angulo de nutaco de Euler: ", NotacoCientifica(MecXos_Angulos[ContaCorpos].y,6), "\n")
            #debug concat("    Angulo de precesso de Euler: ", NotacoCientifica(MecXos_Angulos[ContaCorpos].z,6), "\n")
            #local EcRot=vdot(Omega,MomCin)/2;
            #debug concat("    Energia cinetica de rotaco: ", NotacoCientifica(EcRot,6), "\n")
            #debug concat("\n    Energia cinetica total: ", NotacoCientifica(EcTransl+EcRot,6), "\n")
          #end
        #else
          #local IEixoFixo =
            vdot(uEixoFixo,MecXos_TensInercias[ContaCorpos](uEixoFixo.x,uEixoFixo.y,uEixoFixo.z));
          #local Omega=vdot(MecXos_VelocidadesAngulares[ContaCorpos],uEixoFixo);
          #local Gamma=vdot(SomaMomentos,uEixoFixo)/IEixoFixo;
          #declare MecXos_VelocidadesAngulares[ContaCorpos]=(Omega+Gamma/FreqCalc)*uEixoFixo;
          #declare MecXos_Angulos[ContaCorpos] =
            mod(MecXos_Angulos[ContaCorpos].x+Omega/FreqCalc+Gamma/(2*pow(FreqCalc,2)),2*pi)*x;
          #if(Xose_Mensagens)
            #local Omega=vdot(MecXos_VelocidadesAngulares[ContaCorpos],uEixoFixo);
            #debug concat("\n    Momento das forcas respeito do eixo fixo: ",
              NotacomCientifica(vdot(SomaMomentos,uEixoFixo),6), "\n")
            #debug concat("    Momento cinetico segundo o eixo: ", NotacoCientifica(IEixoFixo*Omega,6),
              "\n")
            #debug concat("    Aceleraco angular: ", NotacoCientifica(Gamma,6), "\n")
            #debug concat("    Velocidade angular: ", NotacoCientifica(Omega,6), "\n")
            #debug concat("    Angulo girado: ", NotacoCientifica(MecXos_Angulos[ContaCorpos].x,6), "\n")
            #local EcRot=IEixoFixo*pow(Omega,2)/2;
            #debug concat("\n    Energia cinetica: ", NotacoCientifica(EcRot,6), "\n")
          #end
        #end
        #if(MecXos_QuantosPontosFixos[ContaCorpos]=0)
          #local PtoRef=MecXos_CMs0[ContaCorpos];
        #end
        MecXos_ActualizaInfoCorpo()
        #if(MecXos_QuantosPontosFixos[ContaCorpos]>0)
          #declare MecXos_VelocidadesCM[ContaCorpos] =
            vcross(MecXos_VelocidadesAngulares[ContaCorpos],MecXos_CMs[ContaCorpos]-PtoRef);
        #end
        #declare ContaCorpos=ContaCorpos+1;
      #end
      #declare Tempus=Tempus+1/FreqCalc;
    #end

  #end

  // Gravamos os dados mecnicos para o vindeiro quadro.
  #fopen Filio NomeDoFilio write
    #write( Filio,
      "// Arquivo de dados temporais para o \"Mecanica.inc\" de Xose\n\n#declare MecXos_InerciasDosCorpos0=array[",
      str(QuantosCorpos,0,0), "][7]{\n"
    )
    #local ContaCorpos=0;
    #while(ContaCorpos<QuantosCorpos)
      #write(Filio, "  {")
      #local Indice=0;
      #while(Indice<7)
        #write(Filio, NotacoCientifica(MecXos_InerciasDosCorpos0[ContaCorpos][Indice],17), ", ")
        #declare Indice=Indice+1;
      #end
      #write(Filio, "}\n")
      #declare ContaCorpos=ContaCorpos+1;
    #end
    #write(Filio, "}\n#declare MecXos_CMs0=array[", str(QuantosCorpos,0,0), "]{\n  ")
    #local ContaCorpos=0;
    #while(ContaCorpos<QuantosCorpos)
      #write(Filio, NotacoCientificaVector(3,MecXos_CMs0[ContaCorpos],17), ", ")
      #declare ContaCorpos=ContaCorpos+1;
    #end
    #write(Filio, "\n}\n#declare MecXos_VelocidadesCM=array[", str(QuantosCorpos,0,0), "]{\n  ")
    #local ContaCorpos=0;
    #while(ContaCorpos<QuantosCorpos)
      #write(Filio, NotacoCientificaVector(3,MecXos_VelocidadesCM[ContaCorpos],17), ", ")
      #declare ContaCorpos=ContaCorpos+1;
    #end
    #write(Filio, "\n}\n#declare MecXos_Despracamentos=array[", str(QuantosCorpos,0,0), "]{\n  ")
    #local ContaCorpos=0;
    #while(ContaCorpos<QuantosCorpos)
      #write(Filio, NotacoCientificaVector(3,MecXos_Despracamentos[ContaCorpos],17), ", ")
      #declare ContaCorpos=ContaCorpos+1;
    #end
    #write(Filio, "\n}\n#declare MecXos_VelocidadesAngulares=array[", str(QuantosCorpos,0,0), "]{\n  ")
    #local ContaCorpos=0;
    #while(ContaCorpos<QuantosCorpos)
      #write(Filio, NotacoCientificaVector(3,MecXos_VelocidadesAngulares[ContaCorpos],17), ", ")
      #declare ContaCorpos=ContaCorpos+1;
    #end
    #write(Filio, "\n}\n#declare MecXos_Angulos=array[", str(QuantosCorpos,0,0), "]{\n  ")
    #local ContaCorpos=0;
    #while(ContaCorpos<QuantosCorpos)
      #write(Filio, NotacoCientificaVector(3,MecXos_Angulos[ContaCorpos],17), ", ")
      #declare ContaCorpos=ContaCorpos+1;
    #end
    #write(Filio, "\n}\n#declare MecXos_Tempo=", NotacoCientifica(Tempo,17), ";")
  #fclose Filio

#end

#macro MecXos_ComecaSimul()  //Subrutina de MovSolRigCampoDeForcas
  #declare MecXos_VelocidadesCM=VelocidadesCM0
  #declare MecXos_Despracamentos=array[QuantosCorpos]
  #declare MecXos_VelocidadesAngulares=VelocidadesAngulares0
  #declare MecXos_Angulos=array[QuantosCorpos]
  #declare MecXos_InerciasDosCorpos0=array[QuantosCorpos][7]
  #declare MecXos_CMs0=array[QuantosCorpos]
  #local ContaCorpos=0;
  #while(ContaCorpos<QuantosCorpos)
    #if(Xose_Mensagens)
      #debug concat( "\n  --------------------------------- Corpo numero ",
        str(ContaCorpos,0,0), " de ", str(QuantosCorpos,0,0), " ---------------------------------\n" )
      #if(MecXos_QuantosPontosFixos[ContaCorpos]>=1)
        #debug concat("    Ponto fixo ",
          NotacoCientificaVector(3,MecXos_PontosFixos[ContaCorpos][0],6), "\n")
        #if(MecXos_QuantosPontosFixos[ContaCorpos]=2)
          #local uEixoFixo =
            vnormalize(MecXos_PontosFixos[ContaCorpos][1]-MecXos_PontosFixos[ContaCorpos][0]);
          #debug concat("    Eixo fixo na direcco de ", NotacoCientificaVector(3,uEixoFixo,6), "\n")
        #end
      #else
        #debug concat( "    Velocidade inicial do centro de massas = ",
          NotacoCientificaVector(3,MecXos_VelocidadesCM[ContaCorpos],6), "\n" )
      #end
      #debug concat( "    Velocidade angular inicial = ",
        NotacoCientificaVector(3,MecXos_VelocidadesAngulares[ContaCorpos],6), "\n" )
    #end
    #declare MecXos_Despracamentos[ContaCorpos]=<0,0,0>;
    #declare MecXos_Angulos[ContaCorpos]=<0,0,0>;   //Se o corpo tem menos de 2 pontos fixos, ngulos de
                                           //Euler (<psi, theta, phi>); sen, <posi angular, 0, 0>
    #local ContaElmts=0;
    #declare MecXos_InerciasDosCorpos0[ContaCorpos][0]=0;
    #local Soma=<0,0,0>;
    #while(ContaElmts<QuantosElementos[ContaCorpos])
      #declare MecXos_InerciasDosCorpos0[ContaCorpos][0] =
        MecXos_InerciasDosCorpos0[ContaCorpos][0]+Inercias0[ContaCorpos][ContaElmts][0];
      #declare Soma=Soma+Posicoes0[ContaCorpos][ContaElmts]*Inercias0[ContaCorpos][ContaElmts][0];
      #declare ContaElmts=ContaElmts+1;
    #end
    #declare MecXos_CMs0[ContaCorpos]=Soma/MecXos_InerciasDosCorpos0[ContaCorpos][0];
    #if(Xose_Mensagens)
      #debug concat("    Massa total do solido = ",
        NotacoCientifica(MecXos_InerciasDosCorpos0[ContaCorpos][0],6), "\n")
      #debug concat( "    Posico inicial do centro de massas = ",
        NotacoCientificaVector(3,MecXos_CMs0[ContaCorpos],6), "\n" )
    #end
    #if(MecXos_QuantosPontosFixos[ContaCorpos]=0)
      #local PtoRef=MecXos_CMs0[ContaCorpos];
    #else
      #local PtoRef=MecXos_PontosFixos[ContaCorpos][0];
    #end
    #declare MecXos_InerciasDosCorpos0[ContaCorpos][1]=0;
    #declare MecXos_InerciasDosCorpos0[ContaCorpos][2]=0;
    #declare MecXos_InerciasDosCorpos0[ContaCorpos][3]=0;
    #declare MecXos_InerciasDosCorpos0[ContaCorpos][4]=0;
    #declare MecXos_InerciasDosCorpos0[ContaCorpos][5]=0;
    #declare MecXos_InerciasDosCorpos0[ContaCorpos][6]=0;
    #local ContaElmts=0;
    #while(ContaElmts<QuantosElementos[ContaCorpos])
      #local PosRel=Posicoes0[ContaCorpos][ContaElmts]-PtoRef;
      #local MElmt=Inercias0[ContaCorpos][ContaElmts][0];
      #declare MecXos_InerciasDosCorpos0[ContaCorpos][1] = MecXos_InerciasDosCorpos0[ContaCorpos][1] +
        Inercias0[ContaCorpos][ContaElmts][1] + MElmt*(pow(PosRel.y,2)+pow(PosRel.z,2));
      #declare MecXos_InerciasDosCorpos0[ContaCorpos][2] = MecXos_InerciasDosCorpos0[ContaCorpos][2] +
        Inercias0[ContaCorpos][ContaElmts][2] + MElmt*(pow(PosRel.x,2)+pow(PosRel.z,2));
      #declare MecXos_InerciasDosCorpos0[ContaCorpos][3] = MecXos_InerciasDosCorpos0[ContaCorpos][3] +
        Inercias0[ContaCorpos][ContaElmts][3] + MElmt*(pow(PosRel.x,2)+pow(PosRel.y,2));
      #declare MecXos_InerciasDosCorpos0[ContaCorpos][4] = MecXos_InerciasDosCorpos0[ContaCorpos][4] +
        Inercias0[ContaCorpos][ContaElmts][4] + MElmt*PosRel.x*PosRel.y;
      #declare MecXos_InerciasDosCorpos0[ContaCorpos][5] = MecXos_InerciasDosCorpos0[ContaCorpos][5] +
        Inercias0[ContaCorpos][ContaElmts][5] + MElmt*PosRel.x*PosRel.z;
      #declare MecXos_InerciasDosCorpos0[ContaCorpos][6] = MecXos_InerciasDosCorpos0[ContaCorpos][6] +
        Inercias0[ContaCorpos][ContaElmts][6] + MElmt*PosRel.y*PosRel.z;
      #declare ContaElmts=ContaElmts+1;
    #end
    #if(Xose_Mensagens)
      #if(MecXos_QuantosPontosFixos[ContaCorpos]=0)
        #local Faug=concat("centro de massas ", NotacoCientificaVector(3,PtoRef,6))
      #else
        #local Faug=concat("ponto fixo ", NotacoCientificaVector(3,PtoRef,6))
      #end
      #debug concat("\n    Tensor de inercia respeito do ", Faug, ", nos eixos OX, OY, OZ, no instante inicial:\n")
      #debug        "             ((                               ))\n"
      #debug concat("            ((   ", NotacoCientifica(MecXos_InerciasDosCorpos0[ContaCorpos][1],6), "  ", NotacoCientifica(-MecXos_InerciasDosCorpos0[ContaCorpos][4],6), "  ", NotacoCientifica(-MecXos_InerciasDosCorpos0[ContaCorpos][5],6), "   ))\n")
      #debug        "           ((                                   ))\n"
      #debug concat("           ((    ", NotacoCientifica(-MecXos_InerciasDosCorpos0[ContaCorpos][4],6), "  ", NotacoCientifica(MecXos_InerciasDosCorpos0[ContaCorpos][2],6), "  ", NotacoCientifica(-MecXos_InerciasDosCorpos0[ContaCorpos][6],6), "    ))\n")
      #debug        "           ((                                   ))\n"
      #debug concat("            ((   ", NotacoCientifica(-MecXos_InerciasDosCorpos0[ContaCorpos][5],6), "  ", NotacoCientifica(-MecXos_InerciasDosCorpos0[ContaCorpos][6],6), "  ", NotacoCientifica(MecXos_InerciasDosCorpos0[ContaCorpos][3],6), "   ))\n")
      #debug        "             ((                               ))\n\n"
    #end
    MecXos_ActualizaInfoCorpo()
    #if(Xose_Mensagens)
      #debug concat("                          O seu inverso:\n")
      #debug        "             ((                             ))\n"
      #debug concat("            ((   ", NotacoCientifica(MecXos_InvTensInercias[ContaCorpos](1,0,0).x,6), "  ", NotacoCientifica(MecXos_InvTensInercias[ContaCorpos](0,1,0).x,6), "  ", NotacoCientifica(MecXos_InvTensInercias[ContaCorpos](0,0,1).x,6), "   ))\n")
      #debug        "           ((                                 ))\n"
      #debug concat("           ((    ", NotacoCientifica(MecXos_InvTensInercias[ContaCorpos](1,0,0).y,6), "  ", NotacoCientifica(MecXos_InvTensInercias[ContaCorpos](0,1,0).y,6), "  ", NotacoCientifica(MecXos_InvTensInercias[ContaCorpos](0,0,1).y,6), "    ))\n")
      #debug        "           ((                                 ))\n"
      #debug concat("            ((   ", NotacoCientifica(MecXos_InvTensInercias[ContaCorpos](1,0,0).z,6), "  ", NotacoCientifica(MecXos_InvTensInercias[ContaCorpos](0,1,0).z,6), "  ", NotacoCientifica(MecXos_InvTensInercias[ContaCorpos](0,0,1).z,6), "   ))\n")
      #debug        "             ((                             ))\n"
    #end
    #declare ContaCorpos=ContaCorpos+1;
  #end
#end

#macro MecXos_ActualizaInfoCorpo()  //Subrutina de MovSolRigCampoDeForcas
  #declare MecXos_Rotacoes[ContaCorpos] =
    transform{
      #if(MecXos_QuantosPontosFixos[ContaCorpos]=2)
        RotVector(MecXos_Angulos[ContaCorpos].x*uEixoFixo)
      #else
        RotEuler( MecXos_Angulos[ContaCorpos].x, MecXos_Angulos[ContaCorpos].y,
          MecXos_Angulos[ContaCorpos].z )
      #end
    }
  #declare MecXos_Movimentos[ContaCorpos] =
    transform{
      translate -PtoRef MecXos_Rotacoes[ContaCorpos]
      translate PtoRef+MecXos_Despracamentos[ContaCorpos]
    }
  #local TransformInercia=transform{
    transform{MecXos_Rotacoes[ContaCorpos] inverse}
    matrix<  MecXos_InerciasDosCorpos0[ContaCorpos][1], -MecXos_InerciasDosCorpos0[ContaCorpos][4], -MecXos_InerciasDosCorpos0[ContaCorpos][5],
            -MecXos_InerciasDosCorpos0[ContaCorpos][4],  MecXos_InerciasDosCorpos0[ContaCorpos][2], -MecXos_InerciasDosCorpos0[ContaCorpos][6],
            -MecXos_InerciasDosCorpos0[ContaCorpos][5], -MecXos_InerciasDosCorpos0[ContaCorpos][6],  MecXos_InerciasDosCorpos0[ContaCorpos][3],
            0                                        , 0                                        , 0                                          >
    MecXos_Rotacoes[ContaCorpos]
  }
  #ifdef(MecXos_TensInercias[ContaCorpos])
    #undef MecXos_TensInercias[ContaCorpos]
    #undef MecXos_InvTensInercias[ContaCorpos]
  #end
  #declare MecXos_TensInercias[ContaCorpos]=function{transform{TransformInercia}}
  #declare MecXos_InvTensInercias[ContaCorpos]=function{transform{TransformInercia inverse}}
  #local fMov=function{transform{MecXos_Movimentos[ContaCorpos]}}
  #local fRot=function{transform{MecXos_Rotacoes[ContaCorpos]}}
  #local NoEsTartamudoPeroHablaDespacito=MecXos_CMs0[ContaCorpos];
  #declare MecXos_CMs[ContaCorpos]=fMov(NoEsTartamudoPeroHablaDespacito.x,
    NoEsTartamudoPeroHablaDespacito.y, NoEsTartamudoPeroHablaDespacito.z);
  #if(Xose_Mensagens)
    #debug concat("\n    Posico do centro de massas: ", NotacoCientificaVector(3,MecXos_CMs[ContaCorpos],6), "\n")
  #end
  #local ContaElmts=0;
  #while(ContaElmts<QuantosElementos[ContaCorpos])
    #local NoEsTartamudoPeroHablaDespacito=Posicoes0[ContaCorpos][ContaElmts];
    #declare MecXos_Posicoes[ContaCorpos][ContaElmts]=fMov( NoEsTartamudoPeroHablaDespacito.x,
      NoEsTartamudoPeroHablaDespacito.y, NoEsTartamudoPeroHablaDespacito.z );
    #if(MecXos_QuantosPontosFixos[ContaCorpos]=0)
      #declare MecXos_Velocidades[ContaCorpos][ContaElmts]=MecXos_VelocidadesCM[ContaCorpos] +
        vcross( MecXos_VelocidadesAngulares[ContaCorpos],
          MecXos_Posicoes[ContaCorpos][ContaElmts]-MecXos_CMs[ContaCorpos] );
    #else
      #declare MecXos_Velocidades[ContaCorpos][ContaElmts] =
        vcross( MecXos_VelocidadesAngulares[ContaCorpos],
          MecXos_Posicoes[ContaCorpos][ContaElmts]-PtoRef );
    #end
    #local Indice=0;
    #while(Indice<dimension_size(MecXos_VectsAgentes0,3))
      #local NoEsTartamudoPeroHablaDespacito=MecXos_VectsAgentes0[ContaCorpos][ContaElmts][Indice];
      #declare MecXos_VectsAgentes[ContaCorpos][ContaElmts][Indice]=fRot(
        NoEsTartamudoPeroHablaDespacito.x, NoEsTartamudoPeroHablaDespacito.y,
        NoEsTartamudoPeroHablaDespacito.z );
      #declare Indice=Indice+1;
    #end
    #declare ContaElmts=ContaElmts+1;
  #end
  #undef fMov #undef fRot
#end


// Macros que ajudam a trabalhar com MovSolRigCampoDeForcas

// Massa dum corpo
#macro MecXos_Massa(Corpo)
  MecXos_InerciasDosCorpos0[Corpo][0]
#end

//J continuarei se puider...


#version versionMecXos;