// Arquivo auxiliar para o traador de raios de Persistence of Vision
// Arquivo: Mecanismos.inc
// Vers: 3.5
// Descri: Diversos elementos mecnicos
// Data de criac: 14-IX-2002
// Autor: Xos B. Carballeira Rabual


#local versionMecanismos = version;
#version 3.5;

#ifndef(Xose_Mensagens)
  #declare Xose_Mensagens=false;
#end

#if(Xose_Mensagens)
  #debug "    Esta-se a executar o arquivo \"Mecanismos.inc\", que define alguns elementos mecanicos.\n"
#end

#macro RodaDentada(DiametroPrimitivo,Grossor,AlturaDosTriangulos,Rebaixes,NumeroDeDentes)
  #local Fanghuo=degrees(atan(DiametroPrimitivo*pi/NumeroDeDentes/2/AlturaDosTriangulos));
  #local Recorte=union{
    plane{y,.95*Rebaixes-AlturaDosTriangulos/2}
    plane{-x,0 rotate<0,0,-Fanghuo> translate DiametroPrimitivo*pi/NumeroDeDentes/4*x}
    plane{x,0 rotate<0,0,Fanghuo> translate -DiametroPrimitivo*pi/NumeroDeDentes/4*x}
    translate<0,DiametroPrimitivo/2,0>
  }
  intersection{
    cylinder{<0,0,-Grossor/2>, <0,0,Grossor/2>, (DiametroPrimitivo+AlturaDosTriangulos)/2-Rebaixes
    }
    #local ContaDentes=0;
    #while(ContaDentes<NumeroDeDentes)
      object{Recorte rotate<0,0,ContaDentes*360/NumeroDeDentes>}
      #declare ContaDentes=ContaDentes+1;
    #end
    bounded_by{
      box{<-(DiametroPrimitivo+AlturaDosTriangulos)/2+Rebaixes*.999,
          -(DiametroPrimitivo+AlturaDosTriangulos)/2+Rebaixes*.999,-Grossor/1.999>,
        <(DiametroPrimitivo+AlturaDosTriangulos)/2-Rebaixes*.999,
          (DiametroPrimitivo+AlturaDosTriangulos)/2-Rebaixes*.999,Grossor/1.999>
      }
    }
  }
#end

#macro Pinho(DiametroPrimitivo,GrossoDosDentes,LargoDaCadeia,Rebaixe,NumeroDeDentes) //H problemas.
  #local Oufa=pi/NumeroDeDentes;
  #local Rmax=DiametroPrimitivo*cos(Oufa)/(cos(Oufa)+1-sin(Oufa));
  //#debug concat("    Raio maximo: ", str(Rmax,0,8), "\n")
  difference{
    union{
      cylinder{-LargoDaCadeia/2*z, LargoDaCadeia/2*z, Rmax*(1/cos(Oufa)-tan(Oufa))}
      cylinder{-GrossoDosDentes/2*z, GrossoDosDentes/2*z, Rmax-Rebaixe}
    }
    #local ContaDentes=0;
    #while(ContaDentes<NumeroDeDentes)
      cylinder{<0,Rmax/cos(Oufa),-max(GrossoDosDentes,LargoDaCadeia)/1.999>,
        <0,Rmax/cos(Oufa),max(GrossoDosDentes,LargoDaCadeia)/1.999>, Rmax*tan(Oufa)
        rotate ContaDentes*360/NumeroDeDentes*z
      }
      #declare ContaDentes=ContaDentes+1;
    #end
  }
#end

#macro TramoRectoDeCadeiaDeTransmisso(ComprimentoDoEslabo,AltoDaMalha,GrossoDaMalha,NoDeMalhasPorEslabon,
  DiametroDoCilindro,Largo,MaterialDaMalha,MaterialDoCilindro,NoDeEslaboes)
  union{
    #local ContaEslaboes=0;
    #while(ContaEslaboes<NoDeEslaboes)
      cylinder{-(Largo/2+GrossoDaMalha)*z, (Largo/2+GrossoDaMalha)*z, DiametroDoCilindro/2
        material{MaterialDoCilindro}
        translate ContaEslaboes*ComprimentoDoEslabo*x
      }
      #local ContaMalhas=0;
      #while(ContaMalhas<NoDeMalhasPorEslabon)
        union{
          cylinder{-GrossoDaMalha/2*z, GrossoDaMalha/2*z, AltoDaMalha/2}
          box{<0,-AltoDaMalha/2,-GrossoDaMalha/2>, <ComprimentoDoEslabo,AltoDaMalha/2,GrossoDaMalha/2>}
          cylinder{<ComprimentoDoEslabo,0,-GrossoDaMalha/2>, <ComprimentoDoEslabo,0,GrossoDaMalha/2>,
            AltoDaMalha/2
          }
          material{MaterialDaMalha}
          translate<ContaEslaboes*ComprimentoDoEslabo,0,
            ((NoDeMalhasPorEslabon=1)? 0: Largo*(ContaMalhas/(NoDeMalhasPorEslabon-1)-1/2))>
        }
        #declare ContaMalhas=ContaMalhas+1;
      #end
      #declare ContaEslaboes=ContaEslaboes+1;
    #end
  }
#end
#macro ArcoDeCadeiaDeTransmisso(Raio,ComprimentoDoEslabo,AltoDaMalha,GrossoDaMalha,NoDeMalhasPorEslabon,
  DiametroDoCilindro,Largo,MaterialDaMalha,MaterialDoCilindro,NoDeEslaboes)
  union{
    #local ContaEslaboes=0;
    #while(ContaEslaboes<NoDeEslaboes)
      cylinder{-(Largo/2+GrossoDaMalha)*z, (Largo/2+GrossoDaMalha)*z, DiametroDoCilindro/2
        material{MaterialDoCilindro}
        translate Raio*y
        rotate -2*ContaEslaboes*degrees(asin(ComprimentoDoEslabo/Raio/2))*z
      }
      #local ContaMalhas=0;
      #while(ContaMalhas<NoDeMalhasPorEslabon)
        union{
          cylinder{-GrossoDaMalha/2*z, GrossoDaMalha/2*z, AltoDaMalha/2}
          box{<0,-AltoDaMalha/2,-GrossoDaMalha/2>, <ComprimentoDoEslabo,AltoDaMalha/2,GrossoDaMalha/2>}
          cylinder{<ComprimentoDoEslabo,0,-GrossoDaMalha/2>, <ComprimentoDoEslabo,0,GrossoDaMalha/2>,
            AltoDaMalha/2
          }
          material{MaterialDaMalha}
          rotate -degrees(asin(ComprimentoDoEslabo/Raio/2))*z
          translate<0,Raio,
            ((NoDeMalhasPorEslabon=1)? 0: Largo*(ContaMalhas/(NoDeMalhasPorEslabon-1)-1/2))>
          rotate -2*ContaEslaboes*degrees(asin(ComprimentoDoEslabo/Raio/2))*z
        }
        #declare ContaMalhas=ContaMalhas+1;
      #end
      #declare ContaEslaboes=ContaEslaboes+1;
    #end
  }
#end

#version versionMecanismos;

