// Arquivo auxiliar para o traador de raios de Persistence of Vision
// Arquivo: Objectos luminosos.inc
// Vers: 3.5
// Descri: Objectos para alumear
// Data de cria: 5-VI-2003
// Data da ltima actualiza: _
// Autor: Xos B. Carballeira Rabual; emprego o sistema de partculas de Rune S. Johansen, a quem agrade-
                                                                                  //o a sua generosidade.


#local versionAlucinhas=version;
#version 3.5;

#ifndef(Xose_Mensagens)
  #declare Xose_Mensagens=false;
#end

#if(Xose_Mensagens)
  #debug "    Esta-se a executar o arquivo \"Objectos luminosos.inc\", que define objectos para alumear.\n"
#end

//Requere:
//#include "Materiais varios.inc"  //Ags quando Xose_SemTexturas=true
//#include "Corpos lineares.inc"
//#include "particle.inc"  //O sistema de partculas de Rune S. Johansen


#ifndef(Xose_SemTexturas)
  #declare Xose_SemTexturas=false;
#end

// Uma vela de cera, com lapa e fume animveis e com rebabas fixas; n se consume.
// A macro responde s seguintes variveis:
//   Vela_LisuraDasRebabas;
//   Vela_IrregLongRebabas;
//   Vela_IrregGrosRebabas;
//   Vela_IrregDireccoRebabas;
//   Vela_IrregMecha;
//   Vela_ComecoLume = valor de clock no que abrolha a chama, 0 por omiss;
//   Vela_RemateLume = valor de clock no que se extingue a chama, 1 por omiss;
//   Vela_ComecoFume = valor de clock no que comea o fume, 0 por omiss;
//   Vela_RemateFume = valor de clock no que deixa de sair fume, 1 por omiss;
//   Vela_Ciclo = valor booleano que decide se a emiss de fume e lume  cclica; nesse caso, as quatro
//     variveis anteriores delimitam o primeiro ciclo, n todo o processo; por omiss, n haver
//     ciclo;
//   Vela_FreqCalcLume = nmero de clculos do movimento das particulas de lume por unidade de clock,
//     100 por omiss;
//   Vela_FreqCalcFume = nmero de clculos do movimento das particulas de fume por unidade de clock,
//     100 por omiss;
//   Vela_ArrastoLume = coeficiente de arrasto do lume polo vento, 10 por omiss;
//   Vela_ArrastoFume = coeficiente de arrasto do fume polo vento, 20 por omiss;
//   Vela_ObjBloq = objecto que bloqueia o lume e mais o fume;
//   Vela_SbmedidaBloq = sobremedida empregada nos clculos do bloqueio;
//   Vela_CorDaChama = cor principal da chama, <1,.6,.3> por omiss;
//   Vela_FulgorChama = cor que se engadir  chama no seu interior;
//   Vela_CorDoFume;
//   Vela_DensFume = densidade do fume, .16/GrosMecha por omiss;
//   Vela_DispersFume = ritmo de dispers do fume no ar;
//   Vela_NoMostrasLume = nmero de mostras por intervalo ao desenhar o lume, 1 por omiss;
//   Vela_NoMostrasFume = nmero de mostras por intervalo ao desenhar o fume, 1 por omiss;
//   Vela_AlongLume = coeficiente de alongamento das partculas de lume com a velocidade, 1 por omiss;
//   Vela_FactorLuz = coeficiente que multiplica a intensidade da fonte de luz associada  lapa.

// E tambm s macros:
//   Vela_GravLume(Tempo,Posico) = acelera gravitatria do lume, 10*y por omiss;
//   Vela_GravFume(Tempo,Posico) = acelera gravitatria do fume, 5*y por omiss;
//   Vela_Vento(Tempo,Posico) = velocidade do vento, <0,0,0> por omiss;
//   Vela_Transformaco(Tempo) = transforma para aplicar  vela, que pode depender do tempo; des-
//     aconselha-se transformar manualmente o resultado da macro Vela, pois o fume e mais a lapa n
//     reflectiriam a inrcia nem a mudana de orienta;
//   Vela_ObjExtint(Tempo) = Objecto que extingue o fume e mais o lume.
#macro Vela(Altura, Diametro, NoDeEstrias, NoDeRebabas, LongRebabas, GrosRebabas, AltMecha, GrosMecha,
  QuantaChama, NomeLume, NomeFume, Semente)
  #ifndef(Vela_CorDaChama) #declare Vela_CorDaChama=<1,.6,.3>; #end
  #local CorDaChama=QuantaChama*Vela_CorDaChama;
  #ifndef(Vela_LisuraDasRebabas) #declare Vela_LisuraDasRebabas=4; #end
  #ifndef(Vela_IrregLongRebabas) #declare Vela_IrregLongRebabas=.5; #end
  #ifndef(Vela_IrregGrosRebabas) #declare Vela_IrregGrosRebabas=.3; #end
  #ifndef(Vela_IrregDireccoRebabas) #declare Vela_IrregDireccoRebabas=.1; #end
  #ifndef(Vela_IrregMecha) #declare Vela_IrregMecha=.2; #end
  #ifndef(Vela_FactorLuz) #declare Vela_FactorLuz=1; #end
  #local IntensChama=( max(abs(CorDaChama.red),abs(CorDaChama.green),abs(CorDaChama.blue)) +
    min(abs(CorDaChama.red),abs(CorDaChama.green),abs(CorDaChama.blue)) ) / 2;
  #if(IntensChama!=0) union{ #end
    union{
      difference{
        union{
          cylinder{0, Altura*y, Diametro/2}
          #if(NoDeEstrias!=0)
            #local Estria=cylinder{Diametro/2*x, <Diametro/2,Altura,0>, Diametro*sin(pi/(2*NoDeEstrias))}
            #local ContaEstrias=0;
            #while(ContaEstrias<NoDeEstrias)
              object{Estria rotate 360*ContaEstrias/NoDeEstrias*y}
              #declare ContaEstrias=ContaEstrias+1;
            #end
          #end
          #local Gargantou=seed(Semente);
          #local ContaRebabas=0;
          #while(ContaRebabas<NoDeRebabas)
            #local TrengueleTrenguele=0;
            #local VouPorAqui=<0,Altura>;
            #local ContaBolinhas=0;
            #local NoDeBolinhas=LongRebabas*(1+Vela_IrregLongRebabas*(rand(Gargantou)-.5)) *
              Vela_LisuraDasRebabas/GrosRebabas;
            union{
              #while(ContaBolinhas<NoDeBolinhas)
                #declare TrengueleTrenguele=TrengueleTrenguele+rand(Gargantou)-.5;
                #declare VouPorAqui=VouPorAqui+GrosRebabas/Vela_LisuraDasRebabas *
                  vnormalize(<Vela_IrregDireccoRebabas*TrengueleTrenguele,-1>);
                sphere{<Diametro/2,VouPorAqui.y,0>,
                  (1+Vela_IrregGrosRebabas*(rand(Gargantou)-.5))*GrosRebabas
                  rotate 360*VouPorAqui.x/(pi*Diametro)*y
                }
                #declare ContaBolinhas=ContaBolinhas+1;
              #end
              rotate 360*rand(Gargantou)*y
            }
            #declare ContaRebabas=ContaRebabas+1;
          #end
        }
        sphere{(Altura+Diametro)*y, 1.3*Diametro}
        #if(!Xose_SemTexturas) texture{Cera pigment{quick_colour rgb 1}} #end
      }
      #local Bamboleo=array[4][8]
      #local Indice=0;
      #local Fact=function(x){prod(i,2,x,i)};
      #while(Indice<8)
        #local Sabrangoncio=0;
        #while(Sabrangoncio<4)
          #declare Bamboleo[Sabrangoncio][Indice]=(2*rand(Gargantou)-1)/Fact(Indice+1);
          #declare Sabrangoncio=Sabrangoncio+1;
        #end
        #declare Indice=Indice+1;
      #end
      #undef Fact
      #macro CorpLin_LinhaCentral(Parametro)
         ( (Altura-Diametro)*y +
            ( (Diametro+AltMecha)*y
              #local Indice=0;
              #while(Indice<8)
                + < Bamboleo[0][Indice]*cos(2*pi*Parametro*(Indice+1)) +
                    Bamboleo[1][Indice]*sin(2*pi*Parametro*(Indice+1)), 0,
                    Bamboleo[2][Indice]*cos(2*pi*Parametro*(Indice+1)) +
                    Bamboleo[3][Indice]*sin(2*pi*Parametro*(Indice+1)) > * Vela_IrregMecha*AltMecha
                #declare Indice=Indice+1;
              #end
            ) * Parametro
          )
      #end
      #local Metxa=Tranca(0,1,GrosMecha/(AltMecha+Diametro),2,1e-6,4,GrosMecha/3,.6*AltMecha/GrosMecha,1,1)
      object{Metxa #if(!Xose_SemTexturas) texture{pigment{rgb .1} finish{specular .8 roughness .01}} #end}
      #ifdef(Vela_Transformaco)
        Vela_Transformaco(clock)
      #else
        #macro Vela_Transformaco(Tempo) #end
      #end
    }

    // Chama
    #if(IntensChama!=0)
      #local CorDaChama=CorDaChama/IntensChama;
      #declare particle_start = #ifdef(Vela_ComecoLume) Vela_ComecoLume; #else 0; #end
      #declare particle_end = #ifdef(Vela_RemateLume) Vela_RemateLume; #else 1; #end
      #ifdef(Vela_Ciclo) #declare particle_cyclic=Vela_Ciclo; #end
      #ifdef(Vela_FreqCalcLume) #declare particle_steps=Vela_FreqCalcLume; #end
      #ifdef(Vela_GravLume)
        #macro particle_gravity(Tempo,Posico) Vela_GravLume(Tempo,Posico) #end
      #else
        #macro particle_gravity(Tempo,Posico) 10*y #end
      #end
      #declare particle_drag = #ifdef(Vela_ArrastoLume) Vela_ArrastoLume; #else 10; #end
      #ifdef(Vela_Vento)
        #macro particle_wind(Tempo,Posico) Vela_Vento(Tempo,Posico) #end
      #else
        #macro particle_wind(Tempo,Posico) <0,0,0> #end
      #end
      #local Midel=(Altura+AltMecha/2)*y;
      #local RapLim=sqrt( (vlength(particle_gravity(particle_start,Midel)) +
        vlength(particle_gravity(particle_end,Midel)))/(2*particle_drag) );
      #declare particle_life=IntensChama*14*Diametro/RapLim;
      #declare particle_lifeturb=0;
      #declare particle_frequency=IntensChama*40/particle_life;
      #declare particle_seed=rand(Gargantou);
      #declare particle_transfer=1;
      #macro particle_emitter(Tempo) <0,0,0> #end
      #macro particle_emitting(Tempo) on #end
      #macro particle_emitvect(Tempo) <0,0,0> #end
      #macro particle_emitturb(Tempo) 0 #end
      #declare Metxa=difference{object{Metxa} plane{y, Altura}}
      #macro particle_emitobj(Tempo) object{Metxa Vela_Transformaco(clock)} #end
      #macro particle_emitobjn (Tempo) 0.0 #end
      #ifdef(Vela_ObjBloq) #declare particle_blockobj=object{Vela_ObjBloq} #end
      #declare particle_bounce=0; #declare particle_friction=0;
      #ifdef(Vela_SbmedidaBloq) #declare particle_bounceoffset=Vela_SbmedidaBloq; #end
      #ifdef(Vela_ObjExtint) #macro particle_killobj(Tempo) Vela_ObjExtint(Tempo) #end #end
      particle_system(NomeLume)
      #include "fire.inc"
      #declare fire_method=1;
      #declare fire_color=CorDaChama;
      #declare fire_intensity=.03/(GrosMecha*IntensChama);
      #declare fire_highlight =
        #ifdef(Vela_FulgorChama) Vela_FulgorChama; #else <-2,-.5,2>/sqrt(IntensChama); #end
      #ifdef(Vela_NoMostrasLume) #declare fire_samples=Vela_NoMostrasLume; #end
      #declare fire_size=4*GrosMecha*IntensChama;
      #declare fire_turbulence=.3*fire_size;
      #ifndef(Vela_AlongLume) #declare Vela_AlongLume=3/RapLim; #end
      #declare fire_stretch=Vela_AlongLume;
      fire_create(NomeLume)
      #if(vlength(Vela_FactorLuz*fire_light)!=0)
        light_source{
          fire_center colour Vela_FactorLuz*.0000001*fire_light*CorDaChama/pow(GrosMecha,2)
          fade_distance 1000*GrosMecha fade_power 2
          media_interaction on media_attenuation on
        }
      #end

      // Fume
      #declare particle_start = #ifdef(Vela_ComecoFume) Vela_ComecoFume; #else 0; #end
      #declare particle_end = #ifdef(Vela_RemateFume) Vela_RemateFume; #else 1; #end
      #ifdef(Vela_FreqCalcFume) #declare particle_steps=Vela_FreqCalcFume; #end
      #ifdef(Vela_GravFume)
        #macro particle_gravity(Tempo,Posico) Vela_GravFume(Tempo,Posico) #end
      #else
        #macro particle_gravity(Tempo,Posico) 5*y #end
      #end
      #declare particle_drag = #ifdef(Vela_ArrastoLume) Vela_ArrastoFume; #else 20; #end
      #local RapLim=sqrt( (vlength(particle_gravity(particle_start,Midel)) +
        vlength(particle_gravity(particle_end,Midel)))/(2*particle_drag) );
      #declare particle_frequency=.1*RapLim/GrosMecha;
      #declare particle_life=particle_end-particle_start;
      #include "smoke.inc"
      #ifdef(Vela_CorDoFume) #declare smoke_color=Vela_CorDoFume; #end
      #ifndef(Vela_DensFume)
        #declare Vela_DensFume=.16/GrosMecha;
      #end
      #declare smoke_absorption=1*Vela_DensFume; #declare smoke_emission=.04*Vela_DensFume;
      #declare smoke_scattering=0;
      #declare smoke_startsize=2*GrosMecha;
      #ifndef(Vela_DispersFume) #declare Vela_DispersFume=10; #end
      #declare smoke_endsize=(1+Vela_DispersFume*particle_life)*2*GrosMecha;
      #ifdef(Vela_NoMostrasFume) #declare smoke_samples=Vela_NoMostrasFume; #end
      #declare smoke_intervals=1;
      #macro particle_element() smoke_element() #end
      particle_system(NomeFume)

  }
    #end
#end

#version versionAlucinhas;