// Arquivo auxiliar para o traador de raios de Persistence of Vision
// Arquivo: Ondas.inc
// Vers: 3.6
// Descri: Simula ondas que verifiquem a equa diferencial geral seguinte num domnio ortodrico:

  //  _2                           _
  //  d Elonga                   d Elonga                                  2 _2
  //  + Amortecimento  + RigResorte Elonga - Rapidez0  V Elonga = Perturba
  //   _      2                     _
  //   d Tempo                      d Tempo

  //      _                                   _2
  // onde d  o smbolo da derivada parcial e V  o do laplaciano.

// Data de cria: 14-VIII-2004; procede do velho Lamina elastica.inc, criado em 15-I-2003 e actualizado
                                                         //pola derradeira vez em 10-VIII-2004
// Data da ltima actualiza: _
// Autor: Xos B. Carballeira Rabual


#local versionLamElast=version;
#version 3.6;

//Requer:
//#include "Matematica.inc"

#ifndef(Xose_Mensagens)
  #declare Xose_Mensagens=false;
#end

#if(Xose_Mensagens)
  #debug "    Esta-se a executar o arquivo \"Ondas.inc\", para simular fenomenos ondulatorios.\n"
#end

#ifndef(Xose_PastaTemp)
  #declare Xose_PastaTemp=""
#end

//********************************** PARA REVISAR *****************************************************
//  A seguinte macro cria um pigmento Ondas_Pigmento e mais uma fun Ondas_Funco(x,y,z) que
//atribuem (mediante a luminosidade em gris ou com o valor devolto, respectivamente) a cada ponto dum dom-
//nio ortodrico a sua elonga em cada instante, calculada aproximadamente por mtodos numricos.
//*****************************************************************************************************

//  Os parmetros s:
//    Nome . O nome do arquivo temporal no que se armazenarm os dados das ondas elsticas. A se emprega-
//                   rem mais dum sistema de ondas na cena, cada um dever levar um nome diferente.
//    DimsDominio . As dimenses do domnio, na forma dum vector: <largo, alto, longo>
//    DensLinCalc . Densidade linear, segundo cada uma das direces principais, da trama cbica de pon-
//                   tos nos que se calcular a elonga; igual ao inverso da distncia entre duas filas,
//                   colunas ou camadas adjacentes.
//                   Nos outros pontos, interpolar-se- trilinearmente. De momento, nenguma das dimenses
//                   da trama igual ao produto da dimens do domnio que corresponder por DensLinCalc
//                   pode ser maior que 256 (mximo nmero de entradas num mapa de cores do POV-Ray), mas
//                   talvez arranje isto algum dia.
//    FreqCalc . Nmero de instantes que se considerarm para o clculo em cada unidade de tempo.
//    Rapidez0 . Rapidez com a que se propagariam as ondas se fosse Amortecimento=0 e RigResorte=0.
//    Amortecimento . Coeficiente de amortecimento, igual  decelera que o tal efeito provoca sobre
//                  cada ponto dividida pola velocidade deste. O coeficiente de absor (mngua relativa
//                  da intensidade das ondas com o caminho percorrido) relaciona-se com o de amortecimento,
//                  no caso das ondas senoidais de freqncia FreqOnda, do seguinte jeito:

//                                   ___________________________________________________________________
//                                  /                 ________________________________
//                           1     /              (  /    2         2                2                 )
//             Absor =  V 4 pi FreqOnda ( V 4 pi  FreqOnda  + Amortecimento  - 2 pi FreqOnda )
//                       Rapidez0

//    RigResorte . Rigidez dum resorte fictcio que segura cada ponto do domnio  origem da sua elonga.
//    QuantosGrausDeLiberdade . Nmero de dimenses nas quais poder oscilar cada ponto do domnio: 1, 2,
//                  3, 4 ou 5. Esse ser o nmero de componentes da elonga.
//    Tempo . Tempo do instante da chamada actual  macro.
//    Tempo0 . Tempo do instante no que se chamou a macro pola primeira vez.

//  Cumpre que DensLinCalc>=4/LongOnda, e que FreqCalc/DensLinCalc>Rapidez0, sendo LongOnda a mnima longitu-
//de previsvel para as ondas que se ham-de simular; isto implica, alis, que FreqCalc>4FreqOnda, onde
//chamamos FreqOnda a freqncia mxima que caiba esperar das ondas. Noutro caso, podem surgir erros consi-
//derveis. Reparemos em que se se violar a segunda inequa, a lentid do modelo impedir s frontes de
//onda avanarem ao seu passo normal, j que segundo o mtodo empregado, os cmbios que acontecem num tem-
//po 1/FreqCalc em cada ponto da trama s levam na conta os seis pontos vizinhos, polo que nenguma pertur-
//ba se propagar com celeridade maior ca FreqCalc/DensLinCalc, isto , as ondas n podem percorrer, en-
//tre dous clculos, mais distncia que a que separa dous pontos consecutivos da trama. Se isto for incom-
//patvel com o valor do parmetro Rapidez0, as perturbaes poder-se-am acumular em determinadas zonas,
//ao n se dar propagado com eficcia, ocasionando que terminem por se disparar caoticamente os valores ab-
//solutos das elongaes em todo o domnio.
//  Antes de usar Ondas, h que definir a perturba que sofrer cada ponto em cada instante
//com uma macro Ondas_Perturbaco(<PosicoX, PosicoY, PosicoZ>, Tempo), que devolver a acelera adicio-
//nal, forada, do ponto.
//  Para que as elongaes e mais as velocidades iniciais dos pontos do domnio n sejam todas nulas, po-
//dem-se definir:
//    1) As matrizes tridimensionais Ondas_Elongacoes0 e Ondas_Velocidades0, que devem conter as
//        elongaes iniciais e as velocidades iniciais, respectivamente, dos pontos de controlo do dom-
//        nio. Estes espargem-se numa trama cbica, segundo os parmetros DimsDominio e DensLinCalc. Os
//        dados de cada um situar-se-am no elemento de ndices [no_ordem_largo][no_ordem_alto]
//        [no_ordem_longo], sendo o primeiro o nmero de ordem do ponto na trama no sentido positivo do
//        eixo OX, o segundo, no do OY, e o terceiro, no do OZ. Todos trs comeam em 0.
//    2) Alternativamente a cada uma das matrizes anteriores, as macros Ondas_MacroElongaco0(<x,y,z>)
//        e Ondas_MacroVelocidade0(<x,y,z>), que ham de devolver, respectivamente, a elonga e mais a
//        velocidade iniciais do ponto (x,y,z) do domnio. Cada uma destas macros s ser efectiva se
//        n se definir a matriz correspondente, pois esta ltima  prioritria ante um conflito.
//  Devido  curiosa feitura dos pigmentos, precisa-se estabelecer uns valores mximo e mnimo para cada
//componente das elongaes; fora do paraleleppedo assim definido, truncar-se-am aos valores aceitveis
//mais prximos. As variveis Ondas_MaxElong e Ondas_MinElong permitem ajustar estas quantidades; os seus
//valores predeterminados s 1 e -1, respectivamente.

//  Podem-se impor condies de contorno s ondas. Para isso cumpre definir a matriz unidimensional
//Ondas_Contorno, com seis elementos, referentes a cadansua cara do domnio, segundo os ndices:
//    0) cara x = 0; 1) cara x = DimsDominio.x; 2) cara y = 0; 3) cara y = DimsDominio.y;
//    4) cara z = 0; 5) cara z = DimsDominio.z
//  Cada um destes elementos h-de ser um vector com duas componentes booleanas, que indiquem se a cara em
//quest do domnio tem condi de elonga (a primeira componente) ou de pendente normal (a segunda). O
//primeiro caso  o dum meio elstico encerrado por paredes rgidas, nas quais as ondas s inteiramente
//reflectidas com oposi de fase; o segundo serve para simular a superfcie livre dum lquido.
//  As condies devem-se estabelecer definindo as macros Ondas_ElongContorno(<PosicoX, PosicoY,
//PosicoZ>, Tempo) e Ondas_PendContorno(<PosicoX, PosicoY, PosicoZ>, Tempo).

//  Tambm se pode forar a integral de volume das elongaes sobre o domnio para um valor dado em ca-
//da instante, definindo a macro Ondas_Integral(Tempo); isto ajuda a simular enchimentos e esvazia-
//mentos de depsitos de auga, assim como a manter constante o volume de lquido numa poa.

//   possvel aplicar uma transforma ao domnio definindo a macro Ondas_Movimento(Tempo). Supe-se que o dom-
//nio tem no comeo um vrtice na origem e uma aresta em cada semieixo positivo, OX (largo), OY (alto) e
//OZ (longo). Cada ponto conservar a sua elonga aps ser transformado. Para obter uma simula realista,
//aconselha-se utilizar apenas transformaes compostas de transla e rota, sem escalas.

//  A forma necessariamente ortodrica do domnio limita muito as possibilidades, mas espero melhorar
//isto algum dia.

//  Cuido que o aumento indefinido da quantidade de memria ocupada polo POV-Ray enquanto gera uma anima-
// como as Ondinhas se deve a algum erro no programa, que atinge o armazenamento das funes defini-
//das polo usurio; a memria destinada a estas n semelha liberar-se eficazmente entre quadro e quadro.
//Talvez seja boa ideia, logo, destruir sempre manualmente todas as funes empregadas na cena, uma vez
//cumpridos os seus objectivos; para isto engadi, ao remate da defini da macro Ondas, a ordem #undef
//UnaCabura, desactivada. A prxima vez que se utilizar esta macro, conviria buscar alguma melhora com
//a referida instru activada, e assemade com a linha #undef Ondas_Funco nalgum lugar do arquivo prin-
//cipal da cena, posterior ao derradeiro uso de Ondas_Funco; naturalmente, se esse arquivo ou algum outro
//que se executar definirem mais funes, cumprir elimin-las todas com #undef para a prova ser efectiva.


#macro Ondas(Nome, DimsDominio, DensLinCalc, FreqCalc, Rapidez0, Amortecimento, RigResorte,
  QuantosGrausDeLiberdade, Tempo, Tempo0)

  #local NoFilas=ceil(DimsDominio.x*DensLinCalc)+1;
  #local NoCols=ceil(DimsDominio.y*DensLinCalc)+1;
  #local NoCams=ceil(DimsDominio.z*DensLinCalc)+1;

  #if(strlen(Xose_PastaTemp)=0)
    #local NomeDoFilego=concat(Nome, ".xose");
    #warning "Ao no se indicar, mediante a variavel Xose_PastaTemp, uma pasta para armazenar os dados\n  temporais (arquivos \".xose\"), empregar-se-a a actual."
    #warning "V. \"Das variaveis xosianas.txt\""
    #warning "Existe o risco de que esta cambie durante o processo e se perda informaco."
    #warning "Isto pode ocasionar erros na simulaco das ondas."
    #warning "Desaconselha-se, de prosseguir, toda acco que poida mudar a pasta actual, seja ou no com o POV-Ray,\n  ate se completarem todos os quadros."
  #else
    #local NomeDoFilego=concat(Xose_PastaTemp, "\\", Nome, ".xose")
  #end

  #ifndef(Ondas_Contorno) #declare Ondas_Contorno=array[6]{<0,0> <0,0> <0,0> <0,0> <0,0> <0,0>} #end
  #if(NoFilas=1)  //Se houver apenas uma fila, n se poderm impor condies de pendente normal
    #declare Ondas_Contorno[0]=Ondas_Contorno[0]*<1,0>;                        //nos lados x=cte
    #declare Ondas_Contorno[1]=Ondas_Contorno[1]*<1,0>;
  #end
  #if(NoCols=1)  //Se houver apenas uma coluna, n se poderm impor condies de pendente normal
    #declare Ondas_Contorno[2]=Ondas_Contorno[2]*<1,0>;                         //nos lados y=cte
    #declare Ondas_Contorno[3]=Ondas_Contorno[3]*<1,0>;
  #end
  #if(NoCams=1)  //Se houver apenas uma camada, n se poderm impor condies de pendente normal
    #declare Ondas_Contorno[4]=Ondas_Contorno[4]*<1,0>;                         //nos lados z=cte
    #declare Ondas_Contorno[5]=Ondas_Contorno[5]*<1,0>;
  #end
  #switch(QuantosGrausDeLiberdade)
    #case(1)
      #local Zero=0;
    #break
    #case(2)
      #local Zero=<0,0>;
    #break
    #case(3)
      #local Zero=<0,0,0>;
    #break
    #case(4)
      #local Zero=<0,0,0,0>;
    #break
    #case(5)
      #local Zero=<0,0,0,0,0>;
    #break
    #else
      #warning "O numero de graus de liberdade das ondas deve ser um inteiro\n  compreendido entre 1 e 5, ambos inclusive."
      #warning "O valor fornecido no serve; considerar-se a um so grau de liberdade."
      #local Zero=0;
  #end
  #ifdef(Ondas_Integral) #local SomaElongacoes=Zero; #end
  #ifndef(Ondas_Movimento)
    #macro Ondas_Movimento(Teimi)
      transform{}
    #end
  #end
  #macro ActualizaFMovimento(Teimi)
    #undef Ondas_FMovimento
    #declare Ondas_FMovimento=function{transform{Ondas_Movimento(Teimi)}}
  #end

  #if(Tempo=Tempo0)

    //Instante inicial
    #if(Xose_Mensagens)
      #debug concat("    Estam-se a guardar as condicoes iniciais do sistema \"", Nome, "\".\n")
    #end
    ActualizaFMovimento(Tempo)
    #declare Ondas_Elongacoes=array[2][NoFilas][NoCols][NoCams]
    #local Fila=0;
    #while(Fila<NoFilas)
      #local Col=0;
      #while(Col<NoCols)
        #local Cam=0;
        #while(Cam<NoCams)
          #ifdef(Ondas_Elongacoes0)
            #declare Ondas_Elongacoes[1][Fila][Col][Cam]=Ondas_Elongacoes0[Fila][Col][Cam];
          #else
            #ifdef(Ondas_MacroElongaco0)
              #declare Ondas_Elongacoes[1][Fila][Col][Cam] =
                Ondas_MacroElongaco0(Ondas_FMovimento(Fila/DensLinCalc,Col/DensLinCalc,Cam/DensLinCalc));
            #else
              #declare Ondas_Elongacoes[1][Fila][Col][Cam]=Zero;
            #end
          #end
          #ifdef(Ondas_Integral)
            #declare SomaElongacoes=SomaElongacoes+Ondas_Elongacoes[1][Fila][Col][Cam];
          #end
          #ifdef(Ondas_Velocidades0)
            #declare Ondas_Elongacoes[0][Fila][Col][Cam]=Ondas_Elongacoes[1][Fila][Col][Cam] -
              Ondas_Velocidades0[Fila][Col][Cam]/FreqCalc;
          #else
            #ifdef(Ondas_MacroVelocidade0)
              #declare Ondas_Elongacoes[0][Fila][Col][Cam]=Ondas_Elongacoes[1][Fila][Col][Cam] -
                Ondas_MacroVelocidade0(Ondas_FMovimento(Fila/DensLinCalc,Col/DensLinCalc,Cam/DensLinCalc))
                / FreqCalc;
            #else
              #declare Ondas_Elongacoes[0][Fila][Col][Cam]=Ondas_Elongacoes[1][Fila][Col][Cam];
            #end
          #end
          #declare Cam=Cam+1;
        #end
        #declare Col=Col+1;
      #end
      #declare Fila=Fila+1;
    #end
    #ifdef(Ondas_Integral) #declare Ondas_ElongacoMedia=SomaElongacoes/(NoFilas*NoCols*NoCams); #end
    #declare Ondas_Vez=0;

  #else

    #if(Xose_Mensagens)
      #debug concat("    Estam-se a calcular as pequenas deformacoes do sistema \"", Nome, "\".\n")
    #end
    #include NomeDoFilego
    #local Tempus=Ondas_Tempo;
    #local FilaInf=(Ondas_Contorno[0].u? 1: 0)+(Ondas_Contorno[0].v? 1: 0);
    #local FilaSup=NoFilas-(Ondas_Contorno[1].u? 1: 0)-(Ondas_Contorno[1].v? 1: 0);
    #local ColInf=(Ondas_Contorno[2].u? 1: 0)+(Ondas_Contorno[2].v? 1: 0);
    #local ColSup=NoCols-(Ondas_Contorno[3].u? 1: 0)-(Ondas_Contorno[3].v? 1: 0);
    #local CamInf=(Ondas_Contorno[4].u? 1: 0)+(Ondas_Contorno[4].v? 1: 0);
    #local CamSup=NoCams-(Ondas_Contorno[5].u? 1: 0)-(Ondas_Contorno[5].v? 1: 0);

    #while(Tempus<Tempo)

      ActualizaFMovimento(Tempus)

      // Pontos interiores
      
      #local Fila=max(FilaInf,1);
      #while(Fila<min(FilaSup,NoFilas-1))
        #local Col=max(ColInf,1);
        #while(Col<min(ColSup,NoCols-1))
          #local Cam=max(CamInf,1)
          #while(Cam<min(CamSup,NoCams-1))
            #declare Ondas_Elongacoes[Ondas_Vez][Fila][Col][Cam] =
              pow(Rapidez0*DensLinCalc/FreqCalc,2) *
                ( Ondas_Elongacoes[!Ondas_Vez][Fila-1][Col][Cam] +
                  Ondas_Elongacoes[!Ondas_Vez][Fila+1][Col][Cam] +
                  Ondas_Elongacoes[!Ondas_Vez][Fila][Col+1][Cam] +
                  Ondas_Elongacoes[!Ondas_Vez][Fila][Col-1][Cam] +
                  Ondas_Elongacoes[!Ondas_Vez][Fila][Col][Cam+1] +
                  Ondas_Elongacoes[!Ondas_Vez][Fila][Col][Cam-1] -
                  6*Ondas_Elongacoes[!Ondas_Vez][Fila][Col][Cam]
                ) +
              Ondas_Perturbaco(Ondas_FMovimento(Fila/DensLinCalc,Col/DensLinCalc,Cam/DensLinCalc),Tempus)
              / pow(FreqCalc,2) +
              Ondas_Elongacoes[!Ondas_Vez][Fila][Col][Cam]*(1-RigResorte/pow(FreqCalc,2)) +
              ( Ondas_Elongacoes[!Ondas_Vez][Fila][Col][Cam] -
                Ondas_Elongacoes[Ondas_Vez][Fila][Col][Cam] ) * (1-Amortecimento/FreqCalc)
              #ifdef(Ondas_Integral)
                + Ondas_Integral(Tempus)/(DimsDominio.x*DimsDominio.y*DimsDominio.z)-Ondas_ElongacoMedia;
                #declare SomaElongacoes=SomaElongacoes+Ondas_Elongacoes[Ondas_Vez][Fila][Col][Cam];
              #else
                ;
              #end
            #declare Cam=Cam+1;
          #end
          #declare Col=Col+1;
        #end
        #declare Fila=Fila+1;
      #end

      // Condies de contorno
      #local Fila=FilaInf;
      #while(Fila<FilaSup)
        #local Col=ColInf
        #while(Col<ColSup)
          #if(NoCams>1)
            #if(Ondas_Contorno[5].u)
              #declare Ondas_Elongacoes[Ondas_Vez][Fila][Col][NoCams-1] =
                Ondas_ElongContorno(
                  Ondas_FMovimento(Fila/DensLinCalc,Col/DensLinCalc,(NoCams-1)/DensLinCalc) ,
                  Tempus
                )
              ;
              #if(Ondas_Contorno[5].v)
                #declare Ondas_Elongacoes[Ondas_Vez][Fila][Col][NoCams-2] =
                  Ondas_Elongacoes[Ondas_Vez][Fila][Col][NoCams-1] -
                  Ondas_PendContorno(
                    Ondas_FMovimento(Fila/DensLinCalc,Col/DensLinCalc,(NoCams-1)/DensLinCalc), Tempus
                  ) / DensLinCalc
                ;
                #ifdef(Ondas_Integral)
                  #declare SomaElongacoes=SomaElongacoes+Ondas_Elongacoes[Ondas_Vez][Fila][Col][NoCams-2];
                #end
              #end
            #else
              #if(Ondas_Contorno[3].v)  
                #declare Ondas_Elongacoes[Ondas_Vez][Fila][Col][NoCams-1] =
                  Ondas_Elongacoes[Ondas_Vez][Fila][Col][NoCams-2] +
                  Ondas_PendContorno(
                    Ondas_FMovimento(Fila/DensLinCalc,Col/DensLinCalc,(NoCams-1)/DensLinCalc), Tempus
                  ) / DensLinCalc
                ;
              #else
                #declare Ondas_Elongacoes[Ondas_Vez][Fila][Col][NoCams-1] =

/************************ ESTOU AQUI *******************************************/

                  #if((Fila>0)&(Fila<NoFilas-1))
                    pow(Rapidez0*DensLinCalc/FreqCalc,2) *
                      ( Ondas_Elongacoes[!Ondas_Vez][Fila-1][NoCams-1] +
                        Ondas_Elongacoes[!Ondas_Vez][Fila+1][NoCams-1] -
                        2*Ondas_Elongacoes[!Ondas_Vez][Fila][NoCams-1]
                      ) +
                  #end
                  Ondas_Perturbaco(<Fila, NoCams-1>/DensLinCalc, Tempus)/pow(FreqCalc,2) +
                  Ondas_Elongacoes[!Ondas_Vez][Fila][NoCams-1]*(1-RigResorte/pow(FreqCalc,2)) +
                  ( Ondas_Elongacoes[!Ondas_Vez][Fila][NoCams-1] -
                    Ondas_Elongacoes[Ondas_Vez][Fila][NoCams-1] ) * (1-Amortecimento/FreqCalc)
                  #ifdef(Ondas_Integral)
                    + Ondas_Integral(Tempus)/(DimsDominio.x*DimsDominio.y)-Ondas_ElongacoMedia
                  #end
                ;
              #end
            #end
            #ifdef(Ondas_Integral)
              #declare SomaElongacoes=SomaElongacoes+Ondas_Elongacoes[Ondas_Vez][Fila][NoCams-1];
            #end
          #end
          #if(Ondas_Contorno[2].u)
            #declare Ondas_Elongacoes[Ondas_Vez][Fila][0] =
              Ondas_ElongContorno(<Fila/DensLinCalc, 0>, Tempus);
            #if(Ondas_Contorno[2].v)  
              #declare Ondas_Elongacoes[Ondas_Vez][Fila][1] =
                Ondas_Elongacoes[Ondas_Vez][Fila][0] +
                Ondas_PendContorno(<Fila/DensLinCalc, 0>, Tempus)/DensLinCalc;
              #ifdef(Ondas_Integral)
                #declare SomaElongacoes=SomaElongacoes+Ondas_Elongacoes[Ondas_Vez][Fila][1];
              #end
            #end
          #else
            #if(Ondas_Contorno[2].v)  
              #declare Ondas_Elongacoes[Ondas_Vez][Fila][0] =
                Ondas_Elongacoes[Ondas_Vez][Fila][1] -
                Ondas_PendContorno(<Fila/DensLinCalc, 0>, Tempus)/DensLinCalc;
            #else
              #declare Ondas_Elongacoes[Ondas_Vez][Fila][0] =
                #if((Fila>0)&(Fila<NoFilas-1))
                  pow(Rapidez0*DensLinCalc/FreqCalc,2) *
                    ( Ondas_Elongacoes[!Ondas_Vez][Fila-1][0] +
                      Ondas_Elongacoes[!Ondas_Vez][Fila+1][0] -
                      2*Ondas_Elongacoes[!Ondas_Vez][Fila][0]
                    ) +
                #end
                Ondas_Perturbaco(<Fila/DensLinCalc, 0>, Tempus)/pow(FreqCalc,2) +
                Ondas_Elongacoes[!Ondas_Vez][Fila][0]*(1-RigResorte/pow(FreqCalc,2)) +
                ( Ondas_Elongacoes[!Ondas_Vez][Fila][0] -
                  Ondas_Elongacoes[Ondas_Vez][Fila][0] ) * (1-Amortecimento/FreqCalc)
                #ifdef(Ondas_Integral)
                  + Ondas_Integral(Tempus)/(DimsDominio.x*DimsDominio.y)-Ondas_ElongacoMedia
                #end
              ;
            #end
          #end
          #ifdef(Ondas_Integral)
            #declare SomaElongacoes=SomaElongacoes+Ondas_Elongacoes[Ondas_Vez][Fila][0];
          #end
          #declare Col=Col+1;
        #end
        #declare Fila=Fila+1;
      #end
      #local Fila=FilaInf;
      #while(Fila<FilaSup)
        #if(NoCols>1)
          #if(Ondas_Contorno[3].u)
            #declare Ondas_Elongacoes[Ondas_Vez][Fila][NoCols-1] =
              Ondas_ElongContorno(<Fila, NoCols-1>/DensLinCalc, Tempus);
            #if(Ondas_Contorno[3].v)  
              #declare Ondas_Elongacoes[Ondas_Vez][Fila][NoCols-2] =
                Ondas_Elongacoes[Ondas_Vez][Fila][NoCols-1] -
                Ondas_PendContorno(<Fila, NoCols-1>/DensLinCalc, Tempus)/DensLinCalc;
              #ifdef(Ondas_Integral)
                #declare SomaElongacoes=SomaElongacoes+Ondas_Elongacoes[Ondas_Vez][Fila][NoCols-2];
              #end
            #end
          #else
            #if(Ondas_Contorno[3].v)  
              #declare Ondas_Elongacoes[Ondas_Vez][Fila][NoCols-1] =
                Ondas_Elongacoes[Ondas_Vez][Fila][NoCols-2] +
                Ondas_PendContorno(<Fila, NoCols-1>/DensLinCalc, Tempus)/DensLinCalc;
            #else
              #declare Ondas_Elongacoes[Ondas_Vez][Fila][NoCols-1] =
                #if((Fila>0)&(Fila<NoFilas-1))
                  pow(Rapidez0*DensLinCalc/FreqCalc,2) *
                    ( Ondas_Elongacoes[!Ondas_Vez][Fila-1][NoCols-1] +
                      Ondas_Elongacoes[!Ondas_Vez][Fila+1][NoCols-1] -
                      2*Ondas_Elongacoes[!Ondas_Vez][Fila][NoCols-1]
                    ) +
                #end
                Ondas_Perturbaco(<Fila, NoCols-1>/DensLinCalc, Tempus)/pow(FreqCalc,2) +
                Ondas_Elongacoes[!Ondas_Vez][Fila][NoCols-1]*(1-RigResorte/pow(FreqCalc,2)) +
                ( Ondas_Elongacoes[!Ondas_Vez][Fila][NoCols-1] -
                  Ondas_Elongacoes[Ondas_Vez][Fila][NoCols-1] ) * (1-Amortecimento/FreqCalc)
                #ifdef(Ondas_Integral)
                  + Ondas_Integral(Tempus)/(DimsDominio.x*DimsDominio.y)-Ondas_ElongacoMedia
                #end
              ;
            #end
          #end
          #ifdef(Ondas_Integral)
            #declare SomaElongacoes=SomaElongacoes+Ondas_Elongacoes[Ondas_Vez][Fila][NoCols-1];
          #end
        #end
        #if(Ondas_Contorno[2].u)
          #declare Ondas_Elongacoes[Ondas_Vez][Fila][0] =
            Ondas_ElongContorno(<Fila/DensLinCalc, 0>, Tempus);
          #if(Ondas_Contorno[2].v)  
            #declare Ondas_Elongacoes[Ondas_Vez][Fila][1] =
              Ondas_Elongacoes[Ondas_Vez][Fila][0] +
              Ondas_PendContorno(<Fila/DensLinCalc, 0>, Tempus)/DensLinCalc;
            #ifdef(Ondas_Integral)
              #declare SomaElongacoes=SomaElongacoes+Ondas_Elongacoes[Ondas_Vez][Fila][1];
            #end
          #end
        #else
          #if(Ondas_Contorno[2].v)  
            #declare Ondas_Elongacoes[Ondas_Vez][Fila][0] =
              Ondas_Elongacoes[Ondas_Vez][Fila][1] -
              Ondas_PendContorno(<Fila/DensLinCalc, 0>, Tempus)/DensLinCalc;
          #else
            #declare Ondas_Elongacoes[Ondas_Vez][Fila][0] =
              #if((Fila>0)&(Fila<NoFilas-1))
                pow(Rapidez0*DensLinCalc/FreqCalc,2) *
                  ( Ondas_Elongacoes[!Ondas_Vez][Fila-1][0] +
                    Ondas_Elongacoes[!Ondas_Vez][Fila+1][0] -
                    2*Ondas_Elongacoes[!Ondas_Vez][Fila][0]
                  ) +
              #end
              Ondas_Perturbaco(<Fila/DensLinCalc, 0>, Tempus)/pow(FreqCalc,2) +
              Ondas_Elongacoes[!Ondas_Vez][Fila][0]*(1-RigResorte/pow(FreqCalc,2)) +
              ( Ondas_Elongacoes[!Ondas_Vez][Fila][0] -
                Ondas_Elongacoes[Ondas_Vez][Fila][0] ) * (1-Amortecimento/FreqCalc)
              #ifdef(Ondas_Integral)
                + Ondas_Integral(Tempus)/(DimsDominio.x*DimsDominio.y)-Ondas_ElongacoMedia
              #end
            ;
          #end
        #end
        #ifdef(Ondas_Integral)
          #declare SomaElongacoes=SomaElongacoes+Ondas_Elongacoes[Ondas_Vez][Fila][0];
        #end
        #declare Fila=Fila+1;
      #end
      #local Col=0;
      #while(Col<NoCols)
        #if(NoFilas>1)
          #if(Ondas_Contorno[1].u)
            #declare Ondas_Elongacoes[Ondas_Vez][NoFilas-1][Col] =
              Ondas_ElongContorno(<NoFilas-1, Col>/DensLinCalc, Tempus);
            #ifdef(Ondas_Integral)
              #declare SomaElongacoes=SomaElongacoes+Ondas_Elongacoes[Ondas_Vez][NoFilas-1][Col];
            #end
            #if(Ondas_Contorno[1].v)  
              #declare Ondas_Elongacoes[Ondas_Vez][NoFilas-2][Col] =
                Ondas_Elongacoes[Ondas_Vez][NoFilas-1][Col] -
                Ondas_PendContorno(<NoFilas-1, Col>/DensLinCalc, Tempus)/DensLinCalc;
              #ifdef(Ondas_Integral)
                #declare SomaElongacoes=SomaElongacoes+Ondas_Elongacoes[Ondas_Vez][NoFilas-2][Col];
              #end
            #end
          #else
            #if(Ondas_Contorno[1].v)  
              #declare Ondas_Elongacoes[Ondas_Vez][NoFilas-1][Col] =
                Ondas_Elongacoes[Ondas_Vez][NoFilas-2][Col] +
                Ondas_PendContorno(<NoFilas-1, Col>/DensLinCalc, Tempus)/DensLinCalc;
              #ifdef(Ondas_Integral)
                #declare SomaElongacoes=SomaElongacoes+Ondas_Elongacoes[Ondas_Vez][NoFilas-1][Col];
              #end
            #else
              #if((Col>max(0,ColInf-1))&(Col<min(NoCols-1,ColSup)))
                #declare Ondas_Elongacoes[Ondas_Vez][NoFilas-1][Col] =
                  pow(Rapidez0*DensLinCalc/FreqCalc,2) *
                    ( Ondas_Elongacoes[!Ondas_Vez][NoFilas-1][Col+1] +
                      Ondas_Elongacoes[!Ondas_Vez][NoFilas-1][Col-1] -
                      2*Ondas_Elongacoes[!Ondas_Vez][NoFilas-1][Col]
                    ) +
                  Ondas_Perturbaco(<NoFilas-1, Col>/DensLinCalc, Tempus)/pow(FreqCalc,2) +
                  Ondas_Elongacoes[!Ondas_Vez][NoFilas-1][Col]*(1-RigResorte/pow(FreqCalc,2)) +
                  ( Ondas_Elongacoes[!Ondas_Vez][NoFilas-1][Col] -
                    Ondas_Elongacoes[Ondas_Vez][NoFilas-1][Col] ) * (1-Amortecimento/FreqCalc)
                  #ifdef(Ondas_Integral)
                    + Ondas_Integral(Tempus)/(DimsDominio.x*DimsDominio.y)-Ondas_ElongacoMedia;
                    #declare SomaElongacoes=SomaElongacoes+Ondas_Elongacoes[Ondas_Vez][NoFilas-1][Col];
                  #else
                    ;
                  #end
              #end
            #end
          #end
        #end
        #if(Ondas_Contorno[0].u)
          #declare Ondas_Elongacoes[Ondas_Vez][0][Col] =
            Ondas_ElongContorno(<0, Col/DensLinCalc>, Tempus);
          #ifdef(Ondas_Integral)
            #declare SomaElongacoes=SomaElongacoes+Ondas_Elongacoes[Ondas_Vez][0][Col];
          #end
          #if(Ondas_Contorno[0].v)  
            #declare Ondas_Elongacoes[Ondas_Vez][1][Col] =
              Ondas_Elongacoes[Ondas_Vez][0][Col] +
              Ondas_PendContorno(<0, Col/DensLinCalc>, Tempus)/DensLinCalc;
            #ifdef(Ondas_Integral)
              #declare SomaElongacoes=SomaElongacoes+Ondas_Elongacoes[Ondas_Vez][1][Col];
            #end
          #end
        #else
          #if(Ondas_Contorno[0].v)  
            #declare Ondas_Elongacoes[Ondas_Vez][0][Col] =
              Ondas_Elongacoes[Ondas_Vez][1][Col] -
              Ondas_PendContorno(<0, Col/DensLinCalc>, Tempus)/DensLinCalc;
            #ifdef(Ondas_Integral)
              #declare SomaElongacoes=SomaElongacoes+Ondas_Elongacoes[Ondas_Vez][0][Col];
            #end
          #else
            #if((Col>max(0,ColInf-1))&(Col<min(NoCols-1,ColSup)))
              #declare Ondas_Elongacoes[Ondas_Vez][0][Col] =
                pow(Rapidez0*DensLinCalc/FreqCalc,2) *
                  ( Ondas_Elongacoes[!Ondas_Vez][0][Col+1] +
                    Ondas_Elongacoes[!Ondas_Vez][0][Col-1] -
                    2*Ondas_Elongacoes[!Ondas_Vez][0][Col]
                  ) +
                Ondas_Perturbaco(<0, Col/DensLinCalc>, Tempus)/pow(FreqCalc,2) +
                Ondas_Elongacoes[!Ondas_Vez][0][Col]*(1-RigResorte/pow(FreqCalc,2)) +
                ( Ondas_Elongacoes[!Ondas_Vez][0][Col] -
                  Ondas_Elongacoes[Ondas_Vez][0][Col] ) * (1-Amortecimento/FreqCalc)
                #ifdef(Ondas_Integral)
                  + Ondas_Integral(Tempus)/(DimsDominio.x*DimsDominio.y)-Ondas_ElongacoMedia;
                  #declare SomaElongacoes=SomaElongacoes+Ondas_Elongacoes[Ondas_Vez][0][Col];
                #else
                  ;
                #end
            #end
          #end
        #end
        #declare Col=Col+1;
      #end

      #ifdef(Ondas_Integral)
        #declare Ondas_ElongacoMedia=SomaElongacoes/(NoFilas*NoCols*NoCams);
      #end
      #declare Ondas_Vez=!Ondas_Vez;
      #declare Tempus=Tempus+1/FreqCalc;
    #end
    #local Tempo=Tempus;

  #end

  #if(Xose_Mensagens)
    #debug concat("    Estam-se a guardar as condicoes actuais do sistema \"", Nome, "\".\n")
  #end
  #fopen Filego NomeDoFilego write
  #write( Filego,
    "// Arquivo de dados temporais para o \"Ondas.inc\" de Xose\n\n#declare Ondas_Elongacoes=array[2][",
    str(NoFilas,0,0),"][",str(NoCols,0,0),"][",str(NoCams,0,0),"]{\n"
  )
  #local Toque=0;
  #while(Toque<2)
    #write(Filego,"  {\n")
    #local Fila=0;
    #while(Fila<NoFilas)
      #write(Filego,"    {\n")
      #local Col=0;
      #while(Col<NoCols)
        #write(Filego,"      {")
        #local Cam=0;
        #while(Cam<NoCams)
          #write(Filego,
            #if(QuantosGrausDeLiberdade=1)
              NotacoCientifica(Ondas_Elongacoes[Toque][Fila][Col][Cam],17)
            #else
              NotacoCientificaVector(QuantosGrausDeLiberdade,Ondas_Elongacoes[Toque][Fila][Col][Cam],17)
            #end
            ,",")
          #declare Cam=Cam+1;
        #end
        #write(Filego,"}\n")
        #declare Col=Col+1;
      #end
      #write(Filego,"    }\n")
      #declare Fila=Fila+1;
    #end
    #write(Filego,"  }\n")
    #declare Toque=Toque+1;
  #end
  #write(Filego,"}\n#declare Ondas_Vez=",str(Ondas_Vez,0,0),";\n#declare Ondas_Tempo=",
    NotacoCientifica(Tempo,17),";")
  #ifdef(Ondas_Integral)
    #write(Filego,"\n#declare Ondas_ElongacoMedia=",
      #if(QuantosGrausDeLiberdade=1)
        NotacoCientifica(Ondas_ElongacoMedia,17)
      #else
        NotacoCientificaVector(QuantosGrausDeLiberdade,Ondas_ElongacoMedia,17)
      #end
      ,";");
  #end
  #fclose Filego

  #if(Xose_Mensagens)
    #debug "    Estam-se a definir Ondas_Pigmento e Ondas_Funco.\n"
  #end

  #ifndef(Ondas_MaxElong)
    #declare Ondas_MaxElong=1;
  #end

  #ifndef(Ondas_MinElong)
    #declare Ondas_MinElong=-1;
  #end

/*  // Instruces para superar o limite de 256  256 pontos na grelha, ainda n prontas.
  #local NivelMaxX=floor(log(NoFilas)/log(256))+1;
  #local NivelMaxZ=floor(log(NoCols)/log(256))+1;
  #local NivelX=NivelMaxX;
  #local NivelZ=NivelMaxZ;
  #declare Ondas_Pigmento=pigment{
    #local Fila=0;
    #while(Fila<NoFilas)
      #if(mod(Fila,pow(256,NivelX)))
        gradient x pigment_map{
      #end
      #declare Fila=Fila+1;
    #end
  }
*/

  #declare Ondas_Pigmento=pigment{
    gradient x pigment_map{
      #local Fila=0;
      #while(Fila<NoFilas)
        [Fila/(NoFilas-1)
          gradient y pigment_map{
            #local Col=0;
            #while(Col<NoCols)
              [Col/(NoCols-1)
                gradient z colour_map{
                  #local Cam=0;
                  #while(Cam<NoCams)
                    [Cam/(NoCams-1)
                      rgb min( max( (Ondas_Elongacoes[!Ondas_Vez][Fila][Col]-Ondas_MinElong) /
                        (Ondas_MaxElong-Ondas_MinElong), 0), 1)
                    ]
                    #declare Cam=Cam+1;
                  #end
                }
              ]
              #declare Col=Col+1;
            #end
          }
        ]
        #declare Fila=Fila+1;
      #end
    }
    scale DimsDominio transform Ondas_Movimento(Tempo)
  }

  #local UnaCabura=function{pattern{pigment_pattern{Ondas_Pigmento}}}
  #declare Ondas_Funco =
    function{UnaCabura(x,y,z)*(Ondas_MaxElong-Ondas_MinElong)+Ondas_MinElong}
  //#undef UnaCabura

#end


#version versionLamElast;