// Arquivo principal de cena para o traador de raios de persistence of vision
// Arquivo: Ondinhas.pov
// Vers: 3.5
// Descri: Prova do arquivo Lamina elastica.inc
// Data: 24-II-2003
// Autor: Xos Benito Carballeira Rabual

// Linha de ordes recomendada:
//+w320 +h240 +a0.3 -j +kf60 +kff1200 +fs

// Deixei-o para prosseguir com:
//+w320 +h240 +a0.3 -j +kf60 +kff1200 +sf874 +fs

#version 3.5;

#include "metals.inc"
#include "Matematica.inc"
#include "Lamina elastica.inc"
#include "particle.inc"

global_settings{
  assumed_gamma 1
  max_trace_level 20
}

camera{
  location<1.5,8,-5>
  right 4/3*x
  up y
  direction 3*z
  look_at<2.1,0,1.5>
  //look_at 1e7*<0,60,4>  //Nuvens
}

sky_sphere{
  pigment{bozo colour_map{
      [0 rgb<.05,.5,.7>]
      [.5 rgb<.05,.5,.7>]
      [.6 rgb<.6,.7,.85>]
      [.8 rgb<.8,.85,.92>]
      [1 rgb<.95,.97,.96>]
    }
    quick_colour rgb<0,1,1>
    warp{turbulence .2 octaves 3 lambda 3 omega .8}
    translate .04*clock  //As nuvens movem-se e transformam-se.
    scale .15
  }
}

light_source{1e7*y rgb 1 rotate<-30,10,0>}

//Bilha
union{
  cylinder{<4.2,0,2>, <4.2,.50001,2>, .1}
  difference{
    torus{.4, .1 rotate 90*x}
    plane{y,0}
    translate<3.8,.5,2>
  }
  texture{T_Brass_3C pigment{quick_colour rgb<1,1,0>}}
}

#macro LamElast_Perturbaco(Onde, Quando)
    ( (Quando<.5899)? 0 :
      100*exp(-pow(10*vlength(Onde-<3.4,2>),4))*(exp(-7*(Quando-.5899))+cos(20*(Quando-.5899))) )
#end
#macro LamElast_MacroElongaco0(Fafon) (-1.5) #end
#declare LamElast_MaxElong=.201;
#declare LamElast_MinElong=-1.5001;

LaminaElastica("Chuingou", <4,3>, 52, 40, .5, 0, clock, 0)

#declare Solo=difference{
  plane{y,.2}
  box{<.0001,-1.4999,.0001>,<3.9999,.201,2.9999>}  //Estanque
}
#declare AMinhaAuga=isosurface{
  function{y-LamElast_Funcion(x,0,z)}
  contained_by{box{-1.5*y,<4,.45,3>}}
  //threshold 0.0
  accuracy 0.001
  //max_gradient 2
  evaluate 2, 2, 0.8
  //max_trace 1
  all_intersections
  //open
}
#declare Mat_Auga=material{
  texture{
    pigment{rgbf<.9,.98,.995,1> quick_colour rgb<0,0,1>}
    finish{ambient 0 diffuse 0 specular .8 roughness .0001 conserve_energy reflection{.05, .8 falloff 1.5}}
  }
  interior{ior 1.33 dispersion 1.007 caustics .7}
}

//Auga
#declare particle_start=0;
#declare particle_end=1e7;
#declare particle_steps=40;
#declare particle_frequency=80;
#declare particle_life=1;
#declare particle_seed=162;
#declare particle_maxnumber=50;
#macro particle_gravity(Quando, Onde) -9.8*y #end
#macro particle_emitter(Quando) <3.4,.5,2> #end
#macro particle_emitvect(Quando) -.5*y #end
#macro particle_emitturb(Quando) .1 #end
#macro particle_killobj(Quando) Solo #end
//#macro particle_element() sphere{p_location, .1, .8+.4*p_random} #end  //Para blob
#macro particle_element() sphere{p_location, .07+.02*p_random} #end  //Esferas independentes
#if(clock>0)
  merge{
    /*blob{  //Chorro que cai da bilha
      threshold .5*/
      particle_system("Ducha pachouf")
    //}
    object{AMinhaAuga}  //Auga acumulada no estanque
    material{Mat_Auga}
  }
#else
  particle_system("Ducha pachouf")
  object{AMinhaAuga material{Mat_Auga}}
#end

object{
  Solo
  texture{
    pigment{
      crackle colour_map{
        [0 rgb<.25,.3,.2>]
        [.3 rgb<.4,.2,.1>]
        [.6 rgb<.2,.4,.5>]
        [.8 rgb<.1,.4,.3>]
        [1 rgb<.2,.3,.18>]
      }
      quick_colour rgb<0,.5,0>
    }
    normal{
      average normal_map{
        [1 crackle 2]
        [1 bumps]
      }
    }
    finish{diffuse .6 specular .5 roughness .02}
  }
}