// Arquivo principal de cena para o traador de raios de persistence of vision
// Arquivo: Proba dos fractais julianos.pov
// Vers: 3.5
// Descri: Prova dos fractais de Julia
// Data: Comeos do 2002
// Autor: Xos B. Carballeira Rabual

// Opes testadas e recomendadas no 1-VIII-2005
// +kff150 +a0.2 +am2 +r2

#version 3.5;

camera{
  location<0.3,.4,-2.5>
  right<4/3,0,0>
  up<0,1,0>
  direction<0,0,1>
  rotate<0,-360*clock,0>
  look_at <0,0,0>
}

global_settings{
  //assumed_gamma 1
  assumed_gamma 2.2
  ambient_light 3
  max_trace_level 100
}

#declare GrandorDoSol=1;  //Dimetro aparente do Sol, em graus sexagessimais.
#declare RotSol=<40,-20,0>;       //Rota para pr o Sol no seu lugar, partindo do zenit.
#declare CorDoSol=color rgb<1,.9,.7>;

sky_sphere{
  pigment{
    gradient y colour_map{
      [0 rgb<.3,.5,1>]
      [.5 rgb<.4,.7,1>]
      [.5+cos(radians(GrandorDoSol)/2)/2 rgb<.6,.8,1>]
      [.5+cos(radians(GrandorDoSol)/2)/2 CorDoSol]
    }
    scale 2 translate<0,-1,0> rotate RotSol
  }
}
plane{-y, -10000
  pigment{
    bozo colour_map{
      [0 rgbf 1]
      [0.5 rgbf 1]
      [0.6 rgbf<1,1,1,.2>]
      [1 rgbf<1,1,1,0>]
    }
    scale 10000
    translate<1,.3,.2>*20000*clock
    turbulence .6
  }
  finish{ambient .3}
}

light_source{1e7*y CorDoSol rotate RotSol media_attenuation on}

plane{y, -6.3
  pigment{agate colour_map{
      [0 rgb<.65,.5,.2>]
      [.4 rgb<.5,.2,.1>]
      [.9 rgb<.7,.3,.13>]
      [1 rgb<.65,.35,.1>]
    }
    scale 10
  }
  normal{bumps scale 4}
}

plane{y,-6
  pigment{
    rgbf <.8,.9,1,1>
  }
  normal{
    ripples
    frequency 1e5
    phase 2*clock
    turbulence .3
    scale 1e5
  }
  finish{
    specular .9
    roughness .0001
    reflection .5
    reflection_exponent .5
  }
  interior{
    ior 1.33
    fade_distance 1
    fade_power 2
  }
}

#declare HiperAltura=-1+2*clock;
julia_fractal {
  <-0.083,0.0,-0.83,-0.025>
  quaternion
  sqr
  max_iteration 8
  precision 15
  slice <0,0,0,1>,HiperAltura

  pigment{
    rgbf<.4,1,.5,1>
  }
  finish{
    ambient .05
    diffuse .4
    brilliance 6
    specular .7
    roughness .0001
    reflection .1
  }
  interior{ior 1.5}
}