// Arquivo principal de cena para o traador de raios de Persistence of Vision
// Arquivo: Torta de aniversario.pov
// Vers: 3.6
// Descri: A torta com que felicitei Andrs polos seus 21 anos; onde vai! :)
// Data de cria: 21-XI-2005
// Data da ltima actualiza: _
// Autor: Xos Benito Carballeira Rabual


#version 3.6;

//-------------------------------------------- OPES -------------------------------------------------------

//#declare VelasSimplificadas=false;
//#declare VelasSimplificadas=true;

//#declare VelasApagadas=false;
#declare VelasApagadas=true;

//-------------------------------------------- CENA -------------------------------------------------------

#include "Materiais varios.inc"
#include "Corpos lineares.inc"
#include "particle.inc"  //O sistema de partculas de Rune S. Johansen
#include "Objectos luminosos.inc"

#ifndef(VelasSimplificadas)
  #declare VelasSimplificadas=false;
#end

#ifndef(VelasApagadas)
  #declare VelasApagadas=false;
#end

global_settings{
  assumed_gamma 1
}

camera{
  location<-.3,.7,-1.25>
  //location<.2,.05,-.4>
  right 4/3*x
  up y
  direction 3*z
  //look_at 0
  look_at .013*y
}

light_source{1e7*y #if(VelasApagadas) rgb 1 #else rgb .7 #end rotate<-40,-30,0>}

// Torta de dias
union{
  //O corpo de tona e morodo, com feitura de posia...
  cylinder{0, .1*y, .25
    texture{
      pigment{
        granite colour_map{
          [0 rgb 1]
          [.4 rgb<1,.1,.3>]
          [1 rgb<1,.01,.03>]
        }
      }
      normal{granite}
      finish{diffuse .7 brilliance 1 specular .8 roughness .05}
      scale .07
    }
  }
  //... um verniz de mel e fruita confeitada...
  cylinder{.095*y, .105*y, .22
    texture{
      pigment{
        crackle colour_map{
          [.3 rgbf<1,.8,.6,.95>]
          [.5 rgbf<.6,1,.8,.7>]
          [1 rgbf<.4,.9,.6,.6>]
        }
        warp{turbulence .1}
      }
      normal{
        average normal_map{
          [0 crackle .2 warp{turbulence .1}]
          [1 bumps 1 scale .1]
        }
      }
      finish{diffuse .6 brilliance 2 specular .8 roughness .1}
      scale .07
    }
  }
  //... duas capas de biscoito...
  #declare Biscoito=cylinder{.01*y, .03*y, .251
    texture{
      pigment{rgb<.38,.12,.05>}
      normal{
        crackle 5 form<1,0,0> scale .005
      }
      finish{ambient .05 diffuse .6 brilliance 1.5 specular .1 roughness .1}
    }
  }
  object{Biscoito} object{Biscoito pigment{rgb<.5,.4,.1>} translate .04*y}
  //... uns chorrinhos de merengue...
  #declare ContaChorrinhos=0;
  #declare EstouQueLamboOsDedos=seed(234);
  union{
    #while(ContaChorrinhos<24)
      julia_fractal{
        < -.2+.05*(2*rand(EstouQueLamboOsDedos)-1),-.38+.05*(2*rand(EstouQueLamboOsDedos)-1),
          .39+.05*(2*rand(EstouQueLamboOsDedos)-1),-.35+.05*(2*rand(EstouQueLamboOsDedos)-1) >
        quaternion
        sqr
        max_iteration 20
        precision 40
        slice <0,0,0,1>, .2
        scale .06 rotate<30*rand(EstouQueLamboOsDedos)-15,30,0> translate<.22,.08,0>
        rotate 360*ContaChorrinhos/24*y
      }
      #declare ContaChorrinhos=ContaChorrinhos+1;
    #end
    texture{
      pigment{
        bozo colour_map{
          [0 rgb 1]
          [.5 rgb<.95,.9,.8>]
          [1 rgb<.85,.7,.4>]
        }
        scale .03
      }
      finish{specular .2 roughness .1}
    }
  }
  //... a bandeja, concerteza...
  difference{
    cone{-.0001*y, .265, .02*y, .305}
    cone{.005*y, .255, .02001*y, .295}
    texture{
      pigment{rgb<.3,.2,1>} finish{specular .3 roughness .003}
      normal{
        wood slope_map{
          [.59 <0,0>]
          [.6 <.001,5>]
          [.61 <.999,5>]
          [.62 <1,0>]
          [.69 <1,0>]
          [.7 <.999,-5>]
          [.71 <.001,-5>]
          [.72 <0,0>]
        }
        rotate 90*x
      }
      scale .03
    }
  }
  //... e mais as 21 velinhas: fuuuu, e que cumpras muitos mais!
  #declare PlasticoParaOsSuportes=texture{pigment{rgb<.1,.2,1>} finish{specular .3 roughness .003}}
  #if(VelasSimplificadas)
    #if(VelasApagadas)
      #macro ChantaUmaVelinha(Aqui,EsteLume,EsteFume)
        //O suporte...
        cone{Aqui+.0999*y, .0041, Aqui+.106*y, .006 texture{PlasticoParaOsSuportes}}
        //... e mais a velinha
        cylinder{0, .08*y, .004 translate Aqui+.1*y texture{Cera}}
      #end
    #else
      #macro ChantaUmaVelinha(Aqui,EsteLume,EsteFume)
        //O suporte...
        cone{Aqui+.0999*y, .0041, Aqui+.106*y, .006 texture{PlasticoParaOsSuportes}}
        //... e mais a velinha
        union{
          cylinder{0, .08*y, .004 texture{Cera}}
          light_source{.085 rgb .96/21}
          translate Aqui+.1*y
        }
      #end
    #end
  #else
    #declare Vela_Ciclo=true;
    #declare Teito=plane{-y, -.6}
    #macro Vela_ObjExtint(Tempo) Teito #end
    #declare Vela_FactorLuz=.8/21;
    #declare Vela_DensFume=40;
    #macro Vela_Vento(Tempo,Posico) .05*z+.1*vturbulence(2,.5,6,Posico) #end
    #macro Vela_Transformaco(Tempo) translate Aqui+.1*y #end
    #if(VelasApagadas)
      #macro ChantaUmaVelinha(Aqui,EsteLume,EsteFume)
        //O suporte...
        cone{Aqui+.0999*y, .0041, Aqui+.106*y, .006 texture{PlasticoParaOsSuportes}}
        //... e mais a velinha
        Vela(.08,.008,9,1+3*rand(EstouQueLamboOsDedos),.05,.0015,.01,.001,0,EsteLume,EsteFume,
          rand(EstouQueLamboOsDedos)*100)
      #end
    #else
      #macro ChantaUmaVelinha(Aqui,EsteLume,EsteFume)
        //O suporte...
        cone{Aqui+.0999*y, .0041, Aqui+.106*y, .006 texture{PlasticoParaOsSuportes}}
        //... e mais a velinha
        Vela(.08,.008,9,1+3*rand(EstouQueLamboOsDedos),.05,.0015,.01,.001,1.2,EsteLume,EsteFume,
          rand(EstouQueLamboOsDedos)*100)
      #end
    #end
  #end
  #declare ContaVelinhas=0;
  #while(ContaVelinhas<5)
    ChantaUmaVelinha( .05*<cos(2*pi*ContaVelinhas/5),0,sin(2*pi*ContaVelinhas/5)>,
      concat("Lume",str(ContaVelinhas+1,-2,0)), concat("Fume",str(ContaVelinhas+1,-2,0)) )
    #declare ContaVelinhas=ContaVelinhas+1;
  #end
  #declare ContaVelinhas=0;
  #while(ContaVelinhas<7)
    ChantaUmaVelinha( .1*<cos(2*pi*ContaVelinhas/7),0,sin(2*pi*ContaVelinhas/7)>,
      concat("Lume",str(ContaVelinhas+6,-2,0)), concat("Fume",str(ContaVelinhas+6,-2,0)) )
    #declare ContaVelinhas=ContaVelinhas+1;
  #end
  #declare ContaVelinhas=0;
  #while(ContaVelinhas<9)
    ChantaUmaVelinha( .15*<cos(2*pi*ContaVelinhas/9),0,sin(2*pi*ContaVelinhas/9)>,
      concat("Lume",str(ContaVelinhas+13,-2,0)), concat("Fume",str(ContaVelinhas+13,-2,0)) )
    #declare ContaVelinhas=ContaVelinhas+1;
  #end
}
