// Arquivo auxiliar para o traador de raios de Persistence of Vision
// Arquivo: Urbanismo.inc
// Vers: 3.5
// Descri: Objectos e texturas urbanos ou de engenharia civil
// Data de cria: 4-VI-2003; procede dos velhos Os nosos materiais.inc e Obxectos de Xose.inc, cria-
                    //dos o 28-IV-2002 e o 17-IV-2002, respectivamente.
// Data da ltima actualiza: _
// Autor: Xos B. Carballeira Rabual


#local versionUrb=version;
#version 3.5;

#ifndef(Xose_Mensagens)
  #declare Xose_Mensagens=false;
#end

#if(Xose_Mensagens)
  #debug "    Esta-se a executar o arquivo \"Urbanismo.inc\", que define elementos urbanos ou de engenharia civil.\n"
#end

//Requer:
//#include "Matematica.inc"  //S quando se empregar a macro TramoCircularDeEstrada

#ifndef(Xose_SemTexturas)
  #declare Xose_SemTexturas=false;
#end


//============================================ TEXTURAS =============================================================

#declare Formigo=texture{
  pigment{rgb<.6,.6,.65>
    quick_color rgb<1,1,0>
  }
  normal{
    granite 6
    scale .05
  }
  finish{
    specular .6 roughness .1
  }
}

#declare FormigoClaro=texture{
  pigment{rgb <.72,.7,.75>}
  normal{wrinkles 5/3 scale .3}
  finish{
    diffuse .7 brilliance 3
    specular .6 roughness .01
  }
}


#local RectaDescontinua=pigment{
  gradient z colour_map{
    [2/3 rgb<.14,.17,.2>]
    [2/3 rgb<.92,.97,.93>]
  }
  phase 1/3
  scale 4
}

#macro CircunferenciaDescontinua(Raio)
  pigment{
    radial colour_map{
      [2/3 rgb<.14,.17,.2>]
      [2/3 rgb<.92,.97,.93>]
    }
    frequency .5*pi*Raio phase 1/3
    scale<1/12,1,1/12>   //Isto economizar transformaes
    rotate<90,0,0>     //no AsfaltoCircular().
  }
#end

#declare Asfalto=texture{
  pigment{rgb<.14,.17,.2>}
  normal{granite 12.5 scale .04}
  finish{
    diffuse .7
  }
}

#declare AsfaltoRecto=texture{ //Asfalto com trs rectas descontnuas brancas que separam os carris, e duas cont-
  pigment{                       //nuas para delimitar as beiravias.
    function{abs(x)} pigment_map{
      [.075/6 RectaDescontinua]
      [.075/6 rgb<.14,.17,.2>]
      [2.425/6 rgb<.14,.17,.2>]
      [2.425/6 RectaDescontinua]
      [2.575/6 RectaDescontinua]
      [2.575/6 rgb<.14,.17,.2>]
      [4.925/6 rgb<.14,.17,.2>]
      [4.925/6 rgb<.92,.97,.93>]
      [5.075/6 rgb<.92,.97,.93>]
      [5.075/6 rgb<.14,.17,.2>]
    }
    scale<6,1,1>
  }
  normal{granite 12.5 scale .04}
  finish{
    diffuse .7
  }
}

#macro AsfaltoCircular(Raio) //Asfalto com os mesmos carris e beiravias que o anterior, no que o eixo da estrada  umha
  texture{                      //circunferncia do raio indicado.
    pigment{
      wood pigment_map{
        [.5-5.075/12 rgb<.14,.17,.2>]
        [.5-5.075/12 rgb<.92,.97,.93>]
        [.5-4.925/12 rgb<.92,.97,.93>]
        [.5-4.925/12 rgb<.14,.17,.2>]
        [.5-2.575/12 rgb<.14,.17,.2>]
        #local Pintada=CircunferenciaDescontinua(Raio-2.5)
        [.5-2.575/12 Pintada]
        [.5-2.425/12 Pintada]
        [.5-2.425/12 rgb<.14,.17,.2>]
        [.5-.075/12 rgb<.14,.17,.2>]
        #declare Pintada=CircunferenciaDescontinua(Raio)
        [.5-.075/12 Pintada]
        [.5+.075/12 Pintada]
        [.5+.075/12 rgb<.14,.17,.2>]
        [.5+2.425/12 rgb<.14,.17,.2>]
        #declare Pintada=CircunferenciaDescontinua(Raio+2.5)
        [.5+2.425/12 Pintada]
        [.5+2.575/12 Pintada]
        [.5+2.575/12 rgb<.14,.17,.2>]
        [.5+4.925/12 rgb<.14,.17,.2>]
        [.5+4.925/12 rgb<.92,.97,.93>]
        [.5+5.075/12 rgb<.92,.97,.93>]
        [.5+5.075/12 rgb<.14,.17,.2>]
      }
      ramp_wave
      phase (6-Raio)/12
      rotate<-90,0,0>
      scale<12,1,12>
    }
    normal{granite 12.5 scale .04}
    finish{
      diffuse .7
    }
  }
#end


//============================================ OBJECTOS =============================================================

#declare Piar=union{ 
  superellipsoid{<.1,.01>
    scale<2,17,1.75>
    rotate<90,0,0>
    translate<0,-1.75,0> 
  }
  superellipsoid{<.1,.8>
    scale<2,50,2>
    rotate<30,0,0>
    translate<0,-.5,14.5> 
    clipped_by{plane{-y,-.5}}
  } 
  superellipsoid{<.1,.8>
    scale<2,50,2>
    translate<0,-.5,0> 
    clipped_by{plane{-y,-.5}}
  }    
  superellipsoid{<.1,.8>
    scale<2,50,2>
    rotate<-30,0,0>
    translate<0,-.5,-14.5> 
    clipped_by{plane{-y,-.5}}
  }  
  cylinder{<0,-.5,-14.5>
    <0,-350,-14.5>, 2
  } 
  cylinder{<0,-.5,14.5>
    <0,-350,14.5>, 2
  } 
  #if(!Xose_SemTexturas) texture{Formigo} #end
}

#macro TramoRectoDeEstrada (Origem, Extremo, Largura, Espesor)
  box{<-Largura/2,0,0>,<Largura/2,-Espesor,vlength(Extremo-Origem)>
    #if(!Xose_SemTexturas) texture{AsfaltoRecto} #end
    rotate<degrees(asin((Extremo-Origem).y/vlength(Extremo-Origem))),
      degrees(atan2((Extremo-Origem).x,(Extremo-Origem).z)),0>
    translate Origem
  }
#end

#macro TramoCircularDeEstrada(Centro,Raio,VectAng,Origem,Largura,Espesor)
  #local Cerebrovacioca=cylinder{<0,0,0>,<0,-Espesor,0>,Raio+Largura/2
    #if(!Xose_SemTexturas) AsfaltoCircular(Raio) #end
  }
  #if(Raio>Largura/2)
    #declare Cerebrovacioca=difference{
      object{Cerebrovacioca}
      cylinder{<0,.001*Espesor,0>,<0,-1.001*Espesor,0>,Raio-Largura/2}
    }
  #end
  #local Sarposo=vlength(VectAng);
  intersection{
    object{Cerebrovacioca}
    #if(sin(Sarposo)>=0)
      plane{z,0}
      plane{-z,0 rotate<0,degrees(Sarposo),0>}
    #else
      union{
        plane{z,0}
        plane{-z,0 rotate<0,degrees(Sarposo),0>}
      }
    #end
    #if(!Xose_SemTexturas) texture{Asfalto} #end
    #local Ktuv=vcross(y,VectAng);
    #if(vlength(Ktuv))
      RotAntDep(y,VectAng)
    #else
      #declare Ktuv=x;
      #if(VectAng.y<0)
        rotate<180,0,0>
      #end
    #end
    #local DeAqui=vcross(VectAng,vaxis_rotate(x,Ktuv,degrees(acos(vnormalize(VectAng).y))));
    #local ParaAlo=vcross(VectAng,Origem-Centro);
    #if(vlength(vcross(DeAqui,ParaAlo)))
      RotAntDep(DeAqui,ParaAlo)
    #else
      #if(vdot(DeAqui,ParaAlo)<0)
        RotVect(pi*vnormalize(VectAng))
      #end
    #end
    translate Centro
  }
#end

//Esta balaustrada consume umha selvajada de recursos do computador.
/*#macro Balaustrada (Comeco, Final, Altura, SeparacoDosBalaustres, DiametroDosBalaustres, TexturaDosBalaustres,
  AltoDoPasamaos, LargoDoPasamaos, TexturaDoPasamaos)
  #local CaraAli=vnormalize(Final-Comeco);
  #local Comprimento=vlength(Final-Comeco-((Final-Comeco).y)*y);
  #local NumeroDeBalaustres=floor(Comprimento/SeparacoDosBalaustres);
  #local ContaBalaustres=0;
  #local ComecoReal=Comeco+(Comprimento/NumeroDeBalaustres-div(Comprimento,NumeroDeBalaustres))/2*CaraAli;
  #local Altt=Altura-AltoDoPasamaos/2;
  union{
    #while (ContaBalaustres<NumeroDeBalaustres)
      cylinder{0,Altt*y, DiametroDosBalaustres/2
        #if(!Xose_SemTexturas) texture{TexturaDosBalaustres} #end
        translate ComecoReal+SeparacoDosBalaustres*ContaBalaustres*CaraAli
      }
      #declare ContaBalaustres=ContaBalaustres+1;
    #end
    cylinder{0,(Final-Comeco)/<LargoDoPasamaos,AltoDoPasamaos,LargoDoPasamaos>,1
      scale<LargoDoPasamaos,AltoDoPasamaos,LargoDoPasamaos>
      #if(!Xose_SemTexturas) texture{TexturaDoPasamaos} #end
      translate Comeco+Altt*y
    }
  }
#end*/

#declare BalaustradaBarateira=union{      //Balaustrada algo mais modesta.
  box{<-2200,-.01,-.001>,<0,1.2,.001>
    #if(!Xose_SemTexturas)
      pigment{gradient x colour_map{
        [.2 rgb<.05,.5,1>]
        [.2 rgbf 1]
        }
        scale .2
      }
    #end
  }
  cylinder{-2200*x,0,.10 scale<1,.5,1>
    translate 1.2*y
    #if(!Xose_SemTexturas)
      pigment{rgb <1,.6,0>}
      finish{brilliance 3 specular .6 roughness .001 metallic}
    #end
  }
}

//Esta barreira tambm  mui cara, ao menos para cobrir grandes comprimentos.
/*#declare Barreira=union{
  superellipsoid{
    <.1,.5> scale <.2,4.75,.075> rotate<0,0,-90> translate<0,0.5,0>
  }
  cylinder{<3,.5,.03>,<3,-.01,.03> ,.03}
  cylinder{<3,.5,-.03>,<3,-.01,-.03> ,.03}
  cylinder{<-3,.5,.03>,<-3,-.01,.03> ,.03}
  cylinder{<-3,.5,-.03>,<-3,-.01,-.03> ,.03}
  cylinder{<.5,.5,.03>,<.5,-.01,.03> ,.03}
  cylinder{<-.5,.5,-.03>,<-.5,-.01,-.03> ,.03}
  cylinder{<.5,.5,-.03>,<.5,-.01,-.03> ,.03}
  cylinder{<-.5,.5,.03>,<-.5,-.01,.03> ,.03}
  cylinder{<1.75,.5,.03>,<1.75,-.01,.03> ,.03}
  cylinder{<1.75,.5,-.03>,<1.75,-.01,-.03> ,.03}
  cylinder{<-1.75,.5,.03>,<-1.75,-.01,.03> ,.03}
  cylinder{<-1.75,.5,-.03>,<-1.75,-.01,-.03> ,.03}
  cylinder{<4.25,.5,.03>,<4.25,-.01,.03> ,.03}
  cylinder{<4.25,.5,-.03>,<4.25,-.01,-.03> ,.03}
  cylinder{<-4.25,.5,.03>,<-4.25,-.01,.03> ,.03}
  cylinder{<-4.25,.5,-.03>,<-4.25,-.01,-.03> ,.03}
  #if(!Xose_SemTexturas)
    texture{
      pigment{rgb <.61,.6,.62>}
      normal{dents scale .1}
      finish{diffuse .7 brilliance 3}
    }
  #end
}*/

#macro Barreiro(Comprimento,Altura,Fondura,Grosso)    //Bastante mais econmico que a barreira anterior.
  box{<0,-Fondura,-Grosso/2>,<Comprimento,Altura,Grosso/2>}
#end

// Farola
#macro Farola(Cor)
  union{
    cone{.4*y,.2,0,.24}
    cylinder{.4*y,8*y,.2}
    torus{2,.2 rotate<90,0,0>
      clipped_by{plane{x,0 rotate<0,0,5>}}
      clipped_by{plane{-y,0}}
      translate<2,8,0>
    }
    union{
      cylinder{<-2,0,0>,<-2,1,0>,.2} 
      union{
        cone{<-.1,0,0>,.2,<.3,0,0>,.3 open scale<2,4,3.2>  translate <-2,1.95,0>}
        cylinder{<-.11,0,0>,<-.09,0,0>,.2 scale<2,4,3.2> translate<-2,1.95,0>}
      }
      intersection{
        cone{<-.1,0,0>,.2,<.3,0,0>,.3 scale.999*<1,1,.8> translate <-2,1.15,0>}
        sphere{<-9.7,0,0>,9.98 translate<-2,1.15,0>}
        #if(!Xose_SemTexturas) texture{pigment{rgbf 1} finish{diffuse 1}} #end
      }
      #if (abs(Cor.red)+abs(Cor.green)+abs(Cor.blue))
        light_source{<-1.9,1.15,0> color Cor fade_distance 16 fade_power 2 media_attenuation on}
      #end
      rotate<0,0,-85>  translate<2,8,0>
    }
  }
#end


#version versionUrb;